/*
 * Copyright 2004-2006 National Institute of Informatics in Japan, 
 * All rights reserved.
 *
 * This file or a portion of this file is licensed under the terms of
 * the NAREGI Public License, found at http://www.naregi.org/download/index.html.
 * If you redistribute this file, with or without modifications, you must 
 * include this notice in the file.
 *
 * [History]
 * Date         Name                  Contents
 * ------------------------------------------------------------------------
 * 2006/06/01   Fujitsu)Yositake      First Version 
 */

package org.naregi.pse.acs.client;

import java.io.ByteArrayInputStream;
import java.io.File;

import javax.xml.namespace.QName;

import org.apache.axis.message.MessageElement;
import org.apache.axis.message.Text;
import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.apache.axis.types.URI;
import org.apache.log4j.Logger;

import org.naregi.pse.acs.ari.stubs.AAType;
import org.naregi.pse.acs.ari.stubs.AATypeBundle;
import org.naregi.pse.acs.ari.stubs.AATypeContent;
import org.naregi.pse.acs.ari.stubs.AATypeDescriptor;
import org.naregi.pse.acs.ari.stubs.ApplicationArchive;
import org.naregi.pse.acs.ari.stubs.ApplicationRepository;
import org.naregi.pse.acs.ari.stubs.Create;
import org.naregi.pse.acs.ari.stubs.CreateResponse;
import org.naregi.pse.acs.ari.stubs.GetArchive;
import org.naregi.pse.acs.ari.stubs.GetArchiveResponse;
import org.naregi.pse.acs.ari.stubs.GetContents;
import org.naregi.pse.acs.ari.stubs.GetContentsResponse;
import org.naregi.pse.acs.ari.stubs.LookupArchives;
import org.naregi.pse.acs.ari.stubs.LookupArchivesResponse;
import org.naregi.pse.acs.ari.stubs.QueryExpressionType;
import org.naregi.pse.acs.ari.stubs.Update;
import org.naregi.pse.acs.ari.stubs.UpdateResponse;
import org.naregi.pse.acs.ari.stubs.service.ApplicationArchivePortTypesServiceAddressingLocator;
import org.naregi.pse.acs.ari.stubs.service.ApplicationRepositoryPortTypesServiceAddressingLocator;
import org.naregi.pse.acs.client.databean.*;
import org.naregi.pse.acs.client.filemanage.ZipFileManager;
import org.naregi.pse.ap.common.PSESystemInfo;

import org.oasis.wsrf.properties.GetMultipleResourcePropertiesResponse;
import org.oasis.wsrf.properties.GetMultipleResourceProperties_Element;
import org.oasis.wsrf.properties.GetResourcePropertyResponse;
import org.oasis_open.docs.wsrf.rl_2.Destroy;
import org.oasis_open.docs.wsrf.rl_2.DestroyResponse;

import org.globus.axis.util.Util;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.encoding.ObjectSerializer;
import org.globus.wsrf.impl.SimpleResourceKey;
import org.globus.wsrf.utils.AddressingUtils;
import org.xml.sax.InputSource;

/**
 * This class provides functionalities to access ACS. 
 * 
 */
public class ACSClient {

	private static Logger log = Logger.getLogger(ACSClient.class.getName());

	// key for reading from property file
	private static final String APPLICATION_REPOSITORY_SERVICE_URI_KEY =
		"APPLICATION_REPOSITORY_SERVICE_URI";

	public static final String ST_TRANS_TYPE_DESCRETE =
		"http://schemas.ggf.org/acs/2006/04/ari/transport-type/discrete";

	public static final String ST_TRANS_TYPE_BUNDLED_ZIP =
		"http://schemas.ggf.org/acs/2006/04/ari/transport-type/bundled/zip";

	public static final String ST_TRANS_METHOD_EMBEDDED =
		"http://schemas.ggf.org/acs/2006/04/ari/transport-method/embedded";

	public static final String ST_TRANS_METHOD_GRIDFTP =
		"http://schemas.ggf.org/acs/2006/04/ari/transport-method/gridftp";
		
	private static final String ST_QUERY_DIALECT_XPATH_1_0 = 
		"http://www.w3.org/TR/1999/REC-xpath-19991116";
		
	public static final String BUNDLED_FILENAME_ZIP = "AAFile.zip";

	public static final int TRANS_TYPE_DESCRETE = 0;
	public static final int TRANS_TYPE_BUNDLED_ZIP = 1;
	public static final int TRANS_METHOD_EMBEDDED = 0;
	public static final int TRANS_METHOD_GRIDFTP = 1;
	public static final int AATYPE_FULL = 0;
	public static final int AATYPE_DIFFERENTIAL = 1;
	public static final int QUERY_DIALECT_XPATH_1_0 = 0; 
	
	static {

		// get repository service URL
		String applicationRepositoryURI =
			PSESystemInfo.getInstance().getProperty(
				APPLICATION_REPOSITORY_SERVICE_URI_KEY);

		if(applicationRepositoryURI.startsWith("https")){
			
			// needed for https access to globus container
			Util.registerTransport();
		}
	}

	/**<PRE>
	 * LookupArchive
	 * @throws Exception</PRE>
	 */
	public void LookupArchives() throws Exception {

		ApplicationRepository applicationRepository = getRepositoryPortType();

		// lookup
		LookupArchivesResponse res =
			applicationRepository.lookupArchives(new LookupArchives());
	}

	/**<PRE>
	 * GetMultipleResourceProperties to Application Repository
	 * QName definition is in ACSQNames class. 
	 * @param resources
	 * @throws Exception</PRE>
	 */
	public void getMultipleRPForAR(QName[] resources) throws Exception {

		ApplicationRepository applicationRepository = getRepositoryPortType();

		// set request
		GetMultipleResourceProperties_Element request = new GetMultipleResourceProperties_Element();
		request.setResourceProperty(resources);
		
		// get resource property
		GetMultipleResourcePropertiesResponse res =
			applicationRepository.getMultipleResourceProperties(request);

		// output
		for (int i = 0; i < res.get_any().length; i++) {
			log.info("GetMultipleResourceProperty " + res.get_any()[i].toString());
		}
	}	

	/**<PRE>
	 * GetMultipleResourceProperties to Application Archive
	 * QName definition is in ACSQNames class.
	 * @param resources
	 * @param epr
	 * @throws Exception</PRE>
	 */
	public void getMultipleRPForAA(QName[] resources, byte[] epr) throws Exception {

		ApplicationArchive applicationArchive = getArchivePortType(epr);

		// set request
		GetMultipleResourceProperties_Element request = new GetMultipleResourceProperties_Element();
		request.setResourceProperty(resources);
		
		// get resource property
		GetMultipleResourcePropertiesResponse res =
			applicationArchive.getMultipleResourceProperties(request);

		// output
		for (int i = 0; i < res.get_any().length; i++) {
			log.info("GetMultipleResourceProperty " + res.get_any()[i].toString());
		}
	}


	/**<PRE>
	 * GetResourceProperty to Application Repository
	 * QName definition is in ACSQNames class.
	 * 
	 * @param resource
	 * @throws Exception</PRE>
	 */
	public void getRPForRepository(QName resource) throws Exception {

		ApplicationRepository applicationRepository = getRepositoryPortType();

		// get resource property
		GetResourcePropertyResponse res =
			applicationRepository.getResourceProperty(resource);

		// output
		for (int i = 0; i < res.get_any().length; i++) {
			log.info("GetResourceProperty " + res.get_any()[i].toString());
		}
	}

	/**<PRE>
	 * GetResourceProperty to Application Archive
	 * QName definition is in ACSQNames class.
	 * 
	 * @param epr
	 * @param resource
	 * @throws Exception</PRE>
	 */
	public void getRPForArchive(byte[] epr, QName resource) throws Exception {

		ApplicationArchive aaPort = getArchivePortType(epr);

		GetResourcePropertyResponse res = aaPort.getResourceProperty(resource);

		for (int i = 0; i < res.get_any().length; i++) {
			log.info("GetResourceProperty " + res.get_any()[i].toString());
		}
	}

	/**
	 * Create operation for Transport type - descrete
	 * @param baseDir
	 * @param descTransMethod
	 * @param descGsiFTPInfo
	 * @param fileInfo
	 * @throws Throwable
	 */
	public void createForDescrete(
		String baseDir,
		int descTransMethod,
		FTPInfo descGsiFTPInfo,
		AcsFileInfo[] fileInfo)
		throws Throwable {

		// check baseDir
		if(baseDir.endsWith("/") == false){
			baseDir = baseDir + "/";
		}

		// check descTransMethod
		if(descTransMethod != 0 && descTransMethod != 1){
			throw new Exception("Transport method for aad.xml not supported.");
		}

		// start Listener
		AcsListener lis = new AcsListener(getRepoEpr(), AcsListener.CREATED_TOPIC);
		lis.startListening();
		
		CreateResponse res = null;
		
		// if exception happens here, stop listener
		try{
			// create request
			Create request = new Create();
			request.setAA(getAATypeForDescrete(baseDir, descTransMethod, descGsiFTPInfo, fileInfo));
	
			// get port type
			ApplicationRepository applicationRepository = getRepositoryPortType();
	
			// call create
			res = applicationRepository.create(request);
		}catch(Throwable t){
			lis.stopListening();
			throw t;		
		}

		// save epr
		String strEpr = ObjectSerializer.toString(res.getArchiveEPR(),
												ACSQNames.AA_RESOURCE_REFERENCE);
		AADFileFactory fileFactory = new AADFileFactory();
		fileFactory.writeFileFrombyte(new File("epr.txt"), strEpr.getBytes());
	}

	/**
	 * Create operation for Transport type - bundled/zip
	 * @param baseDir
	 * @param bundledZipTransMethod
	 * @param bundledGsiFTPInfo
	 * @param fileInfo
	 * @throws Throwable
	 */
	public void createForBundledZip(
		String baseDir,
		int bundledZipTransMethod,
		FTPInfo bundledGsiFTPInfo,
		AcsFileInfo[] fileInfo)
		throws Throwable {

		// check baseDir
		if(baseDir.endsWith("/") == false){
			baseDir = baseDir + "/";
		}

		// check bundledZipTransMethod
		if(bundledZipTransMethod != 0 && bundledZipTransMethod != 1){
			throw new Exception("Transport method for aad.xml not supported.");
		}

		// start Listener
		AcsListener lis = new AcsListener(getRepoEpr(), AcsListener.CREATED_TOPIC);
		lis.startListening();

		CreateResponse res = null;
		
		// if exception happens here, stop listener
		try{
			// create Bundled Zip file
			ZipFileManager zip = new ZipFileManager();
			String zipPath = baseDir + BUNDLED_FILENAME_ZIP;
			
			// add aad.xml to zip entry here
			zip.zipWithDir(zipPath, 
						   acsFileInfo2StringAbsolute(baseDir, fileInfo),
						   acsFileInfo2StringRelative(fileInfo));
	
			// create request
			Create request = new Create();
			request.setAA(getAATypeForBundledZip(bundledZipTransMethod, bundledGsiFTPInfo, zipPath));
	
			// get port type
			ApplicationRepository applicationRepository = getRepositoryPortType();
	
			// call create
			res = applicationRepository.create(request);
		}catch(Throwable t){
			lis.stopListening();
			throw t;			
		}

		// save epr
		String strEpr = ObjectSerializer.toString(res.getArchiveEPR(),
												ACSQNames.AA_RESOURCE_REFERENCE);
		AADFileFactory fileFactory = new AADFileFactory();
		fileFactory.writeFileFrombyte(new File("epr.txt"), strEpr.getBytes());
	}	


	/**
	 * Update operation for Transport type - descrete
	 * 
	 * @param baseDir
	 * @param descTransMethod
	 * @param descGsiFTPInfo
	 * @param fileInfo
	 * @param epr
	 * @throws Throwable
	 */
	public void updateForDescrete(
		String baseDir,
		int descTransMethod,
		FTPInfo descGsiFTPInfo,
		AcsFileInfo[] fileInfo,
		byte[] epr)
		throws Throwable {

		// check baseDir
		if(baseDir.endsWith("/") == false){
			baseDir = baseDir + "/";
		}

		// check descTransMethod
		if(descTransMethod != 0 && descTransMethod != 1){
			throw new Exception("Transport method for aad.xml not supported.");
		}

		// start Listener
		AcsListener lis = new AcsListener(epr, AcsListener.UPDATED_TOPIC);
		lis.startListening();

		UpdateResponse res = null;
		
		// if exception happens here, stop listener 
		try{
			// create request
			Update request = new Update();
			request.setAA(getAATypeForDescrete(baseDir, descTransMethod, descGsiFTPInfo, fileInfo));
	
			// get port type
			ApplicationArchive applicationArchive = getArchivePortType(epr);
	
			// call update
			res = applicationArchive.update(request);
		}catch(Throwable t){
			lis.stopListening();
			throw t;
		}

		// save epr
		String strEpr = ObjectSerializer.toString(res.getArchiveEPR(),
												ACSQNames.AA_RESOURCE_REFERENCE);
		AADFileFactory fileFactory = new AADFileFactory();
		fileFactory.writeFileFrombyte(new File("epr.txt"), strEpr.getBytes());
	}

	/**
	 * Update operation for Transport type - bundeled/zip
	 * 
	 * @param baseDir
	 * @param bundledZipTransMethod
	 * @param bundledGsiFTPInfo
	 * @param fileInfo
	 * @param epr
	 * @throws Throwable
	 */
	public void updateForBundledZip(
		String baseDir,
		int bundledZipTransMethod,
		FTPInfo bundledGsiFTPInfo,
		AcsFileInfo[] fileInfo,
		byte[] epr)
		throws Throwable {

		// check baseDir
		if(baseDir.endsWith("/") == false){
			baseDir = baseDir + "/";
		}

		// check bundledZipTransMethod
		if(bundledZipTransMethod != 0 && bundledZipTransMethod != 1){
			throw new Exception("Transport method for Bundled/Zip not supported.");
		}

		// start Listener
		AcsListener lis = new AcsListener(epr, AcsListener.UPDATED_TOPIC);
		lis.startListening();

		UpdateResponse res = null;
		
		// if exception happens here, stop listener 
		try{		
		
			// create Bundled Zip file
			ZipFileManager zip = new ZipFileManager();
			String zipPath = baseDir + BUNDLED_FILENAME_ZIP;
			
			// add aad.xml to zip entry here
			zip.zipWithDir(zipPath, 
						   acsFileInfo2StringAbsolute(baseDir, fileInfo),
						   acsFileInfo2StringRelative(fileInfo));
	
			// create request
			Update request = new Update();
			request.setAA(getAATypeForBundledZip(bundledZipTransMethod, bundledGsiFTPInfo, zipPath));
	
			// get port type
			ApplicationArchive applicationArchive = getArchivePortType(epr);
	
			// call create
			res = applicationArchive.update(request);
		
		}catch(Throwable t){
			lis.stopListening();
			throw t;
		}

		// save epr
		String strEpr = ObjectSerializer.toString(res.getArchiveEPR(),
												ACSQNames.AA_RESOURCE_REFERENCE);
		AADFileFactory fileFactory = new AADFileFactory();
		fileFactory.writeFileFrombyte(new File("epr.txt"), strEpr.getBytes());
	}	

	private EndpointReferenceType getRepoEpr() throws Exception{
		
		// get repository service URL
		String applicationRepositoryURI = PSESystemInfo.getInstance().getProperty(
										APPLICATION_REPOSITORY_SERVICE_URI_KEY);

		// get epr
		EndpointReferenceType applicationRepositoryEPR =
			new EndpointReferenceType();
		applicationRepositoryEPR.setAddress(
			new Address(applicationRepositoryURI));
		ResourceKey resourceKey =
			new SimpleResourceKey(ACSQNames.AR_RESOURCE_KEY,
			new String("777985"));
		applicationRepositoryEPR =
			AddressingUtils.createEndpointReference(
				applicationRepositoryURI,
				resourceKey);
						
		return applicationRepositoryEPR;
	}

	private String[] acsFileInfo2StringAbsolute(String baseDir, AcsFileInfo[] _fileInfo){
		
		// add content files
		String[] fileList = new String[_fileInfo.length + 1];
		for(int i = 0; i < _fileInfo.length; i++){
			fileList[i] = baseDir + _fileInfo[i].getRelativeFilePath();
		}
		
		// add aad.xml
		fileList[_fileInfo.length] = baseDir + AADFileFactory.AAD_FILENAME;
		return fileList;
	}

	private String[] acsFileInfo2StringRelative(AcsFileInfo[] _fileInfo){
		
		// add content files
		String[] fileList = new String[_fileInfo.length + 1];
		for(int i = 0; i < _fileInfo.length; i++){
			fileList[i] = _fileInfo[i].getRelativeFilePath();
		}
		
		// add aad.xml
		fileList[_fileInfo.length] = AADFileFactory.AAD_FILENAME;
		return fileList;
	}

	private ApplicationRepository getRepositoryPortType() throws Exception{
		
		ApplicationRepositoryPortTypesServiceAddressingLocator locatorAR =
			new ApplicationRepositoryPortTypesServiceAddressingLocator();
									
		// get repository portType
		ApplicationRepository applicationRepository =
			(ApplicationRepository) locatorAR.getApplicationRepositoryPort(
				getRepoEpr());		
		
		return applicationRepository;
	}

	private ApplicationArchive getArchivePortType(byte[] epr) throws Exception{
		
		ApplicationArchivePortTypesServiceAddressingLocator locatorAA =
			new ApplicationArchivePortTypesServiceAddressingLocator();

		ByteArrayInputStream bis = new ByteArrayInputStream(epr);

		EndpointReferenceType eprAA = null;
		eprAA =
			(EndpointReferenceType) ObjectDeserializer.deserialize(
				new InputSource(bis),
				EndpointReferenceType.class);

		ApplicationArchive aaPort = locatorAA.getApplicationArchivePort(eprAA);		
		
		return aaPort;
	}

	private AAType getAATypeForDescrete(
		String baseDir,
		int descTransMethod,
		FTPInfo descGsiFTPInfo,
		AcsFileInfo[] fileInfo)
		throws Exception {

		// create object
		AADFileFactory aaFileFactory = new AADFileFactory();
		AAType aaType = new AAType();

		aaType.setTransportType(new URI(ST_TRANS_TYPE_DESCRETE));

		// set descriptor
		AATypeDescriptor desc = new AATypeDescriptor();
		
		// embedded
		if (descTransMethod == TRANS_METHOD_EMBEDDED) {
			desc.setTransportMethod(new URI(ST_TRANS_METHOD_EMBEDDED));
			desc.setEmbedded(aaFileFactory.readFileAsbyte(baseDir + AADFileFactory.AAD_FILENAME));
		
		// gsiftp
		} else if (descTransMethod == TRANS_METHOD_GRIDFTP) {
			desc.setTransportMethod(new URI(ST_TRANS_METHOD_GRIDFTP));
			MessageElement[] any = new MessageElement[1];
			any[0] = new MessageElement(ACSQNames.URL);
			any[0].addTextNode(((GsiFTPInfo)descGsiFTPInfo).getGsiFtpURL());
			desc.set_any(any);
		}
		aaType.setDescriptor(desc);

		// set content
		AATypeContent[] contents = new AATypeContent[fileInfo.length];
		
		for(int i = 0; i < contents.length; i++){
			AATypeContent content = new AATypeContent();
			
			// for Embedded
			if (fileInfo[i].getTransMethod() == TRANS_METHOD_EMBEDDED) {
				content.setTransportMethod(new URI(ST_TRANS_METHOD_EMBEDDED));
				content.setEmbedded(aaFileFactory.readFileAsbyte(baseDir + fileInfo[i].getRelativeFilePath()));
				content.setPathname(fileInfo[i].getRelativeFilePath());
			
			// for Gridftp
			}else if (fileInfo[i].getTransMethod() == TRANS_METHOD_GRIDFTP) {
				content.setTransportMethod(new URI(ST_TRANS_METHOD_GRIDFTP));
				MessageElement[] any = new MessageElement[1];
				any[0] = new MessageElement(ACSQNames.URL);
				any[0].addTextNode(((GsiFTPInfo)fileInfo[i].getGsiFTPInfo()).getGsiFtpURL());
				content.set_any(any);
				content.setPathname(fileInfo[i].getRelativeFilePath());
			
			// invalid TranportMethod
			}else{
				throw new Exception("TransportMethod for " 
										+ fileInfo[i].getRelativeFilePath() 
										+ " not supported.");	
			}
			// set content 
			contents[i] = content;
		
		}// end of for loop
		
		// set contents
		aaType.setContent(contents); 
		return aaType;
	}

	private AAType getAATypeForBundledZip(
		int bundledZipTransMethod,
		FTPInfo bundledGsiFTPInfo,
		String absoluteZipPath)
		throws Exception {

		// create object
		AADFileFactory aaFileFactory = new AADFileFactory();
		AAType aaType = new AAType();

		// set Transport Type
		aaType.setTransportType(new URI(ST_TRANS_TYPE_BUNDLED_ZIP));

		// set AATypeBundle
		AATypeBundle bundle = new AATypeBundle();
		if (bundledZipTransMethod == TRANS_METHOD_EMBEDDED) {
			bundle.setTransportMethod(new URI(ST_TRANS_METHOD_EMBEDDED));
			bundle.setEmbedded(aaFileFactory.readFileAsbyte(absoluteZipPath));

		} else if (bundledZipTransMethod == TRANS_METHOD_GRIDFTP) {
			bundle.setTransportMethod(new URI(ST_TRANS_METHOD_GRIDFTP));
			MessageElement[] any = new MessageElement[1];
			any[0] = new MessageElement(ACSQNames.URL);
			any[0].addTextNode(((GsiFTPInfo)bundledGsiFTPInfo).getGsiFtpURL());
			bundle.set_any(any);
			
		}
		aaType.setBundle(bundle);
		return aaType;
	}

	/**<PRE>
	 * GetArchive
	 * 
	 * @param transType - see ACSClient static constants
	 * @param transMethod - see ACSClient static constants
	 * @param diff - see ACSClient static constants
	 * @param epr
	 * @return GetArchiveResponse
	 * @throws Throwable</PRE>
	 */
	public GetArchiveResponse getArchive(int transType, int transMethod, int diff, byte[] epr) throws Throwable {
		
		// create request
		GetArchive request = createGetArchiveRequest(transType, transMethod, diff);
		
		// get port type
		ApplicationArchive applicationArchive = getArchivePortType(epr);
	
		// Perform getArchive
		GetArchiveResponse res = applicationArchive.getArchive(request);
	
		ResponseFileGetter resGet = new ResponseFileGetter();
		resGet.getArchiveResponse(res);
		
		return res;
	}

	private GetArchive createGetArchiveRequest(int transType, int transMethod, int diff) throws Exception{
	
		GetArchive request = new GetArchive();
	
		// set differential
		if(diff == AATYPE_FULL){
			request.setDifferential(false);
		}else if(diff == AATYPE_DIFFERENTIAL){
			request.setDifferential(true);
		}else{
			throw new Exception("AA type must be 0 or 1");
		}
		
		// set transport type
		if(transType == TRANS_TYPE_DESCRETE){
			request.setTransportType(new URI(ST_TRANS_TYPE_DESCRETE));
		}else if(transType == TRANS_TYPE_BUNDLED_ZIP){
			request.setTransportType(new URI(ST_TRANS_TYPE_BUNDLED_ZIP));
		}else{
			throw new Exception("Transport type must be 0 or 1");
		}

		// set transport method
		if(transMethod == TRANS_METHOD_EMBEDDED){
			request.setTransportMethod(new URI(ST_TRANS_METHOD_EMBEDDED));
		}else if(transMethod == TRANS_METHOD_GRIDFTP){
			request.setTransportMethod(new URI(ST_TRANS_METHOD_GRIDFTP));
		}else{
			throw new Exception("Transport method must be 0 or 1");
		}		
		
		return request;
	}

	/**<PSE>
	 * GetContents
	 * 
	 * @param queryDialect - see ACSClient static constants
	 * @param query
	 * @param transMethod - see ACSClient static constants
	 * @param epr
	 * @return GetContentsResponse
	 * @throws Throwable</PSE>
	 */
	public GetContentsResponse getContents(int queryDialect, String query, int transMethod, byte[] epr) throws Throwable {
	
		// create request
		GetContents request = createGetContentsRequest(queryDialect, query, transMethod);
		
		// get port type
		ApplicationArchive applicationArchive = getArchivePortType(epr);
	
		// Perform getArchive
		GetContentsResponse res = applicationArchive.getContents(request);

		ResponseFileGetter resGet = new ResponseFileGetter();
		resGet.getContentsResponse(res);

		return res;	
	}
	
	private GetContents createGetContentsRequest(int queryDialect, String query, int transMethod) throws Exception{
		
		GetContents request = new GetContents();
		QueryExpressionType queryExpression = new QueryExpressionType();
		
		// set dialect
		if(queryDialect == QUERY_DIALECT_XPATH_1_0){
			queryExpression.setDialect(new URI(ST_QUERY_DIALECT_XPATH_1_0));
		}
			
		// set expression
		MessageElement[] any = new MessageElement[1];
		any[0] = new MessageElement(new Text(query));
		queryExpression.set_any(any);
		
		// set query expression
		request.setQueryExpression(queryExpression);
		
		// set transport method
		if(transMethod == TRANS_METHOD_EMBEDDED){
			request.setTransportMethod(new URI(ST_TRANS_METHOD_EMBEDDED));
		}else if(transMethod == TRANS_METHOD_GRIDFTP){
			request.setTransportMethod(new URI(ST_TRANS_METHOD_GRIDFTP));
		}else{
			throw new Exception("Transport method must be 0 or 1");
		}
		
		return request;
	}


	/**<PRE>
	 * Destroy to Application Repository
	 * @throws Exception</PRE>
	 */
	public void destroyAR() throws Exception {

		ApplicationRepository applicationRepository = getRepositoryPortType();

		// destroy resource
		org.oasis.wsrf.lifetime.DestroyResponse res = 
				applicationRepository.destroy(new org.oasis.wsrf.lifetime.Destroy());
	}

	/**<PRE>
	 * Destroy to Applciation Archive
	 * @param epr
	 * @throws Exception</PRE>
	 */
	public void destroyAA(byte[] epr) throws Exception {

		ApplicationArchive aaPort = getArchivePortType(epr);

		DestroyResponse res = aaPort.destroy(new Destroy());
	}


	/*
		private org.apache.axis.message.addressing.EndpointReferenceType changeEPR(
			org.w3.www._2005._03.addressing.EndpointReferenceType eprIN)
			throws Exception {

			// get address
			AttributedURIType address = eprIN.getAddress();

			// get key
			MessageElement any = eprIN.getReferenceParameters().get_any()[0];
			Document doc = any.getAsDocument();
			NodeList list =
				doc.getElementsByTagNameNS(
					"http://www.globus.org/namespaces/naregi/pse/acs/ApplicationArchive",
					"ApplicationArchiveResourceKey");
			Node key = list.item(0);
			String strKey = key.getFirstChild().getNodeValue();

			// create 
			org.apache.axis.message.addressing.EndpointReferenceType eprOUT =
				new EndpointReferenceType();
			ResourceKey resourceKey =
				new SimpleResourceKey(
					new QName(
						"http://www.globus.org/namespaces/naregi/pse/acs/ApplicationArchive",
						"ApplicationArchiveResourceKey"),
					new Integer(strKey));
			eprOUT =
				AddressingUtils.createEndpointReference(
					address.toString(),
					resourceKey);

			return eprOUT;
		}
	*/
}
