/*
 * Copyright 2004-2006 National Institute of Informatics in Japan, 
 * All rights reserved.
 *
 * This file or a portion of this file is licensed under the terms of
 * the NAREGI Public License, found at http://www.naregi.org/download/index.html.
 * If you redistribute this file, with or without modifications, you must 
 * include this notice in the file.
 *
 * [History]
 * Date         Name                  Contents
 * ------------------------------------------------------------------------
 * 2006/06/09   Fujitsu)Yositake      First Version 
 */

package org.naregi.pse.acs.client.filemanage;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import org.apache.log4j.Logger;


public class ZipFileManager extends FileManager{

	private static Logger log = Logger.getLogger(ZipFileManager.class.getName());

	/**
	 * unzip zip file(first parameter) to destination(second parameter).
	 * 
	 * @param String
	 *            fileName : as file name to be unzipped.
	 * @param String
	 *            destPath : as destination folder path that unzipped files are
	 *            created.
	 * @throws Exception
	 */
	public String[] getFileFromZip(String zipFilePath, String fileToGet,
			String extractedFileLocation) throws Exception {
		
		FileInputStream fis = null;
		BufferedInputStream bis = null;
		ZipInputStream zis = null;
		ZipEntry zent = null;
		FileOutputStream fos = null;

		String zipEntry = null;
		String fileName = null;
		ArrayList fileList = new ArrayList();

		try {
			fis = new FileInputStream(zipFilePath);
			bis = new BufferedInputStream(fis);
			zis = new ZipInputStream(bis);

			byte[] buf = new byte[1024];
			int len;

			// get file information to be included in the archive.
			int index = 0;
			while ((zent = zis.getNextEntry()) != null) {

				zipEntry = zent.getName();
				
				if (zipEntry != null && zipEntry.endsWith(fileToGet)) {

					// open file to be written into.
					fos = new FileOutputStream(extractedFileLocation);
					while (-1 != (len = zis.read(buf, 0, buf.length))) {
						fos.write(buf, 0, len);
					}

					// Close FileOutputStream.
					fos.close();
				}

				// get file list
				fileName = zipEntry.substring(zipEntry.indexOf("/") + 1);
				if (("".equals(fileName)) == false) {
					fileList.add(index, fileName);
					index++;
				}
			}

		} catch (IOException e) {
			throw e;
		} finally {
			try {
				if(zis != null){
					zis.close();
				}
				if(bis != null){
					bis.close();
				}
				if(fis != null){
					fis.close();
				}
			} catch (Exception e) {
				throw e;
			}
		}
		return (String[]) fileList.toArray(new String[0]);
	}

	/**
	 * do a zip.
	 * @param zipName
	 * @param fileList
	 * @throws IOException
	 */
	public void zip(String zipName, String[] fileList) throws IOException{
    
	 	// Create a buffer for reading the files
	 	byte[] buf = new byte[1024];
    
	 	try {
		 	// make sure zipName exists.
		 	String outFilename = zipName;
		 	
		 	File zip = new File(outFilename);
		 	File dir = zip.getParentFile();
		 	
		 	if(dir.exists() == false){
				boolean check = dir.mkdirs();
				if (check == false){
					//throw new ACSFault();
				}
		 	}
		 	
		 	if(zip.exists() == false){
		 		boolean check = zip.createNewFile();
		 		if (check == false){
		 			//throw new ACSFault();
		 		}
		 	}
		 	
		 	// get file name without extension for extracted base directory
		 	String fileName = zip.getName();
		 	fileName = fileName.substring(0,fileName.indexOf(".zip"));
		 	
		 	ZipOutputStream out = new ZipOutputStream(new FileOutputStream(outFilename));
    
		 	// Compress the files
		 	for (int i=0; i<fileList.length; i++) {
			 	
			 	log.info("fileList = " + i + " " + fileList[i]);
			 	
			 	FileInputStream in = new FileInputStream(fileList[i]);
    
			 	// Add ZIP entry to output stream.
			 	out.putNextEntry(new ZipEntry(fileName + "/" + new File(fileList[i]).getName()));
    
				// Transfer bytes from the file to the ZIP file
				int len;
				while ((len = in.read(buf)) > 0) {
					out.write(buf, 0, len);
				}
    
				// Complete the entry
				out.closeEntry();
				in.close();
			}
    
			// Complete the ZIP file
			out.close();
		} catch (IOException e) {
			throw e;
		}
	}

	/**
	 * Zip just below the archive with file directories.
	 * This is just for making binary.zip.
	 * @param zipName
	 * @param absoluteFileList
	 * @param relativeFileList
	 * @throws IOException
	 */
	public void zipWithDir(String zipName, String[] absoluteFileList, 
								String[] relativeFileList) throws Exception{
    
		// Create a buffer for reading the files
		byte[] buf = new byte[1024];
    
		try {
			// make sure zipName exists.
			String outFilename = zipName;
		 	
			File zip = new File(outFilename);
			File dir = zip.getParentFile();
		 	
			if(dir.exists() == false){
				boolean check = dir.mkdirs();
				if (check == false){
					throw new Exception();
				}
			}
		 	
			if(zip.exists() == false){
				boolean check = zip.createNewFile();
				if (check == false){
					throw new Exception();
				}
			}
		 			 	
			ZipOutputStream out = new ZipOutputStream(new FileOutputStream(outFilename));
    
			// Compress the files
			for (int i=0; i< absoluteFileList.length; i++) {
			 	
				log.info("fileList = " + i + " " + absoluteFileList[i]);
			 	
				FileInputStream in = new FileInputStream(absoluteFileList[i]);
    
				// Add ZIP entry to output stream.
				out.putNextEntry(new ZipEntry(relativeFileList[i]));
    
				// Transfer bytes from the file to the ZIP file
				int len;
				while ((len = in.read(buf)) > 0) {
					out.write(buf, 0, len);
				}
    
				// Complete the entry
				out.closeEntry();
				in.close();
			}
    
			// Complete the ZIP file
			out.close();
		} catch (IOException e) {
			throw e;
		}
	}


	/**
	 * do a unzip operation.
	 * @param tempLocation : 
	 * @param zipName :
	 * @return
	 * @throws Exception
	 */
	public String[] unZipWithDir(File dirForUnzipedFiles, File zipFile) throws Exception {

		log.info("dirForUnzipedFiles = " + dirForUnzipedFiles.getPath());
		log.info("zipFilePath = " + zipFile.getPath());
		
		FileInputStream fis = null;
		BufferedInputStream bis = null;
		ZipInputStream zis = null;
		ZipEntry zent = null;
		FileOutputStream fos = null;
		
		ArrayList fileList = new ArrayList();
		
		// make sure dir exists
		if (dirForUnzipedFiles.exists() == false){
			
			boolean check = dirForUnzipedFiles.mkdirs();
			if (check == false){
				throw new Exception("Failed to create dir " + dirForUnzipedFiles.getPath());
			}
		}
		
		try {
			fis = new FileInputStream(zipFile.getPath());
			bis = new BufferedInputStream(fis);
			zis = new ZipInputStream(bis);

			byte[] buf = new byte[1024];
			int len;

			while ((zent = zis.getNextEntry()) != null) {
				
				log.info("zipEntry = " + zent.getName());
									
				int i = zent.getName().lastIndexOf("/");
				String check = zent.getName().substring(i + 1);
				
				if("".equals(check) == false){

					// make sure parent dir exists
					String dir = zent.getName().substring(0, i + 1);
					File dirFile = new File(dirForUnzipedFiles.getPath() + "/" + dir);
					
					if(dirFile.exists() == false){
						if(dirFile.mkdirs() == false){
							throw new Exception("Failed to make dir " + dirFile.getPath());
						}
						log.info("dir created = " + dirFile.getPath());
					}
	
					//open file to be written into.
					fos = new FileOutputStream(dirForUnzipedFiles.getPath() + "/" + zent.getName());
						
					while (-1 != (len = zis.read(buf, 0, buf.length))) {
						fos.write(buf, 0, len);
					}
					
					// Close FileOutputStream.
					fos.close();
					fileList.add(zent.getName());
				}
			}

		} catch(IOException e) {
			throw e;
		} finally {
			try {
				if(zis != null){
					zis.close();
				}
				if(bis != null){
					bis.close();
				}
				if(fis != null){
					fis.close();
				}
			} catch(Exception e) {
				throw e;
			}
		}		
		return (String[]) fileList.toArray(new String[0]);
	}

	/**
	 * do an unzip operation.
	 * @param tempLocation
	 * @param zipName
	 * @param zipLocation
	 * @return
	 * @throws Exception
	 */
	public String[] unZip(String tempLocation, String zipName, String zipLocation) throws Exception {

		log.info("unZip START : tempLocation = " + tempLocation + " zipName = " 
					+ zipName + " zipLocation = " + zipLocation);
		
		FileInputStream fis = null;
		BufferedInputStream bis = null;
		ZipInputStream zis = null;
		ZipEntry zent = null;
		FileOutputStream fos = null;
		
		String zipEntry = null;
		String fileName = null;
		ArrayList fileList = new ArrayList();
		
		// make sure dir exists
		File unzipDir = new File(tempLocation + zipName);
		if (unzipDir.exists() == false){
			
			boolean check = unzipDir.mkdirs();
			if (check == false){
				throw new Exception();
			}
		}
		
		try {
			fis = new FileInputStream(zipLocation);
			bis = new BufferedInputStream(fis);
			zis = new ZipInputStream(bis);

			byte[] buf = new byte[1024];
			int len;

			int index = 0;
			while ((zent = zis.getNextEntry()) != null) {
				
				zipEntry = zent.getName();

				// get file list
				fileName = zipEntry.substring(zipEntry.indexOf("/") + 1);
				if (("".equals(fileName)) == false) {
									
					//open file to be written into.
					fos = new FileOutputStream(tempLocation + "/" + zipEntry);
				
					while (-1 != (len = zis.read(buf, 0, buf.length))) {
						fos.write(buf, 0, len);
					}
				
					// Close FileOutputStream.
					fos.close();
					
					fileList.add(index, fileName);
					index++;
				}
			}

		} catch(IOException e) {
			throw e;
		} finally {
			try {
				if(zis != null){
					zis.close();
				}
				if(bis != null){
					bis.close();
				}
				if(fis != null){
					fis.close();
				}
			} catch(Exception e) {
				throw e;
			}
		}
		log.info("unZip END");
		return (String[]) fileList.toArray(new String[0]);
	}
}
