/*
 * Copyright 2004-2006 National Institute of Informatics in Japan, 
 * All rights reserved.
 *
 * This file or a portion of this file is licensed under the terms of
 * the NAREGI Public License, found at http://www.naregi.org/download/index.html.
 * If you redistribute this file, with or without modifications, you must 
 * include this notice in the file.
 *
 * [History]
 * Date         Name                  Contents
 * ------------------------------------------------------------------------
 * 2006/06/19   Fujitsu)Yositake      First Version
 */

package org.naregi.pse.acs.client.filetransfer;

import org.apache.log4j.Logger;
import org.globus.ftp.GridFTPClient;
import org.gridforum.jgss.ExtendedGSSManager;
import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSException;
import org.ietf.jgss.GSSManager;

/**
 * the class provide method for access the gridftp server. 
 *
 */
public class CogJGlobusAccess {

	private static Logger log = Logger.getLogger(CogJGlobusAccess.class.getName());

	public GSSCredential getProxyCert() throws GSSException{
	
		log.info("getting grid proxy certification......");
       
		// use default credential
		GSSManager manager = ExtendedGSSManager.getInstance();
		GSSCredential cred = manager.createCredential(GSSCredential.INITIATE_AND_ACCEPT);
	   
		log.info("success get grid proxy certification");
		return cred;
	}

	/**
	 * connect the gridftp server.
	 * @param address
	 * @param port
	 * @param subject
	 * @param cred
	 * @return
	 * @throws Exception
	 */
	public GridFTPClient connect(String address, String port, String subject,
			GSSCredential cred) throws Exception {

		log.info("connecting GridFTP Server");
		log.info("Address = " + address);
		log.info("Port = " + port);

		GridFTPClient connection = null;
		try {
			connection = new GridFTPClient(address, Integer.parseInt(port));
	
			// does not set host subject
			//connection.setAuthorization(new IdentityAuthorization(subject));
	
			connection.authenticate(cred);
		} catch (Exception e) {
			throw e;
		} 

		log.info("success connect GridFTP Server");
				
		return connection;
	}
}
