/*
 * Copyright 2004-2006 National Institute of Informatics in Japan, 
 * All rights reserved.
 *
 * This file or a portion of this file is licensed under the terms of
 * the NAREGI Public License, found at http://www.naregi.org/download/index.html.
 * If you redistribute this file, with or without modifications, you must 
 * include this notice in the file.
 */

package org.naregi.pse.acs.client.test;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.util.ArrayList;

import org.naregi.pse.acs.client.AADFileFactory;
import org.naregi.pse.acs.client.databean.AcsAppInfo;
import org.naregi.pse.acs.client.databean.AcsFileInfo;
import org.naregi.pse.ap.common.PSESystemInfo;

public class TestAADFileFactory {

	private AcsAppInfo appInfo;
	private AcsFileInfo[] fileInfo;
	private BufferedReader bin;
	private String baseDir;

	public static void main(String[] args) {

		TestAADFileFactory test = new TestAADFileFactory();
		try {
			if (args[0].equalsIgnoreCase("PrepareDir")) {
				test.prepareDir();
			} else if (args[0].equalsIgnoreCase("Create")) {
				test.create();
			} else if (args[0].equalsIgnoreCase("Update")) {
				test.update();
			}
		} catch (Throwable e) {
			e.printStackTrace();
		}
	}

	public TestAADFileFactory(){

		this.baseDir =
			PSESystemInfo.getInstance().getProperty(
				AADFileFactory.ACS_CLIENT_TEMP_BASEDIR_KEY);
		if (this.baseDir.endsWith("/") == false) {
			this.baseDir =this.baseDir + "/";
		}
		this.baseDir = this.baseDir + "AAFileFactory/";		
	}

	private void prepareDir() throws Exception {

		File file = new File(this.baseDir);
		if (file.exists() == false) {

			if (file.mkdirs() == false) {
				throw new Exception("Failed to create directory " + this.baseDir + " \n" +
					"Check permittion for write.");
			}
		}
		System.out.println("Directory is ready : " + this.baseDir);
		System.out.println("This is base directory for content files.");
	}

	private void create() throws Throwable {

		// input info
		inputForCreate();

		// delegate AADFileFactory
		AADFileFactory aadFileFactory = new AADFileFactory();
		aadFileFactory.createAAD(
			this.appInfo,
			this.fileInfo,
			AADFileFactory.CREATE);
			
		System.out.println(AADFileFactory.AAD_FILENAME + " has created at " + this.baseDir);
	}

	private void update() throws Throwable {

		// input info
		inputForUpdate();

		// delegate AADFileFactory
		AADFileFactory aadFileFactory = new AADFileFactory();
		aadFileFactory.createAAD(
			this.appInfo,
			this.fileInfo,
			AADFileFactory.UPDATE);
		
		System.out.println(AADFileFactory.AAD_FILENAME + " has created at " + this.baseDir);
	}

	private void inputForCreate() throws Exception {

		ArrayList files = new ArrayList();
		
		try{
			this.appInfo = new AcsAppInfo();
			this.bin = new BufferedReader(new InputStreamReader(System.in));
			String x = null;
	
			// get name
			System.out.print("Application name (required) : ");
			x = this.bin.readLine();
			if(x.equals("") || x.equals(" ")){
				throw new Exception("Application name is required.");
			}else{
				this.appInfo.setAppName(x);
			}
			
	 		// get version
			System.out.print("Application version (required) : ");
			x = this.bin.readLine();
			if(x.equals("") || x.equals(" ")){
				throw new Exception("Application version is required.");
			}else{
				this.appInfo.setAppVersion(x);
			}
			
			// description
			System.out.print("Application description (optional) X for skip : ");
			x = this.bin.readLine();
			if(x.equalsIgnoreCase("X") == false && x.equals("") == false && x.equals(" ") == false){
				this.appInfo.setAppDesc(x);
			}				

			// author name
			System.out.print("Author name (required) : ");
			x = this.bin.readLine();
			if(x.equals("") || x.equals(" ")){
				throw new Exception("Author name is required.");
			}else{
				this.appInfo.setAuthorName(x);
			}
			
			// author description
			System.out.print("Author description (optional) X for skip : ");
			x = this.bin.readLine();
			if(x.equalsIgnoreCase("X") == false && x.equals("") == false && x.equals(" ") == false){
				this.appInfo.setAuthorDesc(x);
			}

			// author country
			System.out.print("Author country (optional) X for skip : ");
			x = this.bin.readLine();
			if(x.equalsIgnoreCase("X") == false && x.equals("") == false && x.equals(" ") == false){
				this.appInfo.setAuthorCountry(x);			
			}			
			
			// author address
			System.out.print("Author address (optional) X for skip : ");
			x = this.bin.readLine();
			if(x.equalsIgnoreCase("X") == false && x.equals("") == false && x.equals(" ") == false){
				this.appInfo.setAuthorAddress(x);		
			}
	
			// get file info
			System.out.println("File path must be relative to " + baseDir);
			boolean stop = false;
			while (stop == false){
				
				AcsFileInfo file = new AcsFileInfo();
				
				// file path
				System.out.print("File path (required): ");
				x = this.bin.readLine();
				if(x.equals("") || x.equals(" ")){
					System.out.println("File path is required. Input again.");
					continue;
				}else{
					file.setRelativeFilePath(x);
				}
				
				// type
				System.out.print("File type as key for getContents (optional) X for skip : ");
				x = this.bin.readLine();
				if(x.equalsIgnoreCase("X") == false && x.equals("") == false && x.equals(" ") == false){
					file.setTypeString(x);
				}				
	
				// add to ArrayList
				files.add(file);
				
				System.out.println("Next?(Yes/No)");
				String next = this.bin.readLine();
				
				if(next.startsWith("N") || next.startsWith("n")){
					stop = true;
				}
			}
		
		}finally{
			this.bin.close();
		}
		
		// copy to AcsFileInfo[]
		this.fileInfo = (AcsFileInfo[])files.toArray(new AcsFileInfo[0]);	
	}

	private void inputForUpdate() throws Exception {

		ArrayList files = new ArrayList();
		
		try{
			this.appInfo = new AcsAppInfo();
			this.bin = new BufferedReader(new InputStreamReader(System.in));
			String x = null;

			// get app info
			// name
			System.out.print("Application name (required) : ");
			x = this.bin.readLine();
			if(x.equals("") || x.equals(" ")){
				throw new Exception("Application name is required.");
			}else{
				this.appInfo.setAppName(x);
			}
			
	 		// new version
			System.out.print("Application new version (required) : ");
			x = this.bin.readLine();
			if(x.equals("") || x.equals(" ")){
				throw new Exception("Application new version is required.");
			}else{
				this.appInfo.setAppVersion(x);
			}

			// base version
			System.out.print("Application base version (required) : ");
			x = this.bin.readLine();
			if(x.equals("") || x.equals(" ")){
				throw new Exception("Application base version is required.");
			}else{
				this.appInfo.setAppBaseVersion(x);
			}
			
			// description
			System.out.print("Application description (optional) X for skip : ");
			x = this.bin.readLine();
			if(x.equalsIgnoreCase("X") == false && x.equals("") == false && x.equals(" ") == false){
				this.appInfo.setAppDesc(x);
			}
	
			System.out.print("Author name (optional) X for skip : ");
			x = this.bin.readLine();
			if(x.equalsIgnoreCase("X") == false && x.equals("") == false && x.equals(" ") == false){
				this.appInfo.setAuthorName(x);
				
				System.out.print("Author description (optional) X for skip : ");
				x = this.bin.readLine();
				if(x.equalsIgnoreCase("X") == false && x.equals("") == false && x.equals(" ") == false){
					this.appInfo.setAuthorDesc(x);
				}
	
				System.out.print("Author country (optional) X for skip : ");
				x = this.bin.readLine();
				if(x.equalsIgnoreCase("X") == false && x.equals("") == false && x.equals(" ") == false){
					this.appInfo.setAuthorCountry(x);
				}
	
				System.out.print("Author address (optional) X for skip : ");
				x = this.bin.readLine();
				if(x.equalsIgnoreCase("X") == false && x.equals("") == false && x.equals(" ") == false){
					this.appInfo.setAuthorAddress(x);
				}
				
			}
			
			// get file info
			System.out.println("Only differential info is supplied.");
			System.out.println("For add/replace, file path info must be relative to " + baseDir);
			System.out.println("For delete, file path info must be same as when it's registed.");
			
			boolean stop = false;
			while (stop == false){

				AcsFileInfo file = new AcsFileInfo();

				// operation
				System.out.print("File operation (add/delete/replace) (required): ");
				x = this.bin.readLine();
				if(x.startsWith("A") || x.startsWith("a")){
					file.setOperation(AcsFileInfo.ADD);
				}else if(x.startsWith("D") || x.startsWith("d")){
					file.setOperation(AcsFileInfo.DELETE);
				}else if(x.startsWith("R") || x.startsWith("r")){
					file.setOperation(AcsFileInfo.REPLACE);
				}else{
					System.out.println("File operation is required. Input file operation again.");
					continue;
				}

				// file path
				System.out.print("File path (required): ");
				x = this.bin.readLine();
				if(x.equals("") || x.equals(" ")){
					System.out.println("File path is required. Input again from file operation.");
					continue;
				}else{
					file.setRelativeFilePath(x);
				}
				
				// type
				if(file.getOperation() != AcsFileInfo.DELETE){

					System.out.print("File type as key for getContents (optional) X for skip : ");
					x = this.bin.readLine();
					if(x.equalsIgnoreCase("X") == false && x.equals("") == false && x.equals(" ") == false){
						file.setTypeString(x);
					}	
				}
				
				// add to ArrayList
				files.add(file);
				
				System.out.println("Next?(Yes/No)");
				String next = this.bin.readLine();
				
				if(next.startsWith("N") || next.startsWith("n")){
					stop = true;
				}
			}
		}finally{
			this.bin.close();
		}
		
		// copy to AcsFileInfo[]
		this.fileInfo = (AcsFileInfo[])files.toArray(new AcsFileInfo[0]);	
	}
}
