/*
 * Copyright 2004-2006 National Institute of Informatics in Japan, 
 * All rights reserved.
 *
 * This file or a portion of this file is licensed under the terms of
 * the NAREGI Public License, found at http://www.naregi.org/download/index.html.
 * If you redistribute this file, with or without modifications, you must 
 * include this notice in the file.
 */

package org.naregi.pse.acs.client.test;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Unmarshaller;
import javax.xml.namespace.QName;

import org.naregi.pse.acs.ari.stubs.GetArchiveResponse;
import org.naregi.pse.acs.ari.stubs.GetContentsResponse;
import org.naregi.pse.acs.client.AADFileFactory;
import org.naregi.pse.acs.client.ACSClient;
import org.naregi.pse.acs.client.ACSQNames;
import org.naregi.pse.acs.client.databean.AcsFileInfo;
import org.naregi.pse.acs.client.databean.GsiFTPInfo;
import org.naregi.pse.acs.client.jaxb.AAD;
import org.naregi.pse.acs.client.jaxb.ContentType;
import org.naregi.pse.acs.client.jaxb.ContentsType;
import org.naregi.pse.acs.client.jaxb.DiffContentType;
import org.naregi.pse.acs.client.jaxb.DiffContentsType;
import org.naregi.pse.acs.client.jaxb.DifferentialAAD;
import org.naregi.pse.ap.common.PSESystemInfo;

public class TestACSClient {

	public static void main(String[] args) {
		
		TestACSClient test = new TestACSClient();
		
		if(args[0].equalsIgnoreCase("Create")){
			test.testCreate();
		}else if(args[0].equalsIgnoreCase("LookupArchives")){
			test.testLookupArchives();			
		}else if(args[0].equalsIgnoreCase("DestroyAR")){
			test.testDestroyAR();
		}else if(args[0].equalsIgnoreCase("GetRepositoryRP")){
			test.getRPForRepository();
		}else if(args[0].equalsIgnoreCase("GetArchiveRP")){
			test.getRPForArchive();
		}else if(args[0].equalsIgnoreCase("Destroy")){
			test.testDestroyAA();
		}else if(args[0].equalsIgnoreCase("Update")){
			test.testUpdate();
		}else if(args[0].equalsIgnoreCase("GetArchive")){
			test.testGetArchive();
		}else if(args[0].equalsIgnoreCase("GetContents")){
			test.testGetContents();
		}else if(args[0].equalsIgnoreCase("GetMultipleRPForAR")){
			test.testGetMultipleRPForAR();
		}else if(args[0].equalsIgnoreCase("GetMultipleRPForAA")){
			test.testGetMultipleRPForAA();
		}
	}

	public void testGetMultipleRPForAR(){

		ACSClient acsClient = new ACSClient();
		try {
			QName[] q = new QName[6];
			q[0] = ACSQNames.AR_RP_VERSION;
			q[1] = ACSQNames.AR_RP_TRANSPORT_TYPE;
			q[2] = ACSQNames.AR_RP_TRANSPORT_METHOD;
			q[3] = ACSQNames.AR_RP_QUERY_EXPRESSION_DIALECT;
			q[4] = ACSQNames.AR_RP_CREATED_MESSAGE;
			q[5] = ACSQNames.AR_RP_FAULT_MESSAGE;
			acsClient.getMultipleRPForAR(q);
		} catch (Throwable e) {
			e.printStackTrace();
		}	
	}

	public void testGetMultipleRPForAA(){

		ACSClient acsClient = new ACSClient();
		AADFileFactory aaFileFactory = new AADFileFactory();
		String eprFile = "epr.txt";
		byte[] eprByte = null;
		try {
			QName[] q = new QName[10];
			q[0] = ACSQNames.AA_RP_STATE;
			q[1] = ACSQNames.AA_RP_AAD;
			q[2] = ACSQNames.AA_RP_DIFF_AAD;
			q[3] = ACSQNames.AA_RP_CREATION_DATETIME;
			q[4] = ACSQNames.AA_RP_BASEAA;
			q[5] = ACSQNames.AA_RP_NEWERAA;
			q[6] = ACSQNames.AA_RP_REPOSITORY;
			q[7] = ACSQNames.AA_RP_QUERY_EXPRESSION_DIALECT;
			q[8] = ACSQNames.AA_RP_UPDATED_MESSAGE;
			q[9] = ACSQNames.AA_RP_FAULT_MESSAGE;		
			eprByte = aaFileFactory.readFileAsbyte(eprFile);
			acsClient.getMultipleRPForAA(q, eprByte);
		} catch (Throwable e) {
			e.printStackTrace();
		}
	}

	public void testDestroyAR(){
		
		ACSClient acsClient = new ACSClient();
		try {
			acsClient.destroyAR();
			System.out.println("Application Repository resource successfully destroyed.");
		} catch (Throwable e) {
			e.printStackTrace();
		}	
	}

	public void testDestroyAA(){
		
		ACSClient acsClient = new ACSClient();
		AADFileFactory aaFileFactory = new AADFileFactory();
		String eprFile = "epr.txt";
		byte[] eprByte = null;
		try {
			eprByte = aaFileFactory.readFileAsbyte(eprFile);
			acsClient.destroyAA(eprByte);
			System.out.println("Application Archive resource successfully destroyed.");
		} catch (Throwable e) {
			e.printStackTrace();
		}
	}
	
	public void testLookupArchives(){
		
		ACSClient acsClient = new ACSClient();
		
		// get epr
		AADFileFactory aaFileFactory = new AADFileFactory();

		try {
			acsClient.LookupArchives();
		} catch (Throwable e) {
			e.printStackTrace();
		}
	}

	public void getRPForRepository(){
		
		ACSClient acsClient = new ACSClient();
		try {
			acsClient.getRPForRepository(ACSQNames.AR_RP_VERSION);
			acsClient.getRPForRepository(ACSQNames.AR_RP_TRANSPORT_TYPE);
			acsClient.getRPForRepository(ACSQNames.AR_RP_TRANSPORT_METHOD);
			acsClient.getRPForRepository(ACSQNames.AR_RP_QUERY_EXPRESSION_DIALECT);
			acsClient.getRPForRepository(ACSQNames.AR_RP_CREATED_MESSAGE);
			acsClient.getRPForRepository(ACSQNames.AA_RP_FAULT_MESSAGE);
		} catch (Throwable e) {
			e.printStackTrace();
		}
	}	

	public void getRPForArchive(){
		
		ACSClient acsClient = new ACSClient();

		// get epr
		AADFileFactory aaFileFactory = new AADFileFactory();

		String eprFile = "epr.txt";

		byte[] eprByte = null;
		try {
			eprByte = aaFileFactory.readFileAsbyte(eprFile);

			acsClient.getRPForArchive(eprByte, ACSQNames.AA_RP_STATE);
			acsClient.getRPForArchive(eprByte, ACSQNames.AA_RP_AAD);
			acsClient.getRPForArchive(eprByte, ACSQNames.AA_RP_DIFF_AAD);
			acsClient.getRPForArchive(eprByte, ACSQNames.AA_RP_CREATION_DATETIME);
			acsClient.getRPForArchive(eprByte, ACSQNames.AA_RP_BASEAA);
			acsClient.getRPForArchive(eprByte, ACSQNames.AA_RP_NEWERAA);
			acsClient.getRPForArchive(eprByte, ACSQNames.AA_RP_REPOSITORY);
			acsClient.getRPForArchive(eprByte, ACSQNames.AA_RP_QUERY_EXPRESSION_DIALECT);
			acsClient.getRPForArchive(eprByte, ACSQNames.AA_RP_UPDATED_MESSAGE);
			acsClient.getRPForArchive(eprByte, ACSQNames.AA_RP_FAULT_MESSAGE);

		} catch (Throwable e) {
			e.printStackTrace();
		}
	}

	public void testCreate(){
		
		try {
			BufferedReader bin
						= new BufferedReader(new InputStreamReader(System.in));			
			
			// get transport type
			int transType = -1;
			System.out.println("Transport type (required)");
			System.out.println("0 for Descrete");
			System.out.println("1 for Bundled/Zip");
			System.out.print("->");
			String strTransType = bin.readLine();
			if(strTransType.equalsIgnoreCase("0") == false && 
					strTransType.equalsIgnoreCase("1") == false){
				throw new Exception("Transport type must be 0 or 1");				
			}
			transType = Integer.parseInt(strTransType);
			
			// idenfity aad
			String descriptor = null;
			String baseDir = 
				PSESystemInfo.getInstance().getProperty(AADFileFactory.ACS_CLIENT_TEMP_BASEDIR_KEY);
			if (baseDir.endsWith("/") == false) {
				baseDir = baseDir + "/";
			}
			
			baseDir = baseDir + "AAFileFactory" + "/";
			descriptor = baseDir + AADFileFactory.AAD_FILENAME;

			// check exist
			File desFile = new File(descriptor);
			if(desFile.exists() == false){
				throw new Exception(descriptor + " can't find.");
			}

			// read AAD
			JAXBContext context = JAXBContext.newInstance("org.naregi.pse.acs.client.jaxb");
			Unmarshaller unmarshaller = context.createUnmarshaller();
			AAD aad = null;
			try{
				aad = (AAD) unmarshaller.unmarshal(desFile);
			}catch(ClassCastException e){
				throw new Exception("Check " + descriptor + " is not DifferentialAAD.");
			}
			
			// get contents from AAD
			ContentsType contents = aad.getContents();
			List list = contents.getContent();
			AcsFileInfo[] fileInfos = new AcsFileInfo[list.size()];
			
			int descTransMethod = -1;
			int bundledZipTransMethod = -1;
			
			GsiFTPInfo descGsiFTPInfo = null;
			GsiFTPInfo bundledGsiFTPInfo = null;
			
			// for Descrete
			if(transType == ACSClient.TRANS_TYPE_DESCRETE){
			
				// get Transport method for descriptor
				System.out.println("Transport method (required) for " + AADFileFactory.AAD_FILENAME);
				System.out.println("0 for Embedded");
				System.out.println("1 for GridFTP");
				System.out.print("->");
				String strTransMethod = bin.readLine();
					
				if(strTransMethod.equalsIgnoreCase("0")){
					descTransMethod = 0;
				}else if(strTransMethod.equalsIgnoreCase("1")){
					descTransMethod = 1;
					
					// get gsiFTPInfo
					System.out.print("GridFTP server address ->");
					String strDescAddress = bin.readLine();
					System.out.print("GridFTP server port ->");
					String strDescPort = bin.readLine();
					descGsiFTPInfo = new GsiFTPInfo();
					descGsiFTPInfo.setGsiFtpURL
					("gsiftp://" + strDescAddress + ":" + strDescPort + descriptor);
					
				}else{
					throw new Exception("Transport method must be 0 or 1");
				}				
				
				for(int i = 0; i < list.size(); i++){
					
					ContentType content = (ContentType)list.get(i);
					AcsFileInfo fileInfo = new AcsFileInfo();
					fileInfos[i] = fileInfo;
					fileInfos[i].setRelativeFilePath(content.getPathname());
					
					// get Transport method for contents
					System.out.println("Transport method (required) for " + fileInfos[i].getRelativeFilePath());
					System.out.println("0 for Embedded");
					System.out.println("1 for GridFTP");
					System.out.print("->");
					strTransMethod = bin.readLine();
					
					if(strTransMethod.equalsIgnoreCase("0")){
						fileInfos[i].setTransMethod(ACSClient.TRANS_METHOD_EMBEDDED);
					}else if(strTransMethod.equalsIgnoreCase("1")){
						
						// get gsiFTPInfo
						System.out.print("GridFTP server address ->");
						String strAddress = bin.readLine();
						System.out.print("GridFTP server port ->");
						String strPort = bin.readLine();
						GsiFTPInfo gsiFTPInfo = new GsiFTPInfo();
						gsiFTPInfo.setGsiFtpURL
						("gsiftp://" + strAddress + ":" + strPort +
											baseDir + fileInfos[i].getRelativeFilePath());
																
						fileInfos[i].setGsiFTPInfo(gsiFTPInfo);
						fileInfos[i].setTransMethod(ACSClient.TRANS_METHOD_GRIDFTP);
						
					}else{
						throw new Exception("Transport method must be 0 or 1");
					}
				}// end of for loop
			
			// for bundled zip
			}else if(transType == ACSClient.TRANS_TYPE_BUNDLED_ZIP){
				
				// get Transport method for descriptor
				System.out.println("Transport method (required) for Bundled/Zip");
				System.out.println("0 for Embedded");
				System.out.println("1 for GridFTP");
				System.out.print("->");
				String strTransMethod = bin.readLine();
					
				if(strTransMethod.equalsIgnoreCase("0")){
					bundledZipTransMethod = 0;
				}else if(strTransMethod.equalsIgnoreCase("1")){
					
					// get gsiFTPInfo
					System.out.print("GridFTP server address ->");
					String strBundledAddress = bin.readLine();
					System.out.print("GridFTP server port ->");
					String strBundledPort = bin.readLine();
					bundledGsiFTPInfo = new GsiFTPInfo();
					bundledGsiFTPInfo.setGsiFtpURL
					("gsiftp://" + strBundledAddress + ":" + strBundledPort +
											baseDir + ACSClient.BUNDLED_FILENAME_ZIP);

					bundledZipTransMethod = 1;
				}else{
					throw new Exception("Transport method must be 0 or 1");
				}

				for(int i = 0; i < list.size(); i++){
					
					ContentType content = (ContentType)list.get(i);
					AcsFileInfo fileInfo = new AcsFileInfo();
					fileInfos[i] = fileInfo;				
					fileInfos[i].setRelativeFilePath(content.getPathname());
					
				}// end of for loop
			}
			
			// call ACSClient
			ACSClient acsClient = new ACSClient();

			if(transType == ACSClient.TRANS_TYPE_DESCRETE){
				acsClient.createForDescrete(baseDir, descTransMethod, descGsiFTPInfo, fileInfos);
			}else{
				acsClient.createForBundledZip(baseDir, bundledZipTransMethod, bundledGsiFTPInfo, fileInfos);
			}
		} catch (Throwable e) {
			e.printStackTrace();
		}
	}

	public void testUpdate(){
		
		try {
			BufferedReader bin
						= new BufferedReader(new InputStreamReader(System.in));			
			
			// get trnasport type
			int transType = -1;
			System.out.println("Transport type (required)");
			System.out.println("0 for Descrete");
			System.out.println("1 for Bundled/Zip");
			System.out.print("->");
			String strTransType = bin.readLine();
			if(strTransType.equalsIgnoreCase("0") == false && 
					strTransType.equalsIgnoreCase("1") == false){
				throw new Exception("Transport type must be 0 or 1");				
			}
			transType = Integer.parseInt(strTransType);
			
			// idenfity aad
			String descriptor = null;
			String baseDir = 
				PSESystemInfo.getInstance().getProperty(AADFileFactory.ACS_CLIENT_TEMP_BASEDIR_KEY);
			if (baseDir.endsWith("/") == false) {
				baseDir = baseDir + "/";
			}
			
			baseDir = baseDir + "AAFileFactory" + "/";
			descriptor = baseDir + AADFileFactory.AAD_FILENAME;

			// check exist
			File desFile = new File(descriptor);
			if(desFile.exists() == false){
				throw new Exception(descriptor + " can't find.");
			}

			// read AAD
			JAXBContext context = JAXBContext.newInstance("org.naregi.pse.acs.client.jaxb");
			Unmarshaller unmarshaller = context.createUnmarshaller();
			DifferentialAAD aad = null;
			try{
				aad = (DifferentialAAD) unmarshaller.unmarshal(desFile);
			}catch(ClassCastException e){
				throw new Exception("Check " + descriptor + " is DifferentialAAD.");
			}

			// get contents from AAD
			DiffContentsType contents = aad.getContents();
			List list = contents.getContent();
			ArrayList fileInfos = new ArrayList();
			
			int descTransMethod = -1;
			int bundledZipTransMethod = -1;
			
			GsiFTPInfo descGsiFTPInfo = null;
			GsiFTPInfo bundledGsiFTPInfo = null;
			
			// for Descrete
			if(transType == ACSClient.TRANS_TYPE_DESCRETE){
			
				// get Transport method for descriptor
				System.out.println("Transport method (required) for " + AADFileFactory.AAD_FILENAME);
				System.out.println("0 for Embedded");
				System.out.println("1 for GridFTP");
				System.out.print("->");
				String strTransMethod = bin.readLine();
					
				if(strTransMethod.equalsIgnoreCase("0")){
					descTransMethod = 0;
				}else if(strTransMethod.equalsIgnoreCase("1")){
				
					// get gsiFTPInfo
					System.out.print("GridFTP server address ->");
					String strDescAddress = bin.readLine();
					System.out.print("GridFTP server port ->");
					String strDescPort = bin.readLine();
					descGsiFTPInfo = new GsiFTPInfo();
					descGsiFTPInfo.setGsiFtpURL
					("gsiftp://" + strDescAddress + ":" + strDescPort + descriptor);
					descTransMethod = 1;
				}else{
					throw new Exception("Transport method must be 0 or 1");
				}	
				
				for(int i = 0; i < list.size(); i++){
					
					DiffContentType content = (DiffContentType)list.get(i);
					
					// if it's delete, no files in request
					if(content.getOperation().equalsIgnoreCase("delete")){
						continue;
					}
					
					AcsFileInfo fileInfo = new AcsFileInfo();
					fileInfo.setRelativeFilePath(content.getPathname());
					
					// get Transport method for contents
					System.out.println("Transport method (required) for " + fileInfo.getRelativeFilePath());
					System.out.println("0 for Embedded");
					System.out.println("1 for GridFTP");
					System.out.print("->");
					strTransMethod = bin.readLine();
					
					if(strTransMethod.equalsIgnoreCase("0")){
						fileInfo.setTransMethod(ACSClient.TRANS_METHOD_EMBEDDED);
					}else if(strTransMethod.equalsIgnoreCase("1")){
						
						// get gsiFTPInfo
						System.out.print("GridFTP server address ->");
						String strAddress = bin.readLine();
						System.out.print("GridFTP server port ->");
						String strPort = bin.readLine();
						GsiFTPInfo gsiFTPInfo = new GsiFTPInfo();
						gsiFTPInfo.setGsiFtpURL
						("gsiftp://" + strAddress + ":" + strPort + baseDir + fileInfo.getRelativeFilePath());
										
						fileInfo.setGsiFTPInfo(gsiFTPInfo);
						
						fileInfo.setTransMethod(ACSClient.TRANS_METHOD_GRIDFTP);
					}else{
						throw new Exception("Transport method must be 0 or 1");
					}
					
					// Add to ArrayList
					fileInfos.add(fileInfo);
					
				}// end of for loop
			
			// for bundled zip
			}else if(transType == ACSClient.TRANS_TYPE_BUNDLED_ZIP){
				
				// get Transport method for bundled zip
				System.out.println("Transport method (required) for Bundled/Zip");
				System.out.println("0 for Embedded");
				System.out.println("1 for GridFTP");
				System.out.print("->");
				String strTransMethod = bin.readLine();
					
				if(strTransMethod.equalsIgnoreCase("0")){
					bundledZipTransMethod = 0;
				}else if(strTransMethod.equalsIgnoreCase("1")){
					
					// get gsiFTPInfo
					System.out.print("GridFTP server address ->");
					String strBundledAddress = bin.readLine();
					System.out.print("GridFTP server port ->");
					String strBundledPort = bin.readLine();
					bundledGsiFTPInfo = new GsiFTPInfo();
					bundledGsiFTPInfo.setGsiFtpURL
					("gsiftp://" + strBundledAddress + ":" + strBundledPort +
						baseDir + ACSClient.BUNDLED_FILENAME_ZIP);
					bundledZipTransMethod = 1;
				}else{
					throw new Exception("Transport method must be 0 or 1");
				}

				for(int i = 0; i < list.size(); i++){
					
					DiffContentType content = (DiffContentType)list.get(i);

					// if it's delete, no files in request
					if(content.getOperation().equalsIgnoreCase("delete")){
						continue;
					}					
					
					AcsFileInfo fileInfo = new AcsFileInfo();
					fileInfo.setRelativeFilePath(content.getPathname());
					
					// Add to ArrayList
					fileInfos.add(fileInfo);				
					
				}// end of for loop
			}
			
			// call ACSClient
			ACSClient acsClient = new ACSClient();

			// get epr
			AADFileFactory aaFileFactory = new AADFileFactory();
			String eprFile = "epr.txt";
			byte[] eprByte = null;
			eprByte = aaFileFactory.readFileAsbyte(eprFile);

			if(transType == ACSClient.TRANS_TYPE_DESCRETE){
				acsClient.updateForDescrete(baseDir, descTransMethod, descGsiFTPInfo, 
											(AcsFileInfo[])fileInfos.toArray(new AcsFileInfo[0]), 
											eprByte);
			}else{
				acsClient.updateForBundledZip(baseDir, bundledZipTransMethod, bundledGsiFTPInfo,
											(AcsFileInfo[])fileInfos.toArray(new AcsFileInfo[0]), 
											eprByte);
			}
		} catch (Throwable e) {
			e.printStackTrace();
		}
	}

	public void testGetArchive(){

		try{			
			// get input
			BufferedReader bin
						= new BufferedReader(new InputStreamReader(System.in));			
	
			// get differential
			int diff = -1;
			System.out.println("AA type (default Full)");
			System.out.println("0 for Full AA");
			System.out.println("1 for Differential AA");
			System.out.print("->");
			String strAAType = bin.readLine();
			if(strAAType.equalsIgnoreCase("1") == false){
				diff = 0;			
			}else{
				diff = 1;
			}
				
			// get transport type
			int transType = -1;
			System.out.println("Transport type (required)");
			System.out.println("0 for Descrete");
			System.out.println("1 for Bundled/Zip");
			System.out.print("->");
			String strTransType = bin.readLine();
			if(strTransType.equalsIgnoreCase("0") == false && 
					strTransType.equalsIgnoreCase("1") == false){
				throw new Exception("Transport type must be 0 or 1");				
			}
			transType = Integer.parseInt(strTransType);
	
			// get transport method
			int transMethod = -1;
			System.out.println("Transport method (required)");
			System.out.println("0 for Embedded");
			System.out.println("1 for GridFTP");
			System.out.print("->");
			String strTransMethod = bin.readLine();
	
			if(strTransMethod.equalsIgnoreCase("0") == false && 
					strTransMethod.equalsIgnoreCase("1") == false){
				throw new Exception("Transport method must be 0 or 1");				
			}
			transMethod = Integer.parseInt(strTransMethod);
					
			// call ACSClient
			ACSClient acsClient = new ACSClient();
	
			// get epr
			AADFileFactory aaFileFactory = new AADFileFactory();
			String eprFile = "epr.txt";
			byte[] eprByte = null;
			eprByte = aaFileFactory.readFileAsbyte(eprFile);
	
			// delegate ACSClient
			GetArchiveResponse res = 
					acsClient.getArchive(transType, transMethod, diff, eprByte);
		}catch (Throwable e){
			e.printStackTrace();
		}
	}


	public void testGetContents(){
		
		try{			
			// get input
			BufferedReader bin
						= new BufferedReader(new InputStreamReader(System.in));			
	
			// get query dialect
			int queryDialect = -1;
			System.out.println("Query expression dialect");
			System.out.println("Currently, only XPath 1.0 is supported.");
			queryDialect = ACSClient.QUERY_DIALECT_XPATH_1_0;
				
			// get query expression
			System.out.println("Query expression (required)");
			System.out.println("Query result must be a set of Content element.");
			System.out.println("Namespace prefix needed and must be same as in ACS specification.");
			System.out.println("ex1) /aaf:AAD/aaf:Contents/aaf:Content");
			System.out.println("ex1) /aaf:AAD/aaf:Contents/aaf:Content[position()=1]");
			System.out.println("ex2) /aaf:AAD/aaf:Contents/aaf:Content[@type='nrg:YOUR_KEY']");
			System.out.println("ex3) /aaf:AAD/aaf:Contents/aaf:Content[aaf:Pathname='YOUR_FILE']");
			System.out.print("->");
			String queryExpression = bin.readLine();
			if(queryExpression.equals("") || queryExpression.equals(" ")){
				throw new Exception("Query expression is required.");
			}
	
			// get transport method
			int transMethod = -1;
			System.out.println("Transport method (required)");
			System.out.println("0 for Embedded");
			System.out.println("1 for GridFTP");
			System.out.print("->");
			String strTransMethod = bin.readLine();
	
			if(strTransMethod.equalsIgnoreCase("0") == false && 
					strTransMethod.equalsIgnoreCase("1") == false){
				throw new Exception("Transport method must be 0 or 1");				
			}
			transMethod = Integer.parseInt(strTransMethod);
					
			// call ACSClient
			ACSClient acsClient = new ACSClient();
	
			// get epr
			AADFileFactory aaFileFactory = new AADFileFactory();
			String eprFile = "epr.txt";
			byte[] eprByte = null;
			eprByte = aaFileFactory.readFileAsbyte(eprFile);
	
			// delegate ACSClient
			GetContentsResponse res = 
					acsClient.getContents(queryDialect, queryExpression, transMethod, eprByte);
		}catch (Throwable e){
			e.printStackTrace();
		}
	}
}
