/*
 * Copyright 2004-2006 National Institute of Informatics in Japan, 
 * All rights reserved.
 *
 * This file or a portion of this file is licensed under the terms of
 * the NAREGI Public License, found at http://www.naregi.org/download/index.html.
 * If you redistribute this file, with or without modifications, you must 
 * include this notice in the file.
 *
 * [History]
 * Date         Name                  Contents
 * ------------------------------------------------------------------------
 * 2006/06/07   Fujitsu)Yositake      First Version 
 */

package org.naregi.pse.acs.service.impl;

import java.net.URL;

import org.apache.axis.MessageContext;
import org.apache.axis.handlers.soap.SOAPService;

//import org.w3.www._2005._03.addressing.EndpointReferenceType;
import org.apache.axis.message.addressing.EndpointReferenceType;

import org.apache.log4j.Logger;
import org.globus.wsrf.NoSuchResourceException;
import org.globus.wsrf.ResourceContext;
import org.globus.wsrf.ResourceContextException;
import org.globus.wsrf.ResourceException;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.container.ServiceHost;

import org.naregi.pse.acs.ari.stubs.AAInstanceStateType;
import org.naregi.pse.acs.ari.stubs.BaseVersionNotMatchFaultType;
import org.naregi.pse.acs.ari.stubs.GetArchive;
import org.naregi.pse.acs.ari.stubs.GetArchiveFailedFaultType;
import org.naregi.pse.acs.ari.stubs.GetArchiveResponse;
import org.naregi.pse.acs.ari.stubs.GetContents;
import org.naregi.pse.acs.ari.stubs.GetContentsResponse;
import org.naregi.pse.acs.ari.stubs.IllegalDescriptorFaultType;
import org.naregi.pse.acs.ari.stubs.ResourceNotReadyFaultType;
import org.naregi.pse.acs.ari.stubs.ResourceUnavailableFaultType;
import org.naregi.pse.acs.ari.stubs.TransportMethodNotSupportedFaultType;
import org.naregi.pse.acs.ari.stubs.TransportTypeNotSupportedFaultType;
import org.naregi.pse.acs.ari.stubs.Update;
import org.naregi.pse.acs.ari.stubs.UpdateFailedFaultType;
import org.naregi.pse.acs.ari.stubs.UpdateResponse;

//import org.naregi.pse.acs.service.impl.functionbean.AddressingUtils;
import org.globus.wsrf.utils.AddressingUtils;

import org.naregi.pse.acs.service.impl.functionbean.FaultHelper;
//import org.globus.wsrf.utils.FaultHelper;

import org.oasis_open.docs.wsrf.r_2.ResourceUnknownFaultType;
import org.oasis_open.docs.wsrf.rl_2.Destroy;
import org.oasis_open.docs.wsrf.rl_2.DestroyResponse;
import org.oasis_open.docs.wsrf.rl_2.ResourceNotDestroyedFaultType;
import org.oasis_open.docs.wsrf.rp_2.InvalidQueryExpressionFaultType;
import org.oasis_open.docs.wsrf.rp_2.UnknownQueryExpressionDialectFaultType;

import org.naregi.pse.acs.service.impl.exception.InvalidQueryExpressionException;
import org.naregi.pse.acs.service.impl.exception.ResourceNotReadyException;
import org.naregi.pse.acs.service.impl.exception.ResourceUnavailableException;
import org.naregi.pse.acs.service.impl.exception.TransportMethodNotSupportedException;
import org.naregi.pse.acs.service.impl.exception.TransportTypeNotSupportedException;
import org.naregi.pse.acs.service.impl.exception.UnknownQueryDialectException;

/**
 * the class implements port which defined in wsdl.
 *
 */
public class ApplicationArchive {

	private static Logger log =
		Logger.getLogger(ApplicationArchive.class.getName());

	/**
	 * Implementation of Update operation.
	 * 
	 * @param request
	 * @return UpdateResponse
	 * @throws ResourceUnknownFaultType
	 * @throws ResourceNotReadyFaultType
	 * @throws ResourceUnavailableFaultType
	 * @throws UpdateFailedFaultType - by notificaiton too
	 * @throws IllegalDescriptorFaultType - by notificaiton
	 * @throws TransportTypeNotSupportedFaultType - by notificaiton
	 * @throws TransportMethodNotSupportedFaultType - by notificaiton
	 * @throws BaseVersionNotMatchFaultType - by notification
	 */
	public UpdateResponse update(Update request)
		throws
			ResourceUnknownFaultType,
			ResourceNotReadyFaultType,
			ResourceUnavailableFaultType,
			UpdateFailedFaultType,
			IllegalDescriptorFaultType,
			TransportTypeNotSupportedFaultType,
			TransportMethodNotSupportedFaultType,
			BaseVersionNotMatchFaultType {

		log.info("update Start");

		UpdateResponse res = null;

		try {

			// for Resource
			ResourceContext rCtx = ResourceContext.getResourceContext();

			// for SOAP(WSDD param)
			SOAPService soapService =
				MessageContext.getCurrentContext().getService();

			// check state
			checkState(rCtx);

			// create AA resource
			ResourceKey newKey = createAAResource(rCtx, soapService);

			// create Epr of ApplicationArchive
			EndpointReferenceType newEpr = this.getAAEpr(newKey, soapService);

			// delegate to UpdateOperation
			UpdateOperation updateOperation =
				new UpdateOperation(request, rCtx, soapService, newKey, newEpr);
			updateOperation.startProcess();

			// setEPR
			res = new UpdateResponse();
			res.setArchiveEPR(newEpr);
			log.info("update End");

		} catch (NoSuchResourceException e) {
			log.fatal("Failed to get resource.", e);
			FaultHelper faultHelper =
				new FaultHelper(new ResourceUnknownFaultType(), true);
			faultHelper.addDescription("Failed to get resource.");
			faultHelper.addFaultCause(e);
			throw (ResourceUnknownFaultType) faultHelper.getFault();

		} catch (ResourceNotReadyException e) {
			log.fatal("Resource is in NotReady state.", e);
			FaultHelper faultHelper =
				new FaultHelper(new ResourceNotReadyFaultType(), true);
			faultHelper.addDescription("Resource is in NotReady state.");
			faultHelper.addDescription("Wait until Ready or Failed.");
			faultHelper.addFaultCause(e);
			throw (ResourceNotReadyFaultType) faultHelper.getFault();

		} catch (ResourceUnavailableException e) {
			log.fatal("Resource not available.", e);
			FaultHelper faultHelper =
				new FaultHelper(new ResourceUnavailableFaultType(), true);
			faultHelper.addDescription("Resource not available. ");
			faultHelper.addDescription("Destroy this resource.");
			faultHelper.addFaultCause(e);
			throw (ResourceUnavailableFaultType) faultHelper.getFault();

		} catch (Throwable t) {
			log.fatal("Failed to update.", t);
			FaultHelper faultHelper =
				new FaultHelper(new UpdateFailedFaultType(), true);
			faultHelper.addDescription("Failed to update.");
			faultHelper.addFaultCause(t);
			throw (UpdateFailedFaultType) faultHelper.getFault();
		}

		return res;
	}

	/**
	 * Implementation of GetContents operation.
	 * 
	 * @param request
	 * @return GetContentsResponse
	 * @throws ResourceUnknownFaultType
	 * @throws ResourceNotReadyFaultType
	 * @throws ResourceUnavailableFaultType
	 * @throws UnknownQueryExpressionDialectFaultType
	 * @throws InvalidQueryExpressionFaultType
	 * @throws TransportMethodNotSupportedFaultType
	 */
	public GetContentsResponse getContents(GetContents request)
		throws
			ResourceUnknownFaultType,
			ResourceNotReadyFaultType,
			ResourceUnavailableFaultType,
			UnknownQueryExpressionDialectFaultType,
			InvalidQueryExpressionFaultType,
			TransportMethodNotSupportedFaultType {

		GetContentsResponse res = null;

		try {

			// for Resource
			ResourceContext rCtx = ResourceContext.getResourceContext();

			// check state
			checkState(rCtx);

			// get ApplicationArchiveResource
			ApplicationArchiveResource aaResource =
				(ApplicationArchiveResource) rCtx.getResource();

			// delegate getArchive operation
			GetContentsOperation getContentsOperation =
				new GetContentsOperation(aaResource, request);

			res = getContentsOperation.getContents();

		} catch (NoSuchResourceException e) {
			log.fatal(
				"Failed to get resource.", e);
			FaultHelper faultHelper =
				new FaultHelper(new ResourceUnknownFaultType(), true);
			faultHelper.addDescription("Failed to get resource.");
			faultHelper.addFaultCause(e);
			throw (ResourceUnknownFaultType) faultHelper.getFault();

		} catch (ResourceNotReadyException e) {
			log.fatal("Resource is in NotReady state.", e);
			FaultHelper faultHelper =
				new FaultHelper(new ResourceNotReadyFaultType(), true);
			faultHelper.addDescription("Resource is in NotReady state.");
			faultHelper.addDescription("Wait until Ready or Failed.");
			faultHelper.addFaultCause(e);
			throw (ResourceNotReadyFaultType) faultHelper.getFault();

		} catch (ResourceUnavailableException e) {
			log.fatal("Resource not available.", e);
			FaultHelper faultHelper =
				new FaultHelper(new ResourceUnavailableFaultType(), true);
			faultHelper.addDescription("Resource not available. ");
			faultHelper.addDescription("Destroy this resource.");
			faultHelper.addFaultCause(e);
			throw (ResourceUnavailableFaultType) faultHelper.getFault();

		} catch (UnknownQueryDialectException e) {
			log.fatal("Query dialect not supported.", e);
			FaultHelper faultHelper =
				new FaultHelper(new UnknownQueryExpressionDialectFaultType(), true);
			faultHelper.addDescription("Query dialect not supported. ");
			faultHelper.addDescription("Check supported query dialect by getResourceProperty.");
			faultHelper.addFaultCause(e);
			throw (UnknownQueryExpressionDialectFaultType) faultHelper.getFault();

		} catch (InvalidQueryExpressionException e) {
			log.fatal("Query expression is invalid.", e);
			FaultHelper faultHelper =
				new FaultHelper(new InvalidQueryExpressionFaultType(), true);
			faultHelper.addDescription("Query expression is invalid. ");
			faultHelper.addDescription("Check expression again.");
			faultHelper.addFaultCause(e);
			throw (InvalidQueryExpressionFaultType) faultHelper.getFault();

		}catch(TransportMethodNotSupportedException e){
			log.fatal("Failed to getContents. " +
			"Check supported transport method by GetResourceProperty for ApplicationRepository.", e);
			FaultHelper faultHelper = new FaultHelper(new TransportMethodNotSupportedFaultType(), true);
			faultHelper.addDescription("Failed to getContens.");
			faultHelper.addDescription("Check supported transport method by GetResourceProperty for ApplicationRepository.");
			faultHelper.addFaultCause(e);
			throw (TransportMethodNotSupportedFaultType) faultHelper.getFault();

		} catch (Throwable t) {
			log.fatal("Failed to getContens.", t);
			FaultHelper faultHelper =
				new FaultHelper(new UpdateFailedFaultType(), true);
			faultHelper.addDescription("Failed to getContens.");
			faultHelper.addFaultCause(t);
			//throw (GetContensFailedFaultType) faultHelper.getFault();
		}
		return res;
	}

	/**
	 * Implementation of getArchive operation.
	 * 
	 * @param request
	 * @return GetArchiveResponse
	 * @throws ResourceUnknownFaultType
	 * @throws ResourceNotReadyFaultType
	 * @throws ResourceUnavailableFaultType
	 * @throws TransportTypeNotSupportedFaultType
	 * @throws TransportMethodNotSupportedFaultType
	 * @throws GetArchiveFailedFaultType
	 */
	public GetArchiveResponse getArchive(GetArchive request)
		throws
			ResourceUnknownFaultType,
			ResourceNotReadyFaultType,
			ResourceUnavailableFaultType,
			TransportTypeNotSupportedFaultType,
			TransportMethodNotSupportedFaultType,
			GetArchiveFailedFaultType {

		GetArchiveResponse res = null;

		try {

			// for Resource
			ResourceContext rCtx = ResourceContext.getResourceContext();

			// check state
			checkState(rCtx);

			// get ApplicationArchiveResource
			ApplicationArchiveResource aaResource =
				(ApplicationArchiveResource) rCtx.getResource();

			// delegate getArchive operation
			GetArchiveOperation getArchiveOperation =
				new GetArchiveOperation(aaResource, request);

			res = getArchiveOperation.getArchive();

		} catch (NoSuchResourceException e) {
			log.fatal(
				"Failed to get resource.", e);
			FaultHelper faultHelper =
				new FaultHelper(new ResourceUnknownFaultType(), true);
			faultHelper.addDescription("Failed to get resource.");
			faultHelper.addFaultCause(e);
			throw (ResourceUnknownFaultType) faultHelper.getFault();

		} catch (ResourceNotReadyException e) {
			log.fatal("Resource is in NotReady state.", e);
			FaultHelper faultHelper =
				new FaultHelper(new ResourceNotReadyFaultType(), true);
			faultHelper.addDescription("Resource is in NotReady state.");
			faultHelper.addDescription("Wait until Ready or Failed.");
			faultHelper.addFaultCause(e);
			throw (ResourceNotReadyFaultType) faultHelper.getFault();

		} catch (ResourceUnavailableException e) {
			log.fatal("Resource not available.", e);
			FaultHelper faultHelper =
				new FaultHelper(new ResourceUnavailableFaultType(), true);
			faultHelper.addDescription("Resource not available. ");
			faultHelper.addDescription("Destroy this resource.");
			faultHelper.addFaultCause(e);
			throw (ResourceUnavailableFaultType) faultHelper.getFault();

		}catch(TransportTypeNotSupportedException e){
			log.fatal("Failed to getArchive. " +
			"Check supported transport type by GetResourceProperty for ApplicationRepository.", e);
			FaultHelper faultHelper = new FaultHelper(new TransportTypeNotSupportedFaultType(), true);
			faultHelper.addDescription("Failed to getArchive.");
			faultHelper.addDescription("Check supported transport type by GetResourceProperty for ApplicationRepository.");
			faultHelper.addFaultCause(e);		
			throw (TransportTypeNotSupportedFaultType)faultHelper.getFault();

		}catch(TransportMethodNotSupportedException e){
			log.fatal("Failed to getArchive. " +
			"Check supported transport method by GetResourceProperty for ApplicationRepository.", e);
			FaultHelper faultHelper = new FaultHelper(new TransportMethodNotSupportedFaultType(), true);
			faultHelper.addDescription("Failed to getArchive.");
			faultHelper.addDescription("Check supported transport method by GetResourceProperty for ApplicationRepository.");
			faultHelper.addFaultCause(e);
			throw (TransportMethodNotSupportedFaultType) faultHelper.getFault();

		} catch (Throwable t) {
			log.fatal("Failed to getArchive.", t);
			FaultHelper faultHelper =
				new FaultHelper(new GetArchiveFailedFaultType(), true);
			faultHelper.addDescription("Failed to getArchive.");
			faultHelper.addFaultCause(t);
			throw (GetArchiveFailedFaultType) faultHelper.getFault();
		}

		return res;
	}

	/**
	 * Implementation of destroy operation.
	 * 
	 * @param request
	 * @return DestroyResponse
	 * @throws ResourceUnknownFaultType
	 * @throws ResourceUnavailableFaultType
	 * @throws ResourceNotDestroyedFaultType
	 */
	public DestroyResponse destroy(Destroy request)
		throws
			ResourceUnknownFaultType,
			org.oasis_open.docs.wsrf.r_2.ResourceUnavailableFaultType,
			ResourceNotDestroyedFaultType {

		try {

			// for Resource
			ResourceContext rCtx = ResourceContext.getResourceContext();

			// get ApplicationArchiveResource
			ApplicationArchiveResource aaResource =
				(ApplicationArchiveResource) rCtx.getResource();

			// delegate destroyArchive operation
			DestroyArchiveOperation destroyArchiveOperation =
				new DestroyArchiveOperation(aaResource);

			destroyArchiveOperation.destroyArchive();

		} catch (NoSuchResourceException e) {
			log.fatal(
				"Failed to get resource.", e);
			FaultHelper faultHelper =
				new FaultHelper(new ResourceUnknownFaultType(), true);
			faultHelper.addDescription("Failed to get resource.");
			faultHelper.addFaultCause(e);
			throw (ResourceUnknownFaultType) faultHelper.getFault();

		} catch (Throwable t) {
			log.fatal("Failed to destroy.", t);
			FaultHelper faultHelper =
				new FaultHelper(new ResourceNotDestroyedFaultType(), true);
			faultHelper.addDescription("Failed to destroy.");
			faultHelper.addFaultCause(t);
			throw (ResourceNotDestroyedFaultType) faultHelper.getFault();
		}

		return new DestroyResponse();
	}

	private void checkState(ResourceContext rCtx)
		throws
			ResourceContextException,
			ResourceException,
			ResourceUnavailableException,
			ResourceNotReadyException {

		AAInstanceStateType state =
			((ApplicationArchiveResource) rCtx.getResource()).getState();

		// Unavailable
		if (state == null || state.getValue() == null) {
			throw new ResourceUnavailableException("Resource not available. Destroy this resource.");

			// Failed
		} else if (
			AAInstanceStateType._value3.getLocalPart().equals(
				state.getValue().getLocalPart())) {
			throw new ResourceUnavailableException("Resource is in Failed state. Destroy this resource.");

			// NotReady
		} else if (
			AAInstanceStateType._value1.getLocalPart().equals(
				state.getValue().getLocalPart())) {
			throw new ResourceNotReadyException("Resource is in NotReady state. Wait until Ready or Failed.");
		}
	}

	private ResourceKey createAAResource(
		ResourceContext rCtx,
		SOAPService soapService)
		throws Exception {

		// get Repository epr
		EndpointReferenceType repoEPR =
			((ApplicationArchiveResource) rCtx.getResource()).getRepository();

		// create AA resource and get key
		ResourceKey key =
			((ApplicationArchiveResourceHome) rCtx.getResourceHome()).create(
				AAInstanceStateType.value1,
				null,
				null,
				null,
				null,
				null,
				repoEPR);

		return key;
	}

	/**
	 * Creates new AA resource that has state_NotReady, leaving other fields null.
	 * Creates EPR of the AA.
	 * 
	 * @param ResourceContext
	 * @param SOAPService
	 * @return EndpointReferenceType
	 * @throws Exception
	 */
	private EndpointReferenceType getAAEpr(
		ResourceKey key,
		SOAPService soapService)
		throws Exception {

		// get URI of AA service
		URL baseURL = ServiceHost.getBaseURL();
		String instanceService = (String) soapService.getOption("instance");
		String instanceURI = baseURL.toString() + instanceService;

		// create epr of AA
		EndpointReferenceType eprAA =
			AddressingUtils.createEndpointReference(instanceURI, key);

		return eprAA;
	}
}
