/*
 * Copyright 2004-2006 National Institute of Informatics in Japan, 
 * All rights reserved.
 *
 * This file or a portion of this file is licensed under the terms of
 * the NAREGI Public License, found at http://www.naregi.org/download/index.html.
 * If you redistribute this file, with or without modifications, you must 
 * include this notice in the file.
 *
 * [History]
 * Date         Name                  Contents
 * ------------------------------------------------------------------------
 * 2006/05/29   Fujitsu)Yositake      First Version 
 */

package org.naregi.pse.acs.service.impl;

import java.util.Map;

import org.apache.axis.types.URI.MalformedURIException;
import org.apache.log4j.Logger;
import org.globus.wsrf.ResourceException;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.impl.ResourceHomeImpl;
import org.globus.wsrf.impl.SimpleResourceKey;
import org.naregi.pse.acs.aaf.stubs.AADType;
import org.naregi.pse.acs.aaf.stubs.DiffAADType;
import org.naregi.pse.acs.ari.stubs.AAInstanceStateType;

//import org.w3.www._2005._03.addressing.EndpointReferenceType;
import org.apache.axis.message.addressing.EndpointReferenceType;

/**
 * ResourceHome class 
 * for ApplicationArchiveResource.
 */
public class ApplicationArchiveResourceHome extends ResourceHomeImpl {

	private static Logger log =
		Logger.getLogger(ApplicationArchiveResourceHome.class.getName());

	public ApplicationArchiveResourceHome(){
		super();
	}

	/**
	 * create ApplicationArchiveResource object.
	 * 
	 * @param _state
	 * @param _aad
	 * @param _diffAAD
	 * @param _creationDateTime
	 * @param _baseAA
	 * @param _newerAA
	 * @param _repository
	 * @return ResourceKey
	 * @throws ClassNotFoundException
	 * @throws ResourceException
	 */
	public ResourceKey create(
		AAInstanceStateType _state,
		AADType _aad,
		DiffAADType _diffAAD,
		String _creationDateTime,
		EndpointReferenceType _baseAA,
		EndpointReferenceType _newerAA,
		EndpointReferenceType _repository) throws ClassNotFoundException, ResourceException, MalformedURIException
		{

		// set class for ApplicationArchiveResource
		setResourceClass("org.naregi.pse.acs.service.impl.ApplicationArchiveResource");

		// create new instance of ApplicationArchiveResource
		ApplicationArchiveResource applicationArchiveResource = null;
		applicationArchiveResource = (ApplicationArchiveResource) createNewInstance();

		// initialize
		applicationArchiveResource.initialize(
				_state,
				_aad,
				_diffAAD,
				_creationDateTime,
				_baseAA,
				_newerAA,
				_repository);

		// get key
		ResourceKey key = new SimpleResourceKey(
				keyTypeName,
				applicationArchiveResource.getID());

		// add the resource to the list of resources in this home
		add(key, applicationArchiveResource);

		return key;
	}
	
	/**
	 * @return Map(key, applicationArchiveResource)
	 */
	public Map getAllResources(){
		
		return this.resources;
	}
}