/*
 * Copyright 2004-2006 National Institute of Informatics in Japan, 
 * All rights reserved.
 *
 * This file or a portion of this file is licensed under the terms of
 * the NAREGI Public License, found at http://www.naregi.org/download/index.html.
 * If you redistribute this file, with or without modifications, you must 
 * include this notice in the file.
 *
 * [History]
 * Date         Name                  Contents
 * ------------------------------------------------------------------------
 * 2005/05/31   Fujitsu)Yositake      First Version 
 */

package org.naregi.pse.acs.service.impl;

import java.net.URL;

//import org.w3.www._2005._03.addressing.EndpointReferenceType;
import org.apache.axis.message.addressing.EndpointReferenceType;

//import org.naregi.pse.acs.service.impl.functionbean.AddressingUtils;
import org.globus.wsrf.utils.AddressingUtils;

//import org.globus.wsrf.utils.FaultHelper;
import org.naregi.pse.acs.service.impl.functionbean.FaultHelper;

import org.apache.axis.handlers.soap.SOAPService;
import org.apache.axis.MessageContext;
import org.apache.log4j.Logger;

import org.globus.wsrf.NoSuchResourceException;
import org.globus.wsrf.ResourceContext;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.container.ServiceHost;
import org.globus.wsrf.impl.ResourceContextImpl;

import org.naregi.pse.acs.ari.stubs.AAInstanceStateType;
import org.naregi.pse.acs.ari.stubs.Create;
import org.naregi.pse.acs.ari.stubs.CreateResponse;
import org.naregi.pse.acs.ari.stubs.CreationFailedFaultType;
import org.naregi.pse.acs.ari.stubs.IllegalDescriptorFaultType;
import org.naregi.pse.acs.ari.stubs.LookupArchives;
import org.naregi.pse.acs.ari.stubs.LookupArchivesResponse;
import org.naregi.pse.acs.ari.stubs.LookupFailedFaultType;
import org.naregi.pse.acs.ari.stubs.OperationNotSupportedFaultType;
import org.naregi.pse.acs.ari.stubs.TransportMethodNotSupportedFaultType;
import org.naregi.pse.acs.ari.stubs.TransportTypeNotSupportedFaultType;

import org.oasis_open.docs.wsrf.r_2.ResourceUnknownFaultType;
import org.oasis_open.docs.wsrf.rp_2.InvalidQueryExpressionFaultType;
import org.oasis_open.docs.wsrf.rp_2.UnknownQueryExpressionDialectFaultType;

/**
 *  ApplicationRepository portType implementation
 *
 */
public class ApplicationRepository {

	private static Logger log =
		Logger.getLogger(ApplicationRepository.class.getName());
	
		// create the only one resource of ApplicationRepository when loaded in memory.
		// destroy ends up with no resource until gt4 container reboots.
		static {
			
			log.info("ApplicationRepository initialization START");
			try{
			
				ResourceContext ctx = null;
				
				ApplicationRepositoryResourceHome home = null;
				ResourceKey key = null;
				
				ctx = ResourceContext.getResourceContext();
				
				// required because no SOAPMessageContext established
				// we want get service name from WSDD or JNDI for service name change
				((ResourceContextImpl)ctx).setService("naregi/pse/acs/ApplicationRepository");
				home = (ApplicationRepositoryResourceHome)ctx.getResourceHome();

				key = home.create();
		
				log.info("ApplicationRepository initialization END");		
			}catch(Throwable t){
				log.fatal("Initialization failed.", t);
			}
		}

	/**
	 * Implementation of Create operation.
	 * 
	 * @param request
	 * @return CreateResponse
	 * @throws ResourceUnknownFaultType
	 * @throws CreationFailedFaultType - by notificaiton too
	 * @throws IllegalDescriptorFaultType - by notificaiton
	 * @throws TransportTypeNotSupportedFaultType - by notificaiton
	 * @throws TransportMethodNotSupportedFaultType - by notificaiton
	 */
	public CreateResponse create(Create request)
		throws
			ResourceUnknownFaultType,
			CreationFailedFaultType,
			IllegalDescriptorFaultType,
			TransportTypeNotSupportedFaultType,
			TransportMethodNotSupportedFaultType {

		log.info("create START");
		CreateResponse res = null;
		
		try {
			// for Resource
			ResourceContext rCtx = ResourceContext.getResourceContext();

			// for SOAP(WSDD param)
			SOAPService soapService =
				MessageContext.getCurrentContext().getService();

			// create AA resource
			ResourceKey aaKey = createAAResource(rCtx, soapService);

			// create Epr of ApplicationArchive
			EndpointReferenceType aaEpr = this.getAAEpr(aaKey, soapService);

			// delegate to CreateOperation
			CreateOperation createOperation =
				new CreateOperation(request, rCtx, soapService, aaKey, aaEpr);
			createOperation.startProcess();

			// setEPR
			res = new CreateResponse();
			res.setArchiveEPR(aaEpr);
			log.info("create END");
			
			return res;
			
		} catch (NoSuchResourceException e) {
			log.fatal("Failed to get resource. " +
						"Reboot GT4 container for default resource creation.", e);
			FaultHelper faultHelper =
				new FaultHelper(new ResourceUnknownFaultType(), true);
			faultHelper.addDescription("Failed to get resource.");
			faultHelper.addDescription("Reboot GT4 container for default resource creation.");
			faultHelper.addFaultCause(e);
			throw (ResourceUnknownFaultType) faultHelper.getFault();

		} catch (Throwable t) {
			log.fatal("Failed to create.", t);
			FaultHelper faultHelper =
				new FaultHelper(new CreationFailedFaultType(), true);
			faultHelper.addDescription("Failed to create.");
			faultHelper.addFaultCause(t);
			throw (CreationFailedFaultType) faultHelper.getFault();
		}
	}

	private ResourceKey createAAResource(
		ResourceContext rCtx,
		SOAPService soapService)
		throws Exception {

		// get Repository epr
		EndpointReferenceType eprAR = this.getAREpr(rCtx, soapService);

		// create AA resource and get key
		((ResourceContextImpl)rCtx).setService("naregi/pse/acs/ApplicationArchive");
		ResourceKey key =
			((ApplicationArchiveResourceHome) rCtx.getResourceHome()).create(
				AAInstanceStateType.value1,
				null,
				null,
				null,
				null,
				null,
				eprAR);

		return key;
	}

	/**
	 * Creates new AA resource that has state_NotReady, leaving other fields null.
	 * Creates EPR of the AA.
	 * 
	 * @param ResourceContext
	 * @param SOAPService
	 * @return EndpointReferenceType
	 * @throws Exception
	 */
	private EndpointReferenceType getAAEpr(
		ResourceKey key,
		SOAPService soapService)
		throws Exception {

		// get URI of AA service instance is defined in WSDD
		URL baseURL = ServiceHost.getBaseURL();
		String instanceService = (String) soapService.getOption("instance");
		String instanceURI = baseURL.toString() + instanceService;

		// create epr of AA
		EndpointReferenceType eprAA =
			AddressingUtils.createEndpointReference(instanceURI, key);

		return eprAA;
	}

	/**
	 * Creates EPR of the AR.
	 * 
	 * @param ResourceContext
	 * @param SOAPService
	 * @return EndpointReferenceType
	 * @throws Exception
	 */
	private EndpointReferenceType getAREpr(
		ResourceContext rCtx,
		SOAPService soapService)
		throws Exception {

		// get ResourceKey
		ResourceKey key = rCtx.getResourceKey();

		// get URI of AA service factory is defined in WSDD
		URL baseURL = ServiceHost.getBaseURL();
		String factoryService = (String) soapService.getOption("factory");
		String factoryURI = baseURL.toString() + factoryService;

		// create epr of AA
		EndpointReferenceType eprAR =
			AddressingUtils.createEndpointReference(factoryURI, key);

		return eprAR;
	}

	/**
	 * Implementation of LookupArchives operation.
	 * Currently, this operation is not supported.
	 * 
	 * @param LookupArchives
	 * @return LookupArchivesResponse
	 * @throws OperationNotSupportedFaultType
	 */
	public LookupArchivesResponse LookupArchives(LookupArchives request)
		throws
			OperationNotSupportedFaultType,
			ResourceUnknownFaultType,
			UnknownQueryExpressionDialectFaultType,
			InvalidQueryExpressionFaultType,
			LookupFailedFaultType {

		if (true) {
			log.error("Currently LookupArchives not implemented.");
			FaultHelper faultHelper =
				new FaultHelper(new OperationNotSupportedFaultType(), true);
			faultHelper.addDescription(
				"Currently LookupArchives not implemented.");
			throw (OperationNotSupportedFaultType) faultHelper.getFault();
		}
		return new LookupArchivesResponse();
	}
}
