/*
 * Copyright 2004-2006 National Institute of Informatics in Japan, 
 * All rights reserved.
 *
 * This file or a portion of this file is licensed under the terms of
 * the NAREGI Public License, found at http://www.naregi.org/download/index.html.
 * If you redistribute this file, with or without modifications, you must 
 * include this notice in the file.
 *
 * [History]
 * Date         Name                  Contents
 * ------------------------------------------------------------------------
 * 2006/05/29   Fujitsu)Yositake      First Version 
 */

package org.naregi.pse.acs.service.impl;

import java.util.Map;

import org.apache.log4j.Logger;
import org.globus.wsrf.NoSuchResourceException;
import org.globus.wsrf.Resource;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.impl.ResourceHomeImpl;
import org.globus.wsrf.impl.SimpleResourceKey;
import org.naregi.pse.acs.service.impl.constants.ACSConstants;
import org.naregi.pse.acs.service.impl.property.ACSSystemInfo;

/**
 * ResourceHome class 
 * for ApplicationRepositoryResource.
 */
public class ApplicationRepositoryResourceHome extends ResourceHomeImpl {

	private static Logger log =
		Logger.getLogger(ApplicationRepositoryResourceHome.class.getName());

	public ApplicationRepositoryResourceHome(){
		super();
	}

	/**
	 * create ApplicationRepositoryResource object.
	 * 
	 * @return
	 * @throws ClassNotFoundException
	 * @throws ResourceException
	 * @throws MalformedURIException
	 */
	public ResourceKey create() throws Exception{
		
		// set class for ApplicationRepositoryResource
		setResourceClass("org.naregi.pse.acs.service.impl.ApplicationRepositoryResource");

		// get key
		ResourceKey keyCheck = new SimpleResourceKey(
				keyTypeName,
				new String(ACSSystemInfo.getInstance().getProperty
								(ACSConstants.AR_RESOURCE_KEY_KEY)));
		boolean notFound = false;
		
		// check if it exists
		try{
			Resource r = this.find(keyCheck);
		}catch(NoSuchResourceException e){
			notFound = true; 
		}
		
		// create if not created
		if(notFound){

			log.info("ApplicationRepository resource not found, creating resource.");
			
			// create new instance of ApplicationRepositoryResource
			ApplicationRepositoryResource applicationRepositoryResource = null; 
			applicationRepositoryResource =
					(ApplicationRepositoryResource) createNewInstance();
		
			// initialize
			applicationRepositoryResource.initialize();

			// get key
			ResourceKey key = new SimpleResourceKey(
					keyTypeName,
					applicationRepositoryResource.getID());

			// add the resource to the list of resources in this home
			add(key, applicationRepositoryResource);			
		
			return key;
		
		// it is already exists
		}else{
			
			log.info("ApplciationRepository resource already exists.");
			return keyCheck;
		}
	}
	
	/**
	 * @return Map(key, ApplicationRepositoryResource)
	 */
	public Map getAllResources(){
		
		return this.resources;
	}
}