/*
 * Copyright 2004-2006 National Institute of Informatics in Japan, 
 * All rights reserved.
 *
 * This file or a portion of this file is licensed under the terms of
 * the NAREGI Public License, found at http://www.naregi.org/download/index.html.
 * If you redistribute this file, with or without modifications, you must 
 * include this notice in the file.
 *
 * [History]
 * Date         Name                  Contents
 * ------------------------------------------------------------------------
 * 2006/06/01   Fujitsu)Yositake      First Version
 * 
 */

package org.naregi.pse.acs.service.impl;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;

import org.apache.axis.types.URI;
import org.apache.log4j.Logger;
import org.naregi.pse.acs.service.impl.constants.ACSConstants;
import org.naregi.pse.acs.service.impl.exception.TransportMethodNotSupportedException;
import org.naregi.pse.acs.service.impl.exception.TransportTypeNotSupportedException;
import org.naregi.pse.acs.service.impl.functionbean.FileManager;
import org.naregi.pse.acs.service.impl.property.ACSSystemInfo;

/**
 * @author Yositake
 * 
 * CommonOperation
 * This class must be exactlly same as CommonOperationThread class except 
 * class name and extends.
 */
public class CommonOperation{

	private static Logger log =
			Logger.getLogger(CommonOperation.class.getName());

	protected FileManager fileUtil;

	protected File tmpDir;

	// actual relative file list found physically
	// contains String object
	protected ArrayList relativeFileList;

	/**
	 * Common
	 * @param dir
	 * @return
	 */
	protected String checkDir(String dir){
		
		log.info("checkDir START");
		log.info("paramDir " + dir);
		
		String retDir = dir;
		
		// attach / to last
		if(retDir.endsWith("/") == false){
			retDir = retDir + "/";
		}
		
		// swap \ to / 
		retDir = retDir.replace('\\', '/');
		
		log.info("retDir = " + retDir);
		log.info("checkDir END");
		
		return retDir ;
	}

	/**
	 * Common
	 * @throws Exception
	 */
	protected void makeTmpDir() throws Exception{
		
		log.info("makeTmpDir START");
		
		// get tmp base dir
		String strTmpDir = 
			ACSSystemInfo.getInstance().getProperty(ACSConstants.ACS_TEMP_BASEDIR_KEY);
		
		// check last /, \
		strTmpDir = checkDir(strTmpDir);
		
		strTmpDir = strTmpDir + new Date().getTime() + "/";

		File fileTmpDir = new File(strTmpDir);

		// check exists, timestamp overlap
		if(fileTmpDir.exists()){
			throw new Exception("Tmp dir already exists. " + fileTmpDir.getPath());
		}
		
		// make dir
		if(fileTmpDir.mkdirs() == false){
			throw new Exception("Faild to make dir. " + fileTmpDir.getPath());
		}
		
		// set to property
		this.tmpDir = fileTmpDir;
		
		log.info("makeTmpDir END");
	}

	/**
	 * Common
	 * @return
	 * @throws Exception
	 */
	protected int checkTransportType(URI uriTransType) throws Exception {
		
		int intTransType = -1;
		
		if(uriTransType == null){
			throw new Exception("Transport Type is null.");
		}
		
		// descrete
		if(uriTransType.toString().equalsIgnoreCase(ACSConstants.ST_TRANS_TYPE_DESCRETE)){
			intTransType = ACSConstants.TRANS_TYPE_DESCRETE;
		
		// bundled/zip
		}else if(uriTransType.toString().equalsIgnoreCase(ACSConstants.ST_TRANS_TYPE_BUNDLED_ZIP)){
			intTransType = ACSConstants.TRANS_TYPE_BUNDLED_ZIP;
		
		// not supported
		}else {
			throw new TransportTypeNotSupportedException
					("Transport Type " + uriTransType.toString() +" Not Supported.");
		}
		
		return intTransType;
	}

	/**
	 * Common
	 * @param uriTransMethod
	 * @return
	 * @throws Exception
	 */
	protected int checkTransportMethod(URI uriTransMethod) throws Exception {

		int intTransMethod = -1;

		if(uriTransMethod == null){
			throw new Exception("Transport Method is null.");
		}
		
		// embedded
		if(uriTransMethod.toString().equalsIgnoreCase(ACSConstants.ST_TRANS_METHOD_EMBEDDED)){
			intTransMethod = ACSConstants.TRANS_METHOD_EMBEDDED;
		
		// gridftp
		}else if(uriTransMethod.toString().equalsIgnoreCase(ACSConstants.ST_TRANS_METHOD_GRIDFTP)){
			intTransMethod = ACSConstants.TRANS_METHOD_GRIDFTP;
		
		// not supported
		}else {
			throw new TransportMethodNotSupportedException
					("Transport Method " + uriTransMethod.toString() +" Not Supported.");
		}	
		
		return intTransMethod;
	}

	/**
	 * Common
	 * @param _file
	 * @throws Exception
	 */
	protected void makeSureFileNotExist(File _file) throws Exception{

		if(_file.exists()){
			if(_file.delete() == false){
				throw new Exception();
			}
		}		
	}

	/**
	 * Common
	 * @param baseDir
	 * @return
	 */
	protected String[] getAbsoluteFromRelative(File baseDir){
	
		String[] absolute = new String[this.relativeFileList.size()];
		
		for(int i = 0; i < this.relativeFileList.size(); i++){
			absolute[i] = baseDir.getPath() + "/" + this.relativeFileList.get(i);
		}
		
		return absolute;	
	}

}
