/*
 * Copyright 2004-2006 National Institute of Informatics in Japan, 
 * All rights reserved.
 *
 * This file or a portion of this file is licensed under the terms of
 * the NAREGI Public License, found at http://www.naregi.org/download/index.html.
 * If you redistribute this file, with or without modifications, you must 
 * include this notice in the file.
 *
 * [History]
 * Date         Name                  Contents
 * ------------------------------------------------------------------------
 * 2006/06/01   Fujitsu)Yositake      First Version
 * 
 */

package org.naregi.pse.acs.service.impl;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;

import org.apache.axis.handlers.soap.SOAPService;

//import org.w3.www._2005._03.addressing.EndpointReferenceType;
import org.apache.axis.message.addressing.EndpointReferenceType;

import org.apache.log4j.Logger;
import org.globus.wsrf.NoResourceHomeException;
import org.globus.wsrf.ResourceContext;
import org.globus.wsrf.ResourceContextException;
import org.globus.wsrf.ResourceException;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.encoding.ObjectSerializer;
import org.globus.wsrf.impl.ResourceContextImpl;

import org.naregi.pse.acs.aaf.stubs.AADType;
import org.naregi.pse.acs.aaf.stubs.AAIDType;
import org.naregi.pse.acs.ari.stubs.AAInstanceStateType;
import org.naregi.pse.acs.ari.stubs.ApplicationArchiveCreatedMessageType;
import org.naregi.pse.acs.ari.stubs.Create;
import org.naregi.pse.acs.ari.stubs.CreationFailedFaultType;
import org.naregi.pse.acs.ari.stubs.IllegalDescriptorFaultType;
import org.naregi.pse.acs.ari.stubs.TransportMethodNotSupportedFaultType;
import org.naregi.pse.acs.ari.stubs.TransportTypeNotSupportedFaultType;
import org.naregi.pse.acs.service.impl.exception.IllegalDescriptorException;
import org.naregi.pse.acs.service.impl.exception.TransportMethodNotSupportedException;
import org.naregi.pse.acs.service.impl.exception.TransportTypeNotSupportedException;
import org.naregi.pse.acs.service.impl.functionbean.AADCheck;
import org.naregi.pse.acs.service.impl.functionbean.AADTransformerForGt4;
import org.naregi.pse.acs.service.impl.functionbean.AcsFileManager;
import org.naregi.pse.acs.service.impl.functionbean.FaultHelper;
import org.naregi.pse.acs.service.impl.functionbean.ZipFileManager;
import org.naregi.pse.acs.service.impl.constants.ACSConstants;

/**
 * @author Yositake
 * 
 * As a thread to implement the Create operation.
 * 
 */
public class CreateOperation extends CreateUpdateCommonOperation{

	private static Logger log =
		Logger.getLogger(CreateOperation.class.getName());

	private Create request;
	
	private ApplicationRepositoryResource arResource;
	
	private ApplicationArchiveResource aaResource;
	
	private EndpointReferenceType aaEPR;
	
	/**
	 * Constructor
	 * @param _request
	 * @param _rCtx
	 * @param _soapService
	 * @param _aaKey
	 * @param _aaEPR
	 * @throws ResourceContextException
	 * @throws ResourceException
	 */
	public CreateOperation(
		Create _request,
		ResourceContext _rCtx,
		SOAPService _soapService,
		ResourceKey _aaKey,
		EndpointReferenceType _aaEPR) throws ResourceContextException, ResourceException {

		
		log.info("CreateOperation constructor START");
		
		this.request = _request;
		this.aaEPR = _aaEPR;
		
		// set AR resource
		setARResource(_rCtx);
		
		// set AA resource
		setAAResource(_rCtx, _aaKey);

		this.fileUtil = new AcsFileManager();
		
		this.relativeFileList = new ArrayList();
		
		log.info("CreateOperation constructor END");
	}

	private void setARResource(ResourceContext _rCtx) throws ResourceContextException, ResourceException{
		
		((ResourceContextImpl)_rCtx).setService("naregi/pse/acs/ApplicationRepository");
		this.arResource = (ApplicationRepositoryResource)_rCtx.getResource();
		
	}

	private void setAAResource(ResourceContext _rCtx, ResourceKey _key) throws NoResourceHomeException, ResourceException, ResourceContextException{
			
		((ResourceContextImpl)_rCtx).setService("naregi/pse/acs/ApplicationArchive");
		this.aaResource = (ApplicationArchiveResource)((ApplicationArchiveResourceHome)_rCtx.getResourceHome()).find(_key);
	}

	/**
	 * main process for Create
	 */
	private void process() throws Throwable {
		log.info("Process for Create START");

		// store files to tmp dir
		try{
			collectFiles();
		}catch(Throwable t){
			throw t;
		}finally{
			if(this.fileTransfer != null){
				this.fileTransfer.closeConnection();
			}
		}

		// check AAD
		checkAAD();

		// store Repository from tmp
		storeRepository();

		// update resource property
		updateAARP();

		log.info("process for Create END");
	}

	private void collectFiles() throws Exception {
		
		log.info("collectFiles START");
		
		// make tmpDir & set to property
		makeTmpDir();
		
		// get transport type
		int transType = checkTransportType(this.request.getAA().getTransportType());
		
		// for descrete
		if(transType == ACSConstants.TRANS_TYPE_DESCRETE){

			collectDescreteFiles(this.request.getAA());
		
		// for bundled/zip
		}else if(transType == ACSConstants.TRANS_TYPE_BUNDLED_ZIP){
			
			collectBundledZipFile(this.request.getAA().getBundle());
		}
		
		log.info("collectFiles END");
	}

	private void checkAAD() throws Exception {

		log.info("checkAAD START");
		
		this.aadCheck = new AADCheck();
		this.aadCheck.checkAAD(this.aad.getPath(), this.relativeFileList);
		
		log.info("checkAAD END");
	}

	private void storeRepository() throws Exception{
		
		log.info("storeRepository START");
		
		// check empty AA
		if(this.relativeFileList.isEmpty()){
			throw new Exception("No contents for AA. \n" +
				"Implementation does not accept AA without any contents.");
		}

		// check already exists
		String zip = this.tmpDir.getPath() + "/" + ACSConstants.BUNDLED_FILENAME_ZIP;
		File zipFile = new File(zip);
		makeSureFileNotExist(zipFile);
		
		// get absolute from relative
		String[] absolute = getAbsoluteFromRelative(this.tmpDir);
		
		// zip
		((ZipFileManager)this.fileUtil).
			zipWithDir(zip, absolute, (String[])this.relativeFileList.toArray(new String[0]));
		
		// get repository location
		File aaStoredDir = ((AcsFileManager)this.fileUtil).
			getAAStoredDirFromAAID(this.aadCheck.getRepoBaseDir(), this.aadCheck.getJaxbAAD().getAAID());
		
		// check AAID duplication
		if(new File(aaStoredDir.getPath() + "/" + ACSConstants.BUNDLED_FILENAME_ZIP).exists()){
			throw new Exception("The specified AAID already exists. \n " +
				"AAID can't be duplicated. Specify other AAID. \n" + 
				"name = " + this.aadCheck.getJaxbAAD().getAAID().getName() + "\n" + 
				"version = " + this.aadCheck.getJaxbAAD().getAAID().getVersion());
		}
		
		// move to repository
		((AcsFileManager)this.fileUtil).moveTmpFileToAAStoredDir(zipFile, aaStoredDir);
			
		log.info("storeRepository END");
	}

	private void updateAARP() throws Throwable {

		log.info("updateRP START");

		// get AADType of gt4
		AADTransformerForGt4 trans = new AADTransformerForGt4();
		trans.jaxbAAD2Gt4AAD(this.aadCheck.getJaxbAAD());
		AADType gt4AAD = trans.getGt4AAD();

		// set AAD
		this.aaResource.setAAD(gt4AAD);

		// set state
		this.aaResource.setState(AAInstanceStateType.value2);

		// set creationDateTime 
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy MM/dd HH:mm:ss");
		String creationDateTime = sdf.format(new Date());
		this.aaResource.setCreationDateTime(creationDateTime);		
	
		// update CreatedMessage
		updateCreatedMessage(creationDateTime, gt4AAD.getAAID());

		log.info("updateRP END");
	}

	private void updateCreatedMessage(
		String creationDateTime,
		AAIDType aaid) throws ResourceException {
		
		log.info("updateCreatedMessage START");
		
		// create object
		ApplicationArchiveCreatedMessageType created = 
				new ApplicationArchiveCreatedMessageType();
		
		// set creationDateTime
		created.setDateTime(Calendar.getInstance());
		
		// set AAID
		created.setAAID(aaid);
		
		// set aaEPR
		created.setArchiveEPR(this.aaEPR);
		
		// set to RP & shot notification
		this.arResource.setApplicationArchiveCreated(created);
		
		log.info("updateCreatedMessage END");
	}

	/**
	 * The handler to catch exception while processing.
	 */
	public void errorHandler() throws Exception{
		
		try{
			process();
		
		}catch(TransportTypeNotSupportedException e){
			log.fatal("Failed to create. " +
			"Check supported transport type by GetResourceProperty for ApplicationRepository.", e);
			FaultHelper fh = new FaultHelper(new TransportTypeNotSupportedFaultType(), true);
			fh.addDescription("Failed to create.");
			fh.addDescription("Check supported transport type by GetResourceProperty for ApplicationRepository.");
			fh.addFaultCause(e);		
			TransportTypeNotSupportedFaultType f = (TransportTypeNotSupportedFaultType)fh.getFault();
			this.arResource.setFaultMessage(ObjectSerializer.toElement(f, 
						TransportTypeNotSupportedFaultType.getTypeDesc().getXmlType()));
			setToFailedState();
		
		}catch(TransportMethodNotSupportedException e){
			log.fatal("Failed to create. " +
			"Check supported transport method by GetResourceProperty for ApplicationRepository.", e);
			FaultHelper fh = new FaultHelper(new TransportMethodNotSupportedFaultType(), true);
			fh.addDescription("Failed to create.");
			fh.addDescription("Check supported transport method by GetResourceProperty for ApplicationRepository.");
			fh.addFaultCause(e);		
			TransportMethodNotSupportedFaultType f = (TransportMethodNotSupportedFaultType)fh.getFault();
			this.arResource.setFaultMessage(ObjectSerializer.toElement(f, 
					TransportMethodNotSupportedFaultType.getTypeDesc().getXmlType()));		
			setToFailedState();
		
		}catch(IllegalDescriptorException e){
			log.fatal("Failed to create. " +
			"AAD is illegal.", e);
			FaultHelper fh = new FaultHelper(new IllegalDescriptorFaultType(), true);
			fh.addDescription("Failed to create.");
			fh.addDescription("AAD is illegal.");
			fh.addFaultCause(e);
			IllegalDescriptorFaultType f = (IllegalDescriptorFaultType)fh.getFault();
			this.arResource.setFaultMessage(ObjectSerializer.toElement(f, 
					IllegalDescriptorFaultType.getTypeDesc().getXmlType()));
			setToFailedState();

		}catch(Throwable t){
			log.fatal("Failed to create.", t);
			FaultHelper fh = new FaultHelper(new CreationFailedFaultType(), true);
			fh.addDescription("Failed to create.");
			fh.addFaultCause(t);		
			CreationFailedFaultType f = (CreationFailedFaultType)fh.getFault();
			this.arResource.setFaultMessage(ObjectSerializer.toElement(f, 
					CreationFailedFaultType.getTypeDesc().getXmlType()));
			setToFailedState();
		}		
	}

	private void setToFailedState() throws ResourceException{
		this.aaResource.setState(AAInstanceStateType.value3);
	}

	/* 
	 * @see java.lang.Runnable#run()
	 */
	public void run() {

		try {
			errorHandler();
		} catch (Exception e) {
			log.fatal("Failed.", e);
		}

	}
}