/*
 * Copyright 2004-2006 National Institute of Informatics in Japan, 
 * All rights reserved.
 *
 * This file or a portion of this file is licensed under the terms of
 * the NAREGI Public License, found at http://www.naregi.org/download/index.html.
 * If you redistribute this file, with or without modifications, you must 
 * include this notice in the file.
 *
 * [History]
 * Date         Name                  Contents
 * ------------------------------------------------------------------------
 * 2006/06/29   Fujitsu)Yositake      First Version
 * 
 */

package org.naregi.pse.acs.service.impl;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;

import org.apache.axis.message.MessageElement;
import org.apache.log4j.Logger;
import org.naregi.pse.acs.ari.stubs.AAType;
import org.naregi.pse.acs.ari.stubs.AATypeBundle;
import org.naregi.pse.acs.ari.stubs.AATypeContent;
import org.naregi.pse.acs.ari.stubs.AATypeDescriptor;
import org.naregi.pse.acs.service.impl.constants.ACSConstants;
import org.naregi.pse.acs.service.impl.databean.UrlInfo;
import org.naregi.pse.acs.service.impl.filetransfer.FileTransfer;
import org.naregi.pse.acs.service.impl.functionbean.AADCheck;
import org.naregi.pse.acs.service.impl.functionbean.ZipFileManager;
import org.w3c.dom.NodeList;

/**
 * @author Yositake
 * 
 * CreateUpdateCommonOperation
 */
public class CreateUpdateCommonOperation extends CommonOperationThread{

	private static Logger log =
			Logger.getLogger(CreateUpdateCommonOperation.class.getName());

	protected FileTransfer fileTransfer;

	// aad.xml
	protected File aad;

	protected AADCheck aadCheck;

	/**
	 * Common
	 * @throws Exception
	 */
	protected void collectDescreteFiles(AAType aa) throws Exception{
		
		log.info("collectDescreteFiles START");
		
		// for descriptor
		AATypeDescriptor des = aa.getDescriptor();
		collectDescriptor(des);
		
		// for contents
		AATypeContent[] contents = aa.getContent();
		collectContents(contents);
		
		log.info("collectDescreteFiles END");
	}

	/**
	 * Common
	 * @param des
	 * @throws Exception
	 */
	protected void collectDescriptor(AATypeDescriptor des) throws Exception{
		
		log.info("collectDescriptor START");
		
		// get Transport Method
		int transMethod = checkTransportMethod(des.getTransportMethod());

		File descFile = null;
		String strDescFile = this.tmpDir.getPath() + "/" + ACSConstants.AAD_FILENAME;
		descFile = new File(strDescFile);
		makeSureFileExists(descFile);
		
		// embedded
		if(transMethod == ACSConstants.TRANS_METHOD_EMBEDDED){
			
			byte[] entity = des.getEmbedded();
			this.fileUtil.writeFileFrombyte(descFile, entity);
		
		// gridftp (guaranteed)
		}else{
			
			MessageElement[] any = des.get_any();
			NodeList text = any[0].getChildNodes();
			String url = text.item(0).getNodeValue();
			UrlInfo urlInfo = getUrlInfo(url);
			if(this.fileTransfer == null){
				this.fileTransfer = new FileTransfer();
			}
			fileTransfer.getFromAcsClientToAcsServer(null, urlInfo, strDescFile);
		}
		
		// set descriptor
		this.aad = descFile;
		
		log.info("collectDescriptor END");
	}

	/**
	 * Candidate of Common
	 * @param contents
	 * @throws Exception
	 */
	protected void collectContents(AATypeContent[] contents) throws Exception{
		
		log.info("collectContents START");
		
		if(contents != null){
			
			// for each content
			for(int i = 0; i < contents.length; i++){
	
				// get Transport Method
				int transMethod = checkTransportMethod(contents[i].getTransportMethod());
				
				String relativePath = null;
				relativePath = checkRelativePath(contents[i].getPathname());
				String strContentFile = this.tmpDir.getPath() + "/" + relativePath;
				File contentFile = new File(strContentFile);
				makeSureFileExists(contentFile);
				
				// embedded
				if(transMethod == ACSConstants.TRANS_METHOD_EMBEDDED){
					
					byte[] entity = contents[i].getEmbedded();
					this.fileUtil.writeFileFrombyte(contentFile, entity);				
			
				// gridftp (guaranteed)
				}else{
					
					// get
					MessageElement[] any = contents[i].get_any();
					NodeList text = any[0].getChildNodes();
					String url = text.item(0).getNodeValue();
					UrlInfo urlInfo = getUrlInfo(url);
					if(this.fileTransfer == null){
						this.fileTransfer = new FileTransfer();
					}
					fileTransfer.getFromAcsClientToAcsServer(null, urlInfo, strContentFile);
				}
				
				// add to actual file list
				this.relativeFileList.add(relativePath);		
			}
		}
		
		log.info("collectContents END");
	}

	/**
	 * Commmon
	 * @throws Exception
	 */
	protected void collectBundledZipFile(AATypeBundle bundle) throws Exception{
		
		log.info("collectBundledZipFile STRAT");
		
		// get Transport Method
		int transMethod = checkTransportMethod(bundle.getTransportMethod());

		String strBundledZipFile = this.tmpDir.getPath() + "/" + ACSConstants.BUNDLED_FILENAME_ZIP;
		File bundledZipFile = new File(strBundledZipFile);
		makeSureFileExists(bundledZipFile);
		
		// embedded
		if(transMethod == ACSConstants.TRANS_METHOD_EMBEDDED){
			
			byte[] entity = bundle.getEmbedded();
			this.fileUtil.writeFileFrombyte(bundledZipFile, entity);

		// gridftp (guaranteed)
		}else{
			// get
			MessageElement[] any = bundle.get_any();
			NodeList text = any[0].getChildNodes();
			String url = text.item(0).getNodeValue();
			UrlInfo urlInfo = getUrlInfo(url);
			if(this.fileTransfer == null){
				this.fileTransfer = new FileTransfer();
			}
			fileTransfer.getFromAcsClientToAcsServer(null, urlInfo, strBundledZipFile);
		}
		
		// unzip
		String[] strFileList = 
			((ZipFileManager)fileUtil).unZipWithDir
				(this.tmpDir, 
				 new File(strBundledZipFile));

		// add to actual file list
		for(int i = 0; i < strFileList.length; i++){
			
			if(strFileList[i].endsWith(ACSConstants.AAD_FILENAME)){
				this.aad = new File(this.tmpDir.getPath() + "/" + strFileList[i]);
			}else{
				this.relativeFileList.add(strFileList[i]);
			}
		}
		
		log.info("collectBundledZipFile END");
	}

	/**
	 * Common
	 * @param url
	 * @return
	 * @throws MalformedURLException
	 */
	protected UrlInfo getUrlInfo(String url) throws MalformedURLException{
		
		UrlInfo urlInfo = new UrlInfo();
		
		URL tmpFtpURL = null;
		if(url.startsWith("gsiftp")){
			tmpFtpURL = new URL(url.replaceFirst("gsiftp", "ftp"));
		}
		urlInfo.setAddress(tmpFtpURL.getHost());
		urlInfo.setPort(String.valueOf(tmpFtpURL.getPort()));
		urlInfo.setFile(tmpFtpURL.getFile());
		
		return urlInfo;
	}

	/**
	 * Common
	 * Guaranteed tmp dir is newly created for this operation.
	 * (No files exists in tmp dir.)
	 * 
	 * @param file
	 * @throws Exception
	 */
	protected void makeSureFileExists(File file) throws Exception{
		
		// make dir inside tmp dir if needed
		File parent = file.getParentFile();
		if(parent.exists() == false){
			if(parent.mkdirs() == false){
				throw new Exception("Failed to make dir. " + parent.getPath());
			}
		}

		// create file
		if(file.createNewFile() == false){
			throw new Exception("Failed to create file. " + file.getPath());
		}
	}

	/**
	 * Common
	 * @param _relativePath
	 * @return
	 * @throws Exception
	 */
	protected String checkRelativePath(String _relativePath) throws Exception{
		
		// check first /
		// check last /
		
		if(_relativePath == null || _relativePath.equals("")){
			throw new Exception("Pathname is invalid.");
		}
		
		return _relativePath;
	}

	/**
	 * kick off the process.
	 */
	public void startProcess() {
		log.info("startProcess() START");
		start();
		log.info("startProcess() END");
	}

}
