/*
 * Copyright 2004-2006 National Institute of Informatics in Japan, 
 * All rights reserved.
 *
 * This file or a portion of this file is licensed under the terms of
 * the NAREGI Public License, found at http://www.naregi.org/download/index.html.
 * If you redistribute this file, with or without modifications, you must 
 * include this notice in the file.
 *
 * [History]
 * Date         Name                  Contents
 * ------------------------------------------------------------------------
 * 2006/06/13   Fujitsu)Yositake      First Version 
 */
 
package org.naregi.pse.acs.service.impl;

import java.io.File;

import org.apache.log4j.Logger;
import org.globus.wsrf.ResourceContext;
import org.globus.wsrf.impl.SimpleResourceKey;
import org.naregi.pse.acs.service.impl.functionbean.AADCheck;
import org.naregi.pse.acs.service.impl.functionbean.AcsFileManager;
import org.naregi.pse.acs.service.impl.functionbean.FileManager;

/**
 * Implementation of DestroyArchive operation.
 */
public class DestroyArchiveOperation {
	
	private static Logger log = Logger.getLogger(DestroyArchiveOperation.class.getName());
	
	private ApplicationArchiveResource aaResource;
	
	private FileManager fileUtil;
	
	public DestroyArchiveOperation(ApplicationArchiveResource _aaResource){
		
		this.aaResource = _aaResource;
		this.fileUtil = new AcsFileManager();
	}
	
	public void destroyArchive() throws Exception{

		log.info("DestroyArchive START");
				
		deleteFile();
		
		deleteRP();
		
		log.info("DestroyArchive END");
	}
	
	private void deleteFile() throws Exception{
		
		log.info("deleteFile START");
		
		if(this.aaResource.getAAD() != null){
			
			// get AA path in repository
			File aaStoredDir = ((AcsFileManager)this.fileUtil).getAAStoredDirFromAAID
					(new AADCheck().getRepoBaseDir(), this.aaResource.getAAD().getAAID());
	
			if(aaStoredDir.exists()){
				// delete
				((AcsFileManager)this.fileUtil).deleteAAFileAndDir(aaStoredDir);
			}

		}else{
			log.info("physical file not deleted");
		}
		
		log.info("deleteFile END");
	}
	
	private void deleteRP() throws Exception{

		log.info("deleteRP START");
		
		// delete resource properties
		ResourceContext ctx = ResourceContext.getResourceContext();
		ApplicationArchiveResourceHome home = (ApplicationArchiveResourceHome) ctx.getResourceHome();
		home.remove(new SimpleResourceKey(home.getKeyTypeName(), this.aaResource.getID()));
		
		log.info("deleteRP END");
	}
}
