/*
 * Copyright 2004-2006 National Institute of Informatics in Japan, 
 * All rights reserved.
 *
 * This file or a portion of this file is licensed under the terms of
 * the NAREGI Public License, found at http://www.naregi.org/download/index.html.
 * If you redistribute this file, with or without modifications, you must 
 * include this notice in the file.
 *
 * [History]
 * Date         Name                  Contents
 * ------------------------------------------------------------------------
 * 2006/06/11   Fujitsu)Yositake      First Version
 */

package org.naregi.pse.acs.service.impl;

import java.io.File;

import java.util.ArrayList;

import org.apache.axis.message.MessageElement;
import org.apache.axis.types.URI;
import org.apache.log4j.Logger;
import org.naregi.pse.acs.aaf.stubs.ContentType;
import org.naregi.pse.acs.aaf.stubs.DiffContentType;
import org.naregi.pse.acs.aaf.stubs.OperationType;
import org.naregi.pse.acs.ari.stubs.AAType;
import org.naregi.pse.acs.ari.stubs.AATypeBundle;
import org.naregi.pse.acs.ari.stubs.AATypeContent;
import org.naregi.pse.acs.ari.stubs.AATypeDescriptor;
import org.naregi.pse.acs.ari.stubs.GetArchive;
import org.naregi.pse.acs.ari.stubs.GetArchiveResponse;
import org.naregi.pse.acs.service.impl.constants.ACSConstants;
import org.naregi.pse.acs.service.impl.constants.ACSQNames;
import org.naregi.pse.acs.service.impl.functionbean.AADCheck;
import org.naregi.pse.acs.service.impl.functionbean.AADTransformerForJaxb;
import org.naregi.pse.acs.service.impl.functionbean.AcsFileManager;
import org.naregi.pse.acs.service.impl.functionbean.ZipFileManager;
import org.naregi.pse.acs.service.impl.property.ACSSystemInfo;

/**
 *  Implementation of GetArchive operation.
 */
public class GetArchiveOperation extends CommonOperation{

	private static Logger log = Logger.getLogger(GetArchiveOperation.class.getName());

	private ApplicationArchiveResource aaResource;
	
	private GetArchive request;
	
	private boolean diff;
	
	private int transType;
	
	private String gridFtpAddress;
	
	private String gridFtpPort;
	
	/**
	 * Constructor
	 * 
	 * @param _aaResource
	 * @param _request
	 * @throws Exception
	 */
	public GetArchiveOperation(ApplicationArchiveResource _aaResource, GetArchive _request) throws Exception{
		
		this.aaResource = _aaResource;
		this.request = _request;
		
		this.fileUtil = new AcsFileManager();
		
		this.gridFtpAddress = ACSSystemInfo.getInstance().getProperty(ACSConstants.ACS_SERVICE_GRIDFTP_ADDRESS_KEY);
		
		this.gridFtpPort = ACSSystemInfo.getInstance().getProperty(ACSConstants.ACS_SERVICE_GRIDFTP_PORT_KEY);
	}

	/**
	 * GetArchiveOperation
	 * @return
	 * @throws Exception
	 */
	public GetArchiveResponse getArchive() throws Exception{

		log.info("GetArchiveOperation START");
		
		// get full or differential
		if(this.request.isDifferential()){
			this.diff = true;
		}else{
			this.diff = false;
		}

		// create tmp dir
		makeTmpDir();

		// prepare files
		prepareFiles();
		
		// create response
		GetArchiveResponse res = createResponse();

		log.info("GetArchiveOperation END");
		
		return res;
	}
	
	private GetArchiveResponse createResponse() throws Exception{

		// create response message
		GetArchiveResponse res = new GetArchiveResponse();
		
		// get transport type
		int transType = checkTransportType(this.request.getTransportType());
		
		// get transport method
		int transMethod = checkTransportMethod(this.request.getTransportMethod());
		
		// create AAType
		// for descrete
		if(transType == ACSConstants.TRANS_TYPE_DESCRETE){
			
			// set
			res.setAA(getAATypeForDescrete(transMethod));
		
		// for bundled zip
		}else{
			
			// zip 
			makeBundledZip();
			
			// set
			res.setAA(getAATypeForBundledZip(transMethod));
		}
		
		return res;
	}

	private void makeBundledZip() throws Exception{

		// check already exists
		String zip = this.tmpDir.getPath() + "/" + ACSConstants.BUNDLED_FILENAME_ZIP;
		File zipFile = new File(zip);
		makeSureFileNotExist(zipFile);
		
		// add AAD to relativeFileList
		this.relativeFileList.add(ACSConstants.AAD_FILENAME);
		
		// get absolute from relative
		String[] absolute = getAbsoluteFromRelative(this.tmpDir);
		
		// zip
		((ZipFileManager)this.fileUtil).
			zipWithDir(zip, absolute, (String[])this.relativeFileList.toArray(new String[0]));		
	}

	private AAType getAATypeForDescrete(
		int transMethod)
		throws Exception {

		// create object
		AAType aaType = new AAType();

		aaType.setTransportType(new URI(ACSConstants.ST_TRANS_TYPE_DESCRETE));

		// set descriptor
		AATypeDescriptor desc = new AATypeDescriptor();
		if (transMethod == ACSConstants.TRANS_METHOD_EMBEDDED) {
			desc.setTransportMethod(new URI(ACSConstants.ST_TRANS_METHOD_EMBEDDED));
			desc.setEmbedded(this.fileUtil.readFileAsbyte(this.tmpDir.getPath() + "/" + ACSConstants.AAD_FILENAME));
		} else if (transMethod == ACSConstants.TRANS_METHOD_GRIDFTP) {
			desc.setTransportMethod(new URI(ACSConstants.ST_TRANS_METHOD_GRIDFTP));
			MessageElement[] any = new MessageElement[1];
			any[0] = new MessageElement(ACSQNames.URL);
			any[0].addTextNode("gsiftp://" + this.gridFtpAddress + ":" + this.gridFtpPort + this.tmpDir.getPath() + "/" + ACSConstants.AAD_FILENAME);
			desc.set_any(any);
		}
		aaType.setDescriptor(desc);

		// set content
		AATypeContent[] contents = new AATypeContent[this.relativeFileList.size()];
		
		for(int i = 0; i < contents.length; i++){
			AATypeContent content = new AATypeContent();
			
			// for Embedded
			if (transMethod == ACSConstants.TRANS_METHOD_EMBEDDED) {
				content.setTransportMethod(new URI(ACSConstants.ST_TRANS_METHOD_EMBEDDED));
				content.setEmbedded(this.fileUtil.readFileAsbyte(this.tmpDir.getPath() + "/" + (String)this.relativeFileList.get(i)));
				content.setPathname((String)this.relativeFileList.get(i));
			
			// for Gridftp
			}else if (transMethod == ACSConstants.TRANS_METHOD_GRIDFTP) {
				content.setTransportMethod(new URI(ACSConstants.ST_TRANS_METHOD_GRIDFTP));
				MessageElement[] any = new MessageElement[1];
				any[0] = new MessageElement(ACSQNames.URL);
				any[0].addTextNode("gsiftp://" + this.gridFtpAddress + ":" + this.gridFtpPort + this.tmpDir.getPath() + "/" + (String)this.relativeFileList.get(i));
				content.set_any(any);
				content.setPathname((String)this.relativeFileList.get(i));
			}
			
			// set content 
			contents[i] = content;
		
		}// end of for loop
		
		// set contents
		aaType.setContent(contents); 
		return aaType;
	}

	private AAType getAATypeForBundledZip(
		int transMethod)
		throws Exception {

		// create object
		AAType aaType = new AAType();

		// set Transport Type
		aaType.setTransportType(new URI(ACSConstants.ST_TRANS_TYPE_BUNDLED_ZIP));

		// set AATypeBundle
		AATypeBundle bundle = new AATypeBundle();
		if (transMethod == ACSConstants.TRANS_METHOD_EMBEDDED) {
			bundle.setTransportMethod(new URI(ACSConstants.ST_TRANS_METHOD_EMBEDDED));
			bundle.setEmbedded(this.fileUtil.readFileAsbyte(this.tmpDir.getPath() + "/" + ACSConstants.BUNDLED_FILENAME_ZIP));

		} else if (transMethod == ACSConstants.TRANS_METHOD_GRIDFTP) {
			bundle.setTransportMethod(new URI(ACSConstants.ST_TRANS_METHOD_GRIDFTP));
			MessageElement[] any = new MessageElement[1];
			any[0] = new MessageElement(ACSQNames.URL);
			any[0].addTextNode("gsiftp://" + this.gridFtpAddress + ":" + this.gridFtpPort + this.tmpDir.getPath() + "/" + ACSConstants.BUNDLED_FILENAME_ZIP);
			bundle.set_any(any);
		}
		aaType.setBundle(bundle);
		return aaType;
	}

	private void getFileList() throws Exception{
		
		this.relativeFileList = new ArrayList();
		
		if(this.diff){
			if(this.aaResource.getDiffAAD() == null){
				throw new Exception("Can't find DifferentialAAD. \n" +
					"This AA might not be Updated yet.");
			}
			
			getFileList(this.aaResource.getDiffAAD().getContents().getContent());
		}else{
			getFileList(this.aaResource.getAAD().getContents().getContent());
		}
	}
	
	private void getFileList(ContentType[] con){
	
		for(int i = 0; i < con.length; i++){
			this.relativeFileList.add(con[i].getPathname());
		}	
	}
	
	private void getFileList(DiffContentType[] con){
		
		for(int i = 0; i < con.length; i++){
			OperationType op = con[i].getOperation();
			if(OperationType._add.equals(op.getValue()) || 
						OperationType._replace.equals(op.getValue())){
				
				this.relativeFileList.add(con[i].getPathname());
			}
		}
	}

	private void createAAD() throws Exception{
		
		AADTransformerForJaxb trans = new AADTransformerForJaxb();
		
		String aad = null;
		
		// this is full
		if(this.diff == false){
			trans.gt4AAD2JaxbAAD(this.aaResource.getAAD());
			aad = trans.getJaxbAADAsString();
		
		// this is differential
		}else {
			trans.gt4DiffAAD2JaxbDiffAAD(this.aaResource.getDiffAAD());
			aad = trans.getJaxbDiffAADAsString();
		}
		
		// write to file 
		File aadFile = new File(this.tmpDir.getPath() + "/" + ACSConstants.AAD_FILENAME);
		fileUtil.writeFileFrombyte(aadFile, aad.getBytes());
	}

	/**
	 * Candidate of Common
	 * @throws Exception
	 */
	private void prepareFiles() throws Exception{
		
		// get AA path in repository
		File aaStoredDir = ((AcsFileManager)this.fileUtil).getAAStoredDirFromAAID
				(new AADCheck().getRepoBaseDir(), this.aaResource.getAAD().getAAID());
		
		// copy the AA to tmp
		
		// get AA file path
		File aaFile = new File(aaStoredDir.getPath() + "/" + ACSConstants.BUNDLED_FILENAME_ZIP);
	
		// copy AA file to tmp dir
		((AcsFileManager)this.fileUtil).copyBaseFileToTmpDir(aaFile, this.tmpDir);
		
		// unzip AA file in tmp dir
		((ZipFileManager)this.fileUtil).unZipWithDir
					(this.tmpDir, 
					new File(this.tmpDir.getPath() + "/" + ACSConstants.BUNDLED_FILENAME_ZIP));
		
		// get file list
		getFileList();
		
		// create AAD or DiffAAD file
		createAAD();
	}
}