/*
 * Copyright 2004-2006 National Institute of Informatics in Japan, 
 * All rights reserved.
 *
 * This file or a portion of this file is licensed under the terms of
 * the NAREGI Public License, found at http://www.naregi.org/download/index.html.
 * If you redistribute this file, with or without modifications, you must 
 * include this notice in the file.
 *
 * [History]
 * Date         Name                  Contents
 * ------------------------------------------------------------------------
 * 2006/06/09   Fujitsu)Yositake      First Version
 */

package org.naregi.pse.acs.service.impl;

import java.io.File;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;

import org.apache.axis.handlers.soap.SOAPService;
import org.apache.axis.types.URI.MalformedURIException;
import org.apache.log4j.Logger;

import org.globus.wsrf.NoResourceHomeException;
import org.globus.wsrf.ResourceContext;
import org.globus.wsrf.ResourceContextException;
import org.globus.wsrf.ResourceException;
import org.globus.wsrf.ResourceKey;
import org.globus.wsrf.container.ServiceHost;
import org.globus.wsrf.encoding.ObjectSerializer;

import org.naregi.pse.acs.aaf.stubs.AADType;
import org.naregi.pse.acs.aaf.stubs.DiffAADType;
import org.naregi.pse.acs.ari.stubs.AAInstanceStateType;
import org.naregi.pse.acs.ari.stubs.ApplicationArchiveUpdatedMessageType;
import org.naregi.pse.acs.ari.stubs.BaseVersionNotMatchFaultType;
import org.naregi.pse.acs.ari.stubs.IllegalDescriptorFaultType;
import org.naregi.pse.acs.ari.stubs.TransportMethodNotSupportedFaultType;
import org.naregi.pse.acs.ari.stubs.TransportTypeNotSupportedFaultType;
import org.naregi.pse.acs.ari.stubs.Update;
import org.naregi.pse.acs.ari.stubs.UpdateFailedFaultType;
import org.naregi.pse.acs.service.impl.constants.ACSConstants;
import org.naregi.pse.acs.service.impl.exception.BaseVersionNotMatchException;
import org.naregi.pse.acs.service.impl.exception.IllegalDescriptorException;
import org.naregi.pse.acs.service.impl.exception.TransportMethodNotSupportedException;
import org.naregi.pse.acs.service.impl.exception.TransportTypeNotSupportedException;
import org.naregi.pse.acs.service.impl.functionbean.AADCheck;
import org.naregi.pse.acs.service.impl.functionbean.AADTransformerForGt4;
import org.naregi.pse.acs.service.impl.functionbean.AADUpdater;
import org.naregi.pse.acs.service.impl.functionbean.AcsFileManager;

//import org.naregi.pse.acs.service.impl.functionbean.AddressingUtils;
import org.globus.wsrf.utils.AddressingUtils;

import org.naregi.pse.acs.service.impl.functionbean.FaultHelper;
import org.naregi.pse.acs.service.impl.functionbean.ZipFileManager;

//import org.w3.www._2005._03.addressing.EndpointReferenceType;
import org.apache.axis.message.addressing.EndpointReferenceType;

/**
 * As a thread to implement the Update operation.
 */
public class UpdateOperation extends CreateUpdateCommonOperation{

	private static Logger log =
		Logger.getLogger(UpdateOperation.class.getName());

	private Update request;

	private ApplicationArchiveResource baseResource;

	private ApplicationArchiveResource newResource;

	private EndpointReferenceType newEPR;
	
	private EndpointReferenceType baseEPR;
	
	// tmp dir for base
	private File baseTmpDir;

	/**
	 * Constructor
	 * 
	 * @param _request
	 * @param _rCtx
	 * @param _soapService
	 * @param _newKey
	 * @param _newEPR
	 * @throws Exception
	 */
	public UpdateOperation(
		Update _request,
		ResourceContext _rCtx,
		SOAPService _soapService,
		ResourceKey _newKey,
		EndpointReferenceType _newEPR)
		throws Exception {

		log.info("UpdateOperation constructor START");

		this.request = _request;
		this.newEPR = _newEPR;

		// set Base resource
		setBaseResource(_rCtx);
		
		// set Base epr
		setBaseEPR(_rCtx, _soapService);

		// set New resource
		setNewResource(_rCtx, _newKey);

		this.fileUtil = new AcsFileManager();

		this.relativeFileList = new ArrayList();

		log.info("UpdateOperation constructor END");
	}

	private void setBaseResource(ResourceContext _rCtx)
		throws ResourceContextException, ResourceException {

		this.baseResource = (ApplicationArchiveResource) _rCtx.getResource();
	}

	private void setNewResource(ResourceContext _rCtx, ResourceKey _key)
		throws NoResourceHomeException, ResourceException, ResourceContextException {

		this.newResource = (ApplicationArchiveResource)
				((ApplicationArchiveResourceHome) _rCtx.getResourceHome()).find(_key);
	}

	private void setBaseEPR(ResourceContext _rCtx, SOAPService _soapService) throws Exception{
		
		// get ResourceKey for base
		ResourceKey key = _rCtx.getResourceKey();

		// get URI of AA service
		URL baseURL = ServiceHost.getBaseURL();
		String instanceService = (String) _soapService.getOption("instance");
		String instanceURI = baseURL.toString() + instanceService;
		
		// create epr of Base
		EndpointReferenceType eprAA = AddressingUtils.createEndpointReference(instanceURI, key);
					
		this.baseEPR = eprAA;		
	}

	/**
	 * main process for Update
	 */
	private void process() throws Throwable {
		log.info("Process for Update START");

		try{
			// store files to tmp dir
			collectFiles();			
		}catch(Throwable t){
			throw t;
		}finally{
			if(this.fileTransfer != null){
				this.fileTransfer.closeConnection();
			}
		}

		// check AAD
		checkAAD();

		// update
		updateFiles();

		// store Repository from tmp
		storeRepository();

		// update resource property
		updateAARP();

		log.info("process for Update END");
	}

	/**
	 * @throws Exception
	 */
	private void collectFiles() throws Exception {
		
		log.info("collectFiles START");
		
		// make tmpDir & set to property
		makeTmpDir();
		
		// get transport type
		int transType = checkTransportType(this.request.getAA().getTransportType());
		
		// for descrete
		if(transType == ACSConstants.TRANS_TYPE_DESCRETE){

			collectDescreteFiles(this.request.getAA());
		
		// for bundled/zip
		}else if(transType == ACSConstants.TRANS_TYPE_BUNDLED_ZIP){
			
			collectBundledZipFile(this.request.getAA().getBundle());
		}
		
		log.info("collectFiles END");
	}

	private void checkAAD() throws Exception {

		log.info("checkAAD START");
		
		this.aadCheck = new AADCheck();
		
		// check grammer and file structure
		this.aadCheck.checkDiffAAD(this.aad.getPath(), this.relativeFileList);
		
		// check base = this
		checkBaseMatchThis();
		
		
		log.info("checkAAD END");
	}

	private void checkBaseMatchThis() throws BaseVersionNotMatchException{
		
		log.info("checkBaseMatchThis START");
		
		// check name
		String diffName = this.aadCheck.getJaxbDiffAAD().getAAID().getName();
		String thisName = this.baseResource.getAAD().getAAID().getName().toString();
		
		if(diffName.equalsIgnoreCase(thisName) == false){
			throw new BaseVersionNotMatchException
						("Update must be performed to same AA name. \n" +
						"AA name for this resource = " + thisName + "\n" +
						"AA name from DiffAAD = " + diffName);
		}
		
		// check version
		String baseVersion = this.aadCheck.getJaxbDiffAAD().getAAID().getBaseVersion();
		String thisVersion = this.baseResource.getAAD().getAAID().getVersion();		

		if(baseVersion.equalsIgnoreCase(thisVersion) == false){
			throw new BaseVersionNotMatchException
					("Base version specified in DiffAADUpdate and \n" +
					 "vesion of this resource's AA must be same. \n" +
					 "Version for this resource = " + thisVersion + "\n" +
					 "Base version from DiffAAD = " + baseVersion);
		}
		
		log.info("checkBaseMatchThis END");
	}

	private void updateFiles() throws Exception{
		
		log.info("updateFiles START");
		
		// get tmpdir for base
		this.baseTmpDir = new File(this.tmpDir.getPath() + "/base_" + new Date().getTime());
		
		// get base file dir location
		File baseDir = 
			((AcsFileManager)this.fileUtil).getBaseAAStoredDirFromDiffAAID
						(this.aadCheck.getRepoBaseDir(),
						 this.aadCheck.getJaxbDiffAAD().getAAID());

		// get base file
		File baseFile = new File(baseDir.getPath() + "/" + ACSConstants.BUNDLED_FILENAME_ZIP);
	
		// copy base file to tmp dir
		((AcsFileManager)this.fileUtil).copyBaseFileToTmpDir(baseFile, this.baseTmpDir);
		
		// unzip base file in tmp dir
		((ZipFileManager)this.fileUtil).unZipWithDir
					(this.baseTmpDir, 
					new File(this.baseTmpDir.getPath() + "/" + ACSConstants.BUNDLED_FILENAME_ZIP));
		
		// update file, relativeFileList from differential to differentail + base
		this.relativeFileList = ((AcsFileManager)this.fileUtil).fileOperationForUpdate
								(this.baseResource.getAAD(),
								 this.aadCheck.getJaxbDiffAAD(),
								 this.baseTmpDir,
								 this.tmpDir);
		
		log.info("updateFiles END");
	}

	private void storeRepository() throws Exception{
		
		log.info("storeRepository START");
		
		// check empty AA
		if(this.relativeFileList.isEmpty()){
			throw new Exception("After this Update, no contents for new AA. \n" +
				"Implementation does not accept AA without any contents. \n" +
				"Destroy or add contents.");
		}
		
		// check already exists
		String zip = this.baseTmpDir.getPath() + "/" + ACSConstants.BUNDLED_FILENAME_ZIP;
		File zipFile = new File(zip);
		makeSureFileNotExist(zipFile);
		
		// get absolute from relative
		String[] absolute = getAbsoluteFromRelative(this.baseTmpDir);
		
		// zip
		((ZipFileManager)this.fileUtil).
			zipWithDir(zip, absolute, (String[])this.relativeFileList.toArray(new String[0]));
		
		// get repository location
		File aaStoredDir = ((AcsFileManager)this.fileUtil).
			getAAStoredDirFromAAID(this.aadCheck.getRepoBaseDir(), this.aadCheck.getJaxbDiffAAD().getAAID());

		// check AAID duplication
		if(new File(aaStoredDir.getPath() + "/" + ACSConstants.BUNDLED_FILENAME_ZIP).exists()){
			throw new Exception("The specified AAID already exists. \n " +
				"AAID can't be duplicated. Specify other AAID. \n" + 
				"name = " + this.aadCheck.getJaxbDiffAAD().getAAID().getName() + "\n" + 
				"version = " + this.aadCheck.getJaxbDiffAAD().getAAID().getVersion());
		}
		
		// move to repository
		((AcsFileManager)this.fileUtil).moveTmpFileToAAStoredDir(zipFile, aaStoredDir);
			
		log.info("storeRepository END");
	}

	private void updateAARP() throws Throwable {

		log.info("updateRP START");

		// get DifferentialAAD of gt4
		AADTransformerForGt4 trans = new AADTransformerForGt4();
		trans.jaxbDiffAAD2Gt4DiffAAD(this.aadCheck.getJaxbDiffAAD());
		DiffAADType gt4DiffAAD = trans.getGt4DiffAAD();

		// for new Resource
		
		// set state
		this.newResource.setState(AAInstanceStateType.value2);

		// set AAD
		AADType newAAD = updateAAD(gt4DiffAAD);
		this.newResource.setAAD(newAAD);

		// set DiffAAD
		this.newResource.setDiffAAD(gt4DiffAAD);

		// set creationDateTime 
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy MM/dd HH:mm:ss");
		String creationDateTime = sdf.format(new Date());
		this.newResource.setCreationDateTime(creationDateTime);	

		// set baseAA
		this.newResource.setBaseAA(this.baseEPR);

		// for base Resource
		
		// add newerAA
		this.baseResource.addNewerAA(this.newEPR);
		
		// update UpdatedMessage
		updateUpdatedMessage(creationDateTime, newAAD, this.baseResource.getAAD(), gt4DiffAAD);

		log.info("updateRP END");
	}

	private AADType updateAAD(DiffAADType gt4DiffAAD) throws MalformedURIException{
		
		AADUpdater updater = new AADUpdater(this.baseResource.getAAD(), gt4DiffAAD);
		
		return updater.updateAAD();
	}

	private void updateUpdatedMessage(
		String creationDateTime, AADType newAAD, AADType baseAAD,
		DiffAADType diffAAD) throws ResourceException {
		
		log.info("updateUpdatedMessage START");
		
		// create object
		ApplicationArchiveUpdatedMessageType updated = 
				new ApplicationArchiveUpdatedMessageType();
		
		// set creationDateTime
		updated.setDateTime(Calendar.getInstance());
		
		// set newAAID
		updated.setAAIDNew(newAAD.getAAID());
		
		// set oldAAID
		updated.setAAIDOld(baseAAD.getAAID());
		
		// set newEPR
		updated.setArchiveEPRNew(this.newEPR);
		
		// set oldEPR
		updated.setArchiveEPROld(this.baseEPR);
		
		// set diffAAD
		updated.setDifferentialAAD(diffAAD);
		
		// set to Base Resource & shot notification
		this.baseResource.setApplicationArchiveUpdated(updated);
		
		log.info("updateUpdatedMessage END");
	}

	/**
	 * The handler to catch exception while processing.
	 */
	public void errorHandler() throws Exception {

		try {
			process();

		}catch(TransportTypeNotSupportedException e){
			log.fatal("Failed to update. " +
			"Check supported transport type by GetResourceProperty for ApplicationRepository.", e);
			FaultHelper fh = new FaultHelper(new TransportTypeNotSupportedFaultType(), true);
			fh.addDescription("Failed to update.");
			fh.addDescription("Check supported transport type by GetResourceProperty for ApplicationRepository.");
			fh.addFaultCause(e);		
			TransportTypeNotSupportedFaultType f = (TransportTypeNotSupportedFaultType)fh.getFault();
			this.baseResource.setFaultMessage(ObjectSerializer.toElement(f, 
						TransportTypeNotSupportedFaultType.getTypeDesc().getXmlType())); 
			setToFailedState();
			
		}catch(TransportMethodNotSupportedException e){
			log.fatal("Failed to update. " +
			"Check supported transport method by GetResourceProperty for ApplicationRepository.", e);
			FaultHelper fh = new FaultHelper(new TransportMethodNotSupportedFaultType(), true);
			fh.addDescription("Failed to update.");
			fh.addDescription("Check supported transport method by GetResourceProperty for ApplicationRepository.");
			fh.addFaultCause(e);		
			TransportMethodNotSupportedFaultType f = (TransportMethodNotSupportedFaultType)fh.getFault();
			this.baseResource.setFaultMessage(ObjectSerializer.toElement(f, 
					TransportMethodNotSupportedFaultType.getTypeDesc().getXmlType()));		
			setToFailedState();
		
		}catch(IllegalDescriptorException e){
			log.fatal("Failed to update. " +
			"AAD is illegal.", e);
			FaultHelper fh = new FaultHelper(new IllegalDescriptorFaultType(), true);
			fh.addDescription("Failed to update.");
			fh.addDescription("AAD is illegal.");
			fh.addFaultCause(e);
			IllegalDescriptorFaultType f = (IllegalDescriptorFaultType)fh.getFault();
			this.baseResource.setFaultMessage(ObjectSerializer.toElement(f, 
					IllegalDescriptorFaultType.getTypeDesc().getXmlType()));
			setToFailedState();

		}catch(BaseVersionNotMatchException e){
			log.fatal("Failed to update. Base version must be this version.", e);
			FaultHelper fh = new FaultHelper(new BaseVersionNotMatchFaultType(), true);
			fh.addDescription("Failed to update.");
			fh.addDescription("Base version must be this version.");
			fh.addFaultCause(e);
			BaseVersionNotMatchFaultType f = (BaseVersionNotMatchFaultType)fh.getFault();
			this.baseResource.setFaultMessage(ObjectSerializer.toElement(f, 
					BaseVersionNotMatchFaultType.getTypeDesc().getXmlType()));
			setToFailedState();

		}catch(Throwable t){
			log.fatal("Failed to update.", t);
			FaultHelper fh = new FaultHelper(new UpdateFailedFaultType(), true);
			fh.addDescription("Failed to update.");
			fh.addFaultCause(t);		
			UpdateFailedFaultType f = (UpdateFailedFaultType)fh.getFault();
			this.baseResource.setFaultMessage(ObjectSerializer.toElement(f, 
					UpdateFailedFaultType.getTypeDesc().getXmlType()));
			setToFailedState();
		}
	}

	private void setToFailedState() throws ResourceException{
		this.newResource.setState(AAInstanceStateType.value3);
	}

	/* 
	 * @see java.lang.Runnable#run()
	 */
	public void run() {

		try {
			errorHandler();
		} catch (Exception e) {
			log.fatal("Failed.", e);
		}
	}
}
