/*
 * Copyright 2004-2006 National Institute of Informatics in Japan, 
 * All rights reserved.
 *
 * This file or a portion of this file is licensed under the terms of
 * the NAREGI Public License, found at http://www.naregi.org/download/index.html.
 * If you redistribute this file, with or without modifications, you must 
 * include this notice in the file.
 *
 * [History]
 * Date         Name                  Contents
 * ------------------------------------------------------------------------
 * 2005/09/12   Fujitsu)Yositake      First Version
 * 2005/04/20	Fujitsu)Yositake	  ACS Unit Customize add getProxyCert()
 */

package org.naregi.pse.acs.service.impl.filetransfer;

import org.apache.log4j.Logger;
import org.globus.ftp.GridFTPClient;
import org.globus.gsi.gssapi.auth.IdentityAuthorization;
import org.globus.myproxy.MyProxy;
import org.globus.myproxy.MyProxyException;
import org.gridforum.jgss.ExtendedGSSManager;
import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSException;
import org.ietf.jgss.GSSManager;
import org.naregi.pse.acs.service.impl.databean.MyProxyServerInfo;
import org.naregi.pse.acs.service.impl.databean.UserInformation;
import org.naregi.pse.acs.service.impl.property.ACSSystemInfo;

/**
 * the class provide method for access the gridftp server. 
 *
 */
public class CogJGlobusAccess {

	private static Logger log = Logger.getLogger(CogJGlobusAccess.class.getName());
	
	private static final String MYPROXY_ADDRESS_KEY = "MYPROXY_ADDRESS";
	
	private static final String MYPROXY_PORT_KEY = "MYPROXY_PORT";
	
	private static final String MYPROXY_SUBJECT_KEY = "MYPROXY_SUBJECT";
	
	private static final String MYPROXY_LIFETIME_BASE_KEY = "MYPROXY_LIFETIME_BASE";
	
	private static final String MYPROXY_LIFETIME_MULTIPLIER_KEY = "MYPROXY_LIFETIME_MULTIPLIER";


	/**
	 * create a MyProxyServerInfo object.
	 * @return
	 */
	public MyProxyServerInfo createMyProxyInfo() throws Exception{

		// get from Property File
		MyProxyServerInfo myProxyInfo = new MyProxyServerInfo();
		
		myProxyInfo.setMyProxyURL(ACSSystemInfo.getInstance().getProperty(MYPROXY_ADDRESS_KEY));
		
		myProxyInfo.setMyProxyPort(ACSSystemInfo.getInstance().getProperty(MYPROXY_PORT_KEY));
		
		myProxyInfo.setMyProxyHostSubject(ACSSystemInfo.getInstance().getProperty(MYPROXY_SUBJECT_KEY));
		
		// get life time
		int base = Integer.parseInt(ACSSystemInfo.getInstance().getProperty(MYPROXY_LIFETIME_BASE_KEY));
		int multiplier = Integer.parseInt(ACSSystemInfo.getInstance().getProperty(MYPROXY_LIFETIME_MULTIPLIER_KEY));	
		
		myProxyInfo.setLifetimeAfterGet(base * multiplier);

		log.info("createMyProxyInfo END");
		return myProxyInfo;
	}

	/**
	 * get a GSSCredential object.
	 * @param userInfo
	 * @param myProxyInfo
	 * @return
	 * @throws MyProxyException
	 */
	public GSSCredential getProxyCert(UserInformation userInfo,
			MyProxyServerInfo myProxyInfo) throws MyProxyException {

		log.info("getting proxy certification......");

		MyProxy myProxy = new MyProxy();
		GSSCredential cred = null;

		// set parameter for accessing MyProxy
		myProxy.setHost(myProxyInfo.getMyProxyURL());
		myProxy.setPort(Integer.parseInt(myProxyInfo.getMyProxyPort()));
		myProxy.setAuthorization(new IdentityAuthorization(myProxyInfo.getMyProxyHostSubject()));

		// get Proxy Certification from MyProxy
		try {
			cred = myProxy.get(null, userInfo.getUserName(), userInfo
					.getPassPhrase(), myProxyInfo.getLifetimeAfterGet());
		} catch (MyProxyException e) {
			throw e;
		}
		log.info("success get proxy certification");
		return cred;
	}

//	2006/04/20 START
	public GSSCredential getProxyCert() throws GSSException{
	
		log.info("getting grid proxy certification......");
       
		// use default credential
		GSSManager manager = ExtendedGSSManager.getInstance();
		GSSCredential cred = manager.createCredential(GSSCredential.INITIATE_AND_ACCEPT);
	   
		log.info("success get grid proxy certification");
		return cred;
	}
//	2006/04/20 END

	/**
	 * connect the gridftp server.
	 * @param address
	 * @param port
	 * @param subject
	 * @param cred
	 * @return
	 * @throws Exception
	 */
	public GridFTPClient connect(String address, String port, String subject,
			GSSCredential cred) throws Exception {

		log.info("connecting GridFTP Server");
		log.info("Address = " + address);
		log.info("Port = " + port);

		GridFTPClient connection = null;
		try {
			connection = new GridFTPClient(address, Integer.parseInt(port));
	
			// does not set host subject
			//connection.setAuthorization(new IdentityAuthorization(subject));
	
			connection.authenticate(cred);
		} catch (Exception e) {
			throw e;
		} 

		log.info("success connect GridFTP Server");
				
		return connection;
	}
}
