/*
 * Copyright 2004-2006 National Institute of Informatics in Japan, 
 * All rights reserved.
 *
 * This file or a portion of this file is licensed under the terms of
 * the NAREGI Public License, found at http://www.naregi.org/download/index.html.
 * If you redistribute this file, with or without modifications, you must 
 * include this notice in the file.
 *
 * [History]
 * Date         Name                  Contents
 * ------------------------------------------------------------------------
 * 2006/06/19   Fujitsu)Yositake      First Version
 */

package org.naregi.pse.acs.service.impl.filetransfer;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;

import org.apache.log4j.Logger;
import org.globus.ftp.GridFTPClient;
import org.globus.ftp.exception.ServerException;
import org.ietf.jgss.GSSCredential;
import org.naregi.pse.acs.service.impl.databean.UrlInfo;
import org.naregi.pse.acs.service.impl.databean.UserInformation;

/**
 * @author Yositake
 *
 */
public class FileTransfer {

	private static Logger log = Logger.getLogger(FileTransfer.class.getName());
	private HashMap con;

	public FileTransfer(){
		this.con = new HashMap();
	}

	public void closeConnection() throws ServerException, IOException{
		
		Collection values = this.con.values();
		Iterator it = values.iterator();
		GridFTPClient gftp = null;
		while(it.hasNext()){
			((GridFTPClient)it.next()).close();
		}
		log.info("Success close all connection.");
	}

	/**
	 * it get information from an ACS server.
	 * 
	 * @param fileLocation
	 * @param userInfo - Clients' info that are accessing ACS
	 * @param acsFileServerInfo
	 * @param tempLocation
	 * @throws Exception
	 */
	public void getFromAcsClientToAcsServer(
		UserInformation uInfo,
		UrlInfo urlInfo,
		String tempLocation)
		throws Exception {

		log.info("(remote)fileLocation = " + urlInfo.getFile());
		log.info("(local)fileLocation = " + tempLocation);
		log.info("GRIDFTP server address = " + urlInfo.getAddress());
		log.info("GRIDFTP server port = " + urlInfo.getPort());

		File tempFile = new File(tempLocation);
		
		if(tempFile.exists() == false){
			boolean check = tempFile.createNewFile();
		
			if(check == false){
				throw new Exception();
			}
		}
		
		GridFTPClient connection = getConnection(urlInfo.getAddress() + ":" + urlInfo.getPort());
		
		if(connection == null){
		
			// create instance
			CogJGlobusAccess cog = new CogJGlobusAccess();
			
			// load infomation for accessing to MyProxy
			// MyProxyServerInfo myProxyInfo = cog.createMyProxyInfo();
	
			// get Proxy Certification from MyProxy
			GSSCredential cred = null;
			//cred = cog.getProxyCert(uInfo, myProxyInfo);
			cred = cog.getProxyCert();

			// get connection with GridFTPServer specifies by ACSFileServerInfo
			connection = cog.connect(
							urlInfo.getAddress(),
							urlInfo.getPort(),
							urlInfo.getHostSubject(),
							cred);
							
			// save connection
			this.con.put(urlInfo.getAddress() + ":" + urlInfo.getPort(), connection);
		
		}else{
			// for second get
			connection.setPassiveMode(true);
		}
		
		// get file from connection
		log.info("getting file.......");
		log.info("current directory = " + connection.getCurrentDir());
	
		// set transfer mode to TYPE_IMAGE
		connection.setType(1);
		connection.get(urlInfo.getFile(), tempFile);
		log.info("success get file");
	}
	
	private GridFTPClient getConnection(String key){
		
		GridFTPClient client = (GridFTPClient)this.con.get(key);
		return client;
	}
	
}
