/*
 * Copyright 2004-2006 National Institute of Informatics in Japan, 
 * All rights reserved.
 *
 * This file or a portion of this file is licensed under the terms of
 * the NAREGI Public License, found at http://www.naregi.org/download/index.html.
 * If you redistribute this file, with or without modifications, you must 
 * include this notice in the file.
 *
 * [History]
 * Date         Name                  Contents
 * ------------------------------------------------------------------------
 * 2006/06/07   Fujitsu)Yositake      First Version 
 */

package org.naregi.pse.acs.service.impl.functionbean;

import java.io.ByteArrayInputStream;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.namespace.QName;

import org.apache.axis.types.URI;
import org.apache.axis.types.URI.MalformedURIException;
import org.apache.axis.utils.ByteArrayOutputStream;
import org.apache.log4j.Logger;
import org.globus.wsrf.encoding.ObjectSerializer;
import org.globus.wsrf.encoding.SerializationException;
import org.naregi.pse.acs.service.impl.constants.ACSQNames;
import org.naregi.pse.acs.service.impl.jaxb.ObjectFactory;
import org.xml.sax.InputSource;


public class AADTransformerForGt4 {

	private static Logger log = Logger.getLogger(AADTransformerForGt4.class.getName());

	private org.naregi.pse.acs.aaf.stubs.AADType gt4AAD;
	private org.naregi.pse.acs.aaf.stubs.DiffAADType gt4DiffAAD;
	
	public AADTransformerForGt4(){
	
	}
		
	public void jaxbAAD2Gt4AAD(org.naregi.pse.acs.service.impl.jaxb.AAD _jaxbAAD) throws Exception{

		// create AAD object
		this.gt4AAD = new org.naregi.pse.acs.aaf.stubs.AADType();
		
		// AAID
		org.naregi.pse.acs.aaf.stubs.AAIDType gt4AAID = copyAAID(_jaxbAAD.getAAID());
		this.gt4AAD.setAAID(gt4AAID);
		
		// Author
		org.naregi.pse.acs.aaf.stubs.AuthorType gt4Author = copyAuthor(_jaxbAAD.getAuthor());
		this.gt4AAD.setAuthor(gt4Author);
		
		// Descriptions
		if(_jaxbAAD.getDescriptions() != null){
			org.naregi.pse.acs.aaf.stubs.DescriptionsType gt4Descs = copyDescriptions(_jaxbAAD.getDescriptions());
			this.gt4AAD.setDescriptions(gt4Descs);			
		}
		
		// Contents
		org.naregi.pse.acs.aaf.stubs.ContentsType gt4Contents = copyContents(_jaxbAAD.getContents());
		this.gt4AAD.setContents(gt4Contents);
	}

	public void jaxbDiffAAD2Gt4DiffAAD(org.naregi.pse.acs.service.impl.jaxb.DifferentialAAD _jaxbDiffAAD) throws Throwable{

		// create DiffAAD
		this.gt4DiffAAD = new org.naregi.pse.acs.aaf.stubs.DiffAADType();
		
		// DiffAAID
		org.naregi.pse.acs.aaf.stubs.DiffAAIDType gt4DiffAAID = copyDiffAAID(_jaxbDiffAAD.getAAID());
		this.gt4DiffAAD.setAAID(gt4DiffAAID);
		
		// Author
		if(_jaxbDiffAAD.getAuthor() != null){
			org.naregi.pse.acs.aaf.stubs.AuthorType gt4Author = copyAuthor(_jaxbDiffAAD.getAuthor());
			this.gt4DiffAAD.setAuthor(gt4Author);
		}	
		
		// Desscriptions
		if(_jaxbDiffAAD.getDescriptions() != null){
			org.naregi.pse.acs.aaf.stubs.DescriptionsType gt4Descs = copyDescriptions(_jaxbDiffAAD.getDescriptions());
			this.gt4DiffAAD.setDescriptions(gt4Descs);			
		}
		
		// DiffContents
		org.naregi.pse.acs.aaf.stubs.DiffContentsType gt4DiffContents = copyDiffContents(_jaxbDiffAAD.getContents());
		this.gt4DiffAAD.setContents(gt4DiffContents);			
	}

	private org.naregi.pse.acs.aaf.stubs.AAIDType copyAAID(org.naregi.pse.acs.service.impl.jaxb.AAIDType _jaxbAAID) throws Exception{
		
		// create AAIDType
		org.naregi.pse.acs.aaf.stubs.AAIDType gt4AAID = new org.naregi.pse.acs.aaf.stubs.AAIDType();
		
		// name
		try {
			gt4AAID.setName(new URI(_jaxbAAID.getName()));
		} catch (MalformedURIException e) {
			log.fatal("local error", e);
			if(e.getMessage() != null){
				throw new Exception("/AAD/AAID/Name must be URI. " + e.getMessage());
			}else{
				throw new Exception("/AAD/AAID/Name must be URI.");
			}
		}
		
		// version
		gt4AAID.setVersion(_jaxbAAID.getVersion());
		
		return gt4AAID;
	}

	private org.naregi.pse.acs.aaf.stubs.DiffAAIDType copyDiffAAID(org.naregi.pse.acs.service.impl.jaxb.DiffAAIDType _jaxbDiffAAID) throws Exception{
		
		// create DiffAAIDType
		org.naregi.pse.acs.aaf.stubs.DiffAAIDType gt4DiffAAID = new org.naregi.pse.acs.aaf.stubs.DiffAAIDType();
		
		// name
		try {
			gt4DiffAAID.setName(new URI(_jaxbDiffAAID.getName()));
		} catch (MalformedURIException e) {
			log.fatal("local error", e);
			if(e.getMessage() != null){
				throw new Exception("/DifferentialAAD/AAID/Name must be URI. " + e.getMessage());
			}else{
				throw new Exception("/DifferentialAAD/AAID/Name must be URI.");
			}
		}
		
		// version
		gt4DiffAAID.setVersion(_jaxbDiffAAID.getVersion());
		
		// base version
		gt4DiffAAID.setBaseVersion(_jaxbDiffAAID.getBaseVersion());
		
		return gt4DiffAAID;
	}

	private org.naregi.pse.acs.aaf.stubs.AuthorType copyAuthor(org.naregi.pse.acs.service.impl.jaxb.AuthorType _jaxbAuthor){
		
		// create AuthorType
		org.naregi.pse.acs.aaf.stubs.AuthorType gt4Author = new org.naregi.pse.acs.aaf.stubs.AuthorType();
		
		// name
		gt4Author.setName(_jaxbAuthor.getName());
		
		// description
		List list = _jaxbAuthor.getDescription();
		
		if(list != null){
			
			// create container
			org.naregi.pse.acs.aaf.stubs.DescriptionType[] gt4DescArray = 
						new org.naregi.pse.acs.aaf.stubs.DescriptionType[list.size()];
			gt4Author.setDescription(gt4DescArray);
				
			for(int i = 0; i < list.size(); i++){
			
				// create DescriptionType
				org.naregi.pse.acs.aaf.stubs.DescriptionType gt4DescType = 
						new org.naregi.pse.acs.aaf.stubs.DescriptionType();
			
				// get jaxb value
				String value = ((org.naregi.pse.acs.service.impl.jaxb.DescriptionType)list.get(i)).getValue();
				
				// set to gt4
				gt4DescType.set_value(value);
				gt4Author.setDescription(i, gt4DescType);
			}
		}
		
		// location
		org.naregi.pse.acs.service.impl.jaxb.AuthorType.LocationType jaxbLoc = _jaxbAuthor.getLocation();
		
		if(jaxbLoc != null){
			
			org.naregi.pse.acs.aaf.stubs.AuthorTypeLocation gt4Loc = 
					new org.naregi.pse.acs.aaf.stubs.AuthorTypeLocation();

			if(jaxbLoc.getCountry() != null){
				
				// country
				gt4Loc.setCountry(jaxbLoc.getCountry());				
			}

			if(jaxbLoc.getAddress() != null){

				// address
				gt4Loc.setAddress(jaxbLoc.getAddress());				
			}
			
			gt4Author.setLocation(gt4Loc);
		}
		return gt4Author;
	}

	private org.naregi.pse.acs.aaf.stubs.DescriptionsType copyDescriptions(org.naregi.pse.acs.service.impl.jaxb.DescriptionsType _jaxbDescriptions){
		
		// create Descriptions
		org.naregi.pse.acs.aaf.stubs.DescriptionsType gt4Descs = new org.naregi.pse.acs.aaf.stubs.DescriptionsType();
		
		// description
		List list = _jaxbDescriptions.getDescription();
		
		if(list != null){
			
			// create container
			org.naregi.pse.acs.aaf.stubs.DescriptionType[] gt4DescArray = 
						new org.naregi.pse.acs.aaf.stubs.DescriptionType[list.size()];
			gt4Descs.setDescription(gt4DescArray);			
			
			for(int i = 0; i < list.size(); i++){
			
				// create DescriptionType
				org.naregi.pse.acs.aaf.stubs.DescriptionType gt4Desc = new org.naregi.pse.acs.aaf.stubs.DescriptionType();
			
				// get jaxb value
				String value = ((org.naregi.pse.acs.service.impl.jaxb.DescriptionType)list.get(i)).getValue();
			
				// set to gt4
				gt4Desc.set_value(value);
				gt4Descs.setDescription(i, gt4Desc);
			}			
		}
		return gt4Descs;
	}

	private org.naregi.pse.acs.aaf.stubs.ContentsType copyContents(org.naregi.pse.acs.service.impl.jaxb.ContentsType _jaxbContents){
		
		// create ContentsType
		org.naregi.pse.acs.aaf.stubs.ContentsType gt4Contents = new org.naregi.pse.acs.aaf.stubs.ContentsType();
		
		// contents
		List list = _jaxbContents.getContent();
		if(list != null){
			
			// create container
			org.naregi.pse.acs.aaf.stubs.ContentType[] gt4ContentArray = 
					new org.naregi.pse.acs.aaf.stubs.ContentType[list.size()];
			gt4Contents.setContent(gt4ContentArray);
			
			for(int i = 0; i < list.size(); i++){
			
				org.naregi.pse.acs.aaf.stubs.ContentType gt4Content = 
								copyContent((org.naregi.pse.acs.service.impl.jaxb.ContentType)list.get(i));

				gt4Contents.setContent(i, gt4Content);
			}			
		}
		return gt4Contents;
	}

	private org.naregi.pse.acs.aaf.stubs.ContentType copyContent(org.naregi.pse.acs.service.impl.jaxb.ContentType jaxbContent){

		// create ContentType
		org.naregi.pse.acs.aaf.stubs.ContentType gt4Content = new org.naregi.pse.acs.aaf.stubs.ContentType();
			
		// get jaxb pathName
		String pathName = jaxbContent.getPathname();
			
		// get jaxb type
		QName type = jaxbContent.getType();
			
		// set to gt4
		if(type != null){
			gt4Content.setType(type);
			
			log.info("test " + gt4Content.getType().getPrefix());
		}
		
		gt4Content.setPathname(pathName);
		
		gt4Content.setDigestInfoGroup(null);
		
		return gt4Content;
	}

	private org.naregi.pse.acs.aaf.stubs.DiffContentsType copyDiffContents(org.naregi.pse.acs.service.impl.jaxb.DiffContentsType _jaxbDiffContents){
		
		// create ContentsType
		org.naregi.pse.acs.aaf.stubs.DiffContentsType gt4DiffContents = 
								new org.naregi.pse.acs.aaf.stubs.DiffContentsType();
		
		// contents
		List list = _jaxbDiffContents.getContent();
		if(list != null){
			
			// create container
			org.naregi.pse.acs.aaf.stubs.DiffContentType[] gt4DiffContentArray = 
					new org.naregi.pse.acs.aaf.stubs.DiffContentType[list.size()];
			gt4DiffContents.setContent(gt4DiffContentArray);
			
			for(int i = 0; i < list.size(); i++){

				// copy DiffContentType
				org.naregi.pse.acs.aaf.stubs.DiffContentType gt4DiffContent = 
					copyDiffContent((org.naregi.pse.acs.service.impl.jaxb.DiffContentType)list.get(i));

				// set to DiffContents
				gt4DiffContents.setContent(i, gt4DiffContent);
			}			
		}
		return gt4DiffContents;
	}

	private org.naregi.pse.acs.aaf.stubs.DiffContentType copyDiffContent(org.naregi.pse.acs.service.impl.jaxb.DiffContentType jaxbDiffContent){

		// create DiffContentType
		org.naregi.pse.acs.aaf.stubs.DiffContentType gt4DiffContent =
						new org.naregi.pse.acs.aaf.stubs.DiffContentType();
			
		// get jaxb pathName
		String pathName = jaxbDiffContent.getPathname();
			
		// set to gt4
		gt4DiffContent.setPathname(pathName);

		// get jaxb type
		QName type = jaxbDiffContent.getType();
			
		// set to gt4
		if(type != null){
			gt4DiffContent.setType(type);
		}
		
		// get jaxb operation
		String ope = jaxbDiffContent.getOperation();			
		if(ope != null){
			org.naregi.pse.acs.aaf.stubs.OperationType gt4Ope = 
						org.naregi.pse.acs.aaf.stubs.OperationType.fromValue(ope);
			gt4DiffContent.setOperation(gt4Ope);					
		}		
		
		return gt4DiffContent;
	}

	public String gt4AADToString(org.naregi.pse.acs.aaf.stubs.AADType gt4AAD) throws SerializationException{
		
		// seriarize
		String strGt4AAD = ObjectSerializer.toString(gt4AAD, ACSQNames.AA_RP_AAD);
		
		return strGt4AAD;
	}

	/**
	 * @param gt4AAD
	 * @return
	 * @throws SerializationException
	 */
	public String gt4AADToStringForXPath
				(org.naregi.pse.acs.aaf.stubs.AADType gt4AAD) 
								throws SerializationException, JAXBException{
		
		// seriarize
		String strGt4AAD = ObjectSerializer.toString(gt4AAD, ACSQNames.AA_RP_AAD);
		
		// to jaxb
		org.naregi.pse.acs.service.impl.jaxb.AAD jaxbAAD = StringAADToJaxbAAD(strGt4AAD);
			
		// change type QName
		changeContents(jaxbAAD, gt4AAD);
		
		// jaxbToString
		return jaxbToString(jaxbAAD);
	}

	private String jaxbToString(org.naregi.pse.acs.service.impl.jaxb.AAD aad) throws JAXBException{
		
		// get context
		JAXBContext jaxbContext = JAXBContext
					.newInstance("org.naregi.pse.acs.service.impl.jaxb");
		
		// create Marshaller
		Marshaller marshaller = jaxbContext.createMarshaller();

		// set property for formatted output
		marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
		
		ByteArrayOutputStream bout = new ByteArrayOutputStream();

		// Marshal
		marshaller.marshal(aad, bout);

		String str = bout.toString();
		
		log.info("return \n " + str);
		return str;
	}

	private void changeContents(org.naregi.pse.acs.service.impl.jaxb.AAD jaxbAAD, 
									org.naregi.pse.acs.aaf.stubs.AADType gt4AAD) throws JAXBException{
		
		// get gt4 ContentsType
		org.naregi.pse.acs.aaf.stubs.ContentType[] gt4con = 
				gt4AAD.getContents().getContent();
		
		// create jaxb Contents
		ObjectFactory objFactory = new ObjectFactory();
		
		org.naregi.pse.acs.service.impl.jaxb.ContentsType jaxbCon = 
			objFactory.createContentsType();

		// delete 
		jaxbAAD.setContents(null);
		
		// add Content
		for (int i = 0; i < gt4con.length; i++) {
			jaxbCon.getContent().add(
				createContent(
					objFactory,
					gt4con[i].getPathname(),
					gt4con[i].getType()));
		}

		// set
		jaxbAAD.setContents(jaxbCon);			
	}

	private org.naregi.pse.acs.service.impl.jaxb.ContentType createContent(
		ObjectFactory objFactory,
		String relativePath,
		QName type)
		throws JAXBException {

		org.naregi.pse.acs.service.impl.jaxb.ContentType content = 
				objFactory.createContentType();
		if(type != null){
			content.setType(new QName(type.getNamespaceURI(),
									   type.getLocalPart(),
									   type.getPrefix()));
		}
		content.setPathname(relativePath);

		return content;
	}

	private org.naregi.pse.acs.service.impl.jaxb.AAD StringAADToJaxbAAD(String _aad) throws JAXBException{
		
		// get context
		JAXBContext jaxbContext = JAXBContext
					.newInstance("org.naregi.pse.acs.service.impl.jaxb");

		// create Unmarshaller
		Unmarshaller unmarshaller = jaxbContext.createUnmarshaller();

		// set for validation enabled unmarshall
		//unmarshaller.setValidating(true);

		ByteArrayInputStream bis = new ByteArrayInputStream(_aad.getBytes());
		
		// unmarshal
		org.naregi.pse.acs.service.impl.jaxb.AAD aad = 
			(org.naregi.pse.acs.service.impl.jaxb.AAD) unmarshaller
						.unmarshal(new InputSource(bis));
		
		return aad;
	}

	public String gt4DiffAADToString(org.naregi.pse.acs.aaf.stubs.DiffAADType gt4DiffAAD) throws SerializationException{
		
		// seriarize
		String strGt4DiffAAD = ObjectSerializer.toString(gt4DiffAAD, ACSQNames.AA_RP_DIFF_AAD);
		log.info("ret = " + strGt4DiffAAD);
		
		return strGt4DiffAAD;
	}

	/**
	 * @return
	 */
	public org.naregi.pse.acs.aaf.stubs.AADType getGt4AAD() {
		
		return this.gt4AAD;
	}

	/**
	 * @return
	 */
	public org.naregi.pse.acs.aaf.stubs.DiffAADType getGt4DiffAAD() {
		
		return gt4DiffAAD;
	}

}