/*
 * Copyright 2004-2006 National Institute of Informatics in Japan, 
 * All rights reserved.
 *
 * This file or a portion of this file is licensed under the terms of
 * the NAREGI Public License, found at http://www.naregi.org/download/index.html.
 * If you redistribute this file, with or without modifications, you must 
 * include this notice in the file.
 *
 * [History]
 * Date         Name                  Contents
 * ------------------------------------------------------------------------
 * 2006/06/07   Fujitsu)Yositake      First Version 
 */

package org.naregi.pse.acs.service.impl.functionbean;

import java.io.ByteArrayInputStream;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.namespace.QName;

import org.apache.axis.types.URI.MalformedURIException;
import org.apache.axis.utils.ByteArrayOutputStream;
import org.apache.log4j.Logger;
import org.naregi.pse.acs.service.impl.jaxb.ObjectFactory;
import org.naregi.pse.acs.service.impl.namespace.AADNamespacePrefixMapperImpl;
import org.xml.sax.InputSource;

public class AADTransformerForJaxb {

	private static Logger log = Logger.getLogger(AADTransformerForJaxb.class.getName());

	private org.naregi.pse.acs.service.impl.jaxb.AAD jaxbAAD;
	private org.naregi.pse.acs.service.impl.jaxb.DifferentialAAD jaxbDiffAAD;
	private ObjectFactory objFactory;
	
	public AADTransformerForJaxb(){
	
	}
		
	public void gt4AAD2JaxbAAD(org.naregi.pse.acs.aaf.stubs.AADType _gt4AAD) throws Exception{

		// get context
		JAXBContext jaxbContext = JAXBContext
					.newInstance("org.naregi.pse.acs.service.impl.jaxb");
		
		// create an ObjectFactory instance.
		this.objFactory = new ObjectFactory();

		// create AAD
		this.jaxbAAD = objFactory.createAAD();

		// AAID
		org.naregi.pse.acs.service.impl.jaxb.AAIDType jaxbAAID = copyAAID(_gt4AAD.getAAID());
		this.jaxbAAD.setAAID(jaxbAAID);
		
		// Author
		org.naregi.pse.acs.service.impl.jaxb.AuthorType jaxbAuthor = copyAuthor(_gt4AAD.getAuthor());
		this.jaxbAAD.setAuthor(jaxbAuthor);
		
		// Descriptions
		if(_gt4AAD.getDescriptions() != null){
			org.naregi.pse.acs.service.impl.jaxb.DescriptionsType jaxbDescs = copyDescriptions(_gt4AAD.getDescriptions());
			this.jaxbAAD.setDescriptions(jaxbDescs);			
		}
		
		// Contents
		org.naregi.pse.acs.service.impl.jaxb.ContentsType jaxbContents = copyContents(_gt4AAD.getContents());
		this.jaxbAAD.setContents(jaxbContents);
	}

	public void gt4DiffAAD2JaxbDiffAAD(org.naregi.pse.acs.aaf.stubs.DiffAADType _gt4DiffAAD) throws Exception{

		// get context
		JAXBContext jaxbContext = JAXBContext
					.newInstance("org.naregi.pse.acs.service.impl.jaxb");
		
		// create an ObjectFactory instance.
		this.objFactory = new ObjectFactory();

		// create DiffAAD
		this.jaxbDiffAAD = objFactory.createDifferentialAAD();
		
		// DiffAAID
		org.naregi.pse.acs.service.impl.jaxb.DiffAAIDType jaxbDiffAAID = copyDiffAAID(_gt4DiffAAD.getAAID());
		this.jaxbDiffAAD.setAAID(jaxbDiffAAID);
		
		// Author
		if(_gt4DiffAAD.getAuthor() != null){
			org.naregi.pse.acs.service.impl.jaxb.AuthorType jaxbAuthor = copyAuthor(_gt4DiffAAD.getAuthor());
			this.jaxbDiffAAD.setAuthor(jaxbAuthor);
		}	
		
		// Desscriptions
		if(_gt4DiffAAD.getDescriptions() != null){
			org.naregi.pse.acs.service.impl.jaxb.DescriptionsType jaxbDescs = copyDescriptions(_gt4DiffAAD.getDescriptions());
			this.jaxbDiffAAD.setDescriptions(jaxbDescs);			
		}
		
		// DiffContents
		org.naregi.pse.acs.service.impl.jaxb.DiffContentsType jaxbDiffContents = copyDiffContents(_gt4DiffAAD.getContents());
		this.jaxbDiffAAD.setContents(jaxbDiffContents);			
	}

	private org.naregi.pse.acs.service.impl.jaxb.AAIDType copyAAID(org.naregi.pse.acs.aaf.stubs.AAIDType _gt4AAID) throws MalformedURIException, JAXBException{
		
		// create AAIDType
		org.naregi.pse.acs.service.impl.jaxb.AAIDType jaxbAAID = this.objFactory.createAAIDType();
		
		// name
		jaxbAAID.setName(_gt4AAID.getName().toString());
		
		// version
		jaxbAAID.setVersion(_gt4AAID.getVersion());
		
		return jaxbAAID;
	}

	private org.naregi.pse.acs.service.impl.jaxb.DiffAAIDType copyDiffAAID(org.naregi.pse.acs.aaf.stubs.DiffAAIDType _gt4DiffAAID) throws MalformedURIException, JAXBException{
		
		// create DiffAAIDType
		org.naregi.pse.acs.service.impl.jaxb.DiffAAIDType jaxbDiffAAID = this.objFactory.createDiffAAIDType();
		
		// name
		jaxbDiffAAID.setName(_gt4DiffAAID.getName().toString());
		
		// version
		jaxbDiffAAID.setVersion(_gt4DiffAAID.getVersion());
		
		// base version
		jaxbDiffAAID.setBaseVersion(_gt4DiffAAID.getBaseVersion());
		
		return jaxbDiffAAID;
	}

	private org.naregi.pse.acs.service.impl.jaxb.AuthorType copyAuthor(org.naregi.pse.acs.aaf.stubs.AuthorType _gt4Author) throws JAXBException{
		
		// create AuthorType
		org.naregi.pse.acs.service.impl.jaxb.AuthorType jaxbAuthor = this.objFactory.createAuthorType();
		
		// name
		jaxbAuthor.setName(_gt4Author.getName());
		
		// description
		org.naregi.pse.acs.aaf.stubs.DescriptionType[] list = _gt4Author.getDescription();
		
		if(list != null){
				
			for(int i = 0; i < list.length; i++){
			
				// create DescriptionType
				org.naregi.pse.acs.service.impl.jaxb.DescriptionType authDesc = objFactory.createDescriptionType();
			
				// get gt4 value
				String value = list[i].get_value();
				
				// set to jaxb
				authDesc.setValue(value);
				jaxbAuthor.getDescription().add(i, authDesc);
			}
		}
		
		// location
		org.naregi.pse.acs.aaf.stubs.AuthorTypeLocation gt4Loc = _gt4Author.getLocation();
		
		if(gt4Loc != null){
			
			org.naregi.pse.acs.service.impl.jaxb.AuthorType.LocationType jaxbLoc = 
					this.objFactory.createAuthorTypeLocationType();

			if(gt4Loc.getCountry() != null){
				
				// country
				jaxbLoc.setCountry(gt4Loc.getCountry());				
			}

			if(gt4Loc.getAddress() != null){

				// address
				jaxbLoc.setAddress(gt4Loc.getAddress());				
			}
			
			jaxbAuthor.setLocation(jaxbLoc);
		}
		return jaxbAuthor;
	}

	private org.naregi.pse.acs.service.impl.jaxb.DescriptionsType copyDescriptions(org.naregi.pse.acs.aaf.stubs.DescriptionsType _gt4Descriptions) throws JAXBException{
		
		// create Descriptions
		org.naregi.pse.acs.service.impl.jaxb.DescriptionsType jaxbDescs = this.objFactory.createDescriptionsType();
		
		// description
		org.naregi.pse.acs.aaf.stubs.DescriptionType[] list = _gt4Descriptions.getDescription();
		
		if(list != null){		
			
			for(int i = 0; i < list.length; i++){
			
				// create DescriptionType
				org.naregi.pse.acs.service.impl.jaxb.DescriptionType jaxbDesc = this.objFactory.createDescriptionType();
			
				// get gt4 value
				String value = list[i].get_value();
			
				// set to jaxb
				jaxbDesc.setValue(value);
				jaxbDescs.getDescription().add(i, jaxbDesc);
			}			
		}
		return jaxbDescs;
	}

	private org.naregi.pse.acs.service.impl.jaxb.ContentsType copyContents(org.naregi.pse.acs.aaf.stubs.ContentsType _gt4Contents) throws JAXBException{
		
		// create ContentsType
		org.naregi.pse.acs.service.impl.jaxb.ContentsType jaxbContents = this.objFactory.createContentsType();
		
		// contents
		org.naregi.pse.acs.aaf.stubs.ContentType[] list = _gt4Contents.getContent();
		if(list != null){
			
			for(int i = 0; i < list.length; i++){
			
				org.naregi.pse.acs.service.impl.jaxb.ContentType jaxbContent = 
								copyContent(list[i]);

				jaxbContents.getContent().add(i, jaxbContent);
			}			
		}
		return jaxbContents;
	}

	private org.naregi.pse.acs.service.impl.jaxb.ContentType copyContent(org.naregi.pse.acs.aaf.stubs.ContentType gt4Content) throws JAXBException{

		// create ContentType
		org.naregi.pse.acs.service.impl.jaxb.ContentType jaxbContent = this.objFactory.createContentType();
			
		// get gt4 pathName
		String pathName = gt4Content.getPathname();
			
		// get gt4 type
		QName type = gt4Content.getType();
			
		// set to jaxb
		if(type != null){
			jaxbContent.setType(type);
		}
		
		jaxbContent.setPathname(pathName);
		
		return jaxbContent;
	}

	private org.naregi.pse.acs.service.impl.jaxb.DiffContentsType copyDiffContents(org.naregi.pse.acs.aaf.stubs.DiffContentsType _gt4DiffContents) throws JAXBException{
		
		// create ContentsType
		org.naregi.pse.acs.service.impl.jaxb.DiffContentsType jaxbDiffContents = this.objFactory.createDiffContentsType();
		
		// contents
		org.naregi.pse.acs.aaf.stubs.DiffContentType[] list = _gt4DiffContents.getContent();
		if(list != null){
			
			for(int i = 0; i < list.length; i++){

				// copy DiffContentType
				org.naregi.pse.acs.service.impl.jaxb.DiffContentType jaxbDiffContent = 
					copyDiffContent(list[i]);

				// set to DiffContents
				jaxbDiffContents.getContent().add(i, jaxbDiffContent);
			}			
		}
		return jaxbDiffContents;
	}

	private org.naregi.pse.acs.service.impl.jaxb.DiffContentType copyDiffContent(org.naregi.pse.acs.aaf.stubs.DiffContentType gt4DiffContent) throws JAXBException{

		// create DiffContentType
		org.naregi.pse.acs.service.impl.jaxb.DiffContentType jaxbDiffContent = this.objFactory.createDiffContentType();
			
		// get gt4 pathName
		String pathName = gt4DiffContent.getPathname();
			
		// set to jaxb
		jaxbDiffContent.setPathname(pathName);

		// get jaxb type
		QName type = gt4DiffContent.getType();
			
		// set to gt4
		if(type != null){
			jaxbDiffContent.setType(type);
		}
		
		// get gt4 operation
		String ope = gt4DiffContent.getOperation().getValue();			
		if(ope != null){
			jaxbDiffContent.setOperation(ope);					
		}		
		
		return jaxbDiffContent;
	}


	public String getJaxbAADAsString() throws JAXBException{
		
		// get context
		JAXBContext jaxbContext = JAXBContext
					.newInstance("org.naregi.pse.acs.service.impl.jaxb");
		
		// create Marshaller
		Marshaller marshaller = jaxbContext.createMarshaller();

		// set property for namespace prefix
		marshaller.setProperty("com.sun.xml.bind.namespacePrefixMapper", new AADNamespacePrefixMapperImpl());
		
		// set property for formatted output
		marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
		
		ByteArrayOutputStream bout = new ByteArrayOutputStream();

		// Marshal
		marshaller.marshal(this.jaxbAAD, bout);

		String str = bout.toString();
		
		log.info("return \n " + str);
		return str;
	}

	public String getJaxbDiffAADAsString() throws JAXBException{
		
		// get context
		JAXBContext jaxbContext = JAXBContext
					.newInstance("org.naregi.pse.acs.service.impl.jaxb");
		
		// create Marshaller
		Marshaller marshaller = jaxbContext.createMarshaller();

		// set property for namespace prefix
		marshaller.setProperty("com.sun.xml.bind.namespacePrefixMapper", new AADNamespacePrefixMapperImpl());

		// set property for formatted output
		marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
		
		ByteArrayOutputStream bout = new ByteArrayOutputStream();

		// Marshal
		marshaller.marshal(this.jaxbDiffAAD, bout);

		String str = bout.toString();
		
		log.info("return \n " + str);
		return str;
	}


	/**
	 * @return
	 */
	public org.naregi.pse.acs.service.impl.jaxb.AAD getJaxbAAD() {
		
		return this.jaxbAAD;
	}

	/**
	 * @return
	 */
	public org.naregi.pse.acs.service.impl.jaxb.DifferentialAAD getJaxbDiffAAD() {
		
		return this.jaxbDiffAAD;
	}
	
	
	private org.naregi.pse.acs.service.impl.jaxb.AAD StringAADToJaxbAAD(String _aad) throws JAXBException{
		
		// get context
		JAXBContext jaxbContext = JAXBContext
					.newInstance("org.naregi.pse.acs.service.impl.jaxb");

		// create Unmarshaller
		Unmarshaller unmarshaller = jaxbContext.createUnmarshaller();

		// set for validation enabled unmarshall
		unmarshaller.setValidating(true);

		ByteArrayInputStream bis = new ByteArrayInputStream(_aad.getBytes());
		
		// unmarshal
		org.naregi.pse.acs.service.impl.jaxb.AAD aad = 
			(org.naregi.pse.acs.service.impl.jaxb.AAD) unmarshaller
						.unmarshal(new InputSource(bis));
		
		return aad;
	}

}