/*
 * Copyright 2004-2006 National Institute of Informatics in Japan, 
 * All rights reserved.
 *
 * This file or a portion of this file is licensed under the terms of
 * the NAREGI Public License, found at http://www.naregi.org/download/index.html.
 * If you redistribute this file, with or without modifications, you must 
 * include this notice in the file.
 *
 * [History]
 * Date         Name                  Contents
 * ------------------------------------------------------------------------
 * 2006/06/08   Fujitsu)Yositake      First Version 
 */

package org.naregi.pse.acs.service.impl.functionbean;

import java.util.ArrayList;

import javax.xml.namespace.QName;

import org.apache.axis.types.URI;
import org.apache.axis.types.URI.MalformedURIException;
import org.apache.log4j.Logger;
import org.naregi.pse.acs.aaf.stubs.AADType;
import org.naregi.pse.acs.aaf.stubs.AAIDType;
import org.naregi.pse.acs.aaf.stubs.AuthorType;
import org.naregi.pse.acs.aaf.stubs.AuthorTypeLocation;
import org.naregi.pse.acs.aaf.stubs.ContentType;
import org.naregi.pse.acs.aaf.stubs.ContentsType;
import org.naregi.pse.acs.aaf.stubs.DescriptionType;
import org.naregi.pse.acs.aaf.stubs.DescriptionsType;
import org.naregi.pse.acs.aaf.stubs.DiffAADType;
import org.naregi.pse.acs.aaf.stubs.DiffAAIDType;
import org.naregi.pse.acs.aaf.stubs.DiffContentType;
import org.naregi.pse.acs.aaf.stubs.DiffContentsType;
import org.naregi.pse.acs.aaf.stubs.OperationType;

public class AADUpdater {

	private static Logger log = Logger.getLogger(AADUpdater.class.getName());
	
	private AADType baseAAD;
	private DiffAADType diffAAD;
	private AADType newAAD;
	
	public AADUpdater(AADType _baseAAD, DiffAADType _diffAAD){
		
		this.baseAAD = _baseAAD;
		this.diffAAD = _diffAAD;
		this.newAAD = new AADType();
	}
	
	public AADType updateAAD() throws MalformedURIException{
		
		setAAID();
		setAuthor();
		setDescriptions();
		setContents();
		
		return newAAD;
	}
	
	private void setAAID() throws MalformedURIException{
		
		// create AAIDType
		AAIDType newAAID = new AAIDType();
		
		// get data from DiffAAD
		DiffAAIDType diffAAID = this.diffAAD.getAAID();
		
		// set name (required)
		newAAID.setName(new URI(diffAAID.getName()));
		
		// set version (required)
		newAAID.setVersion(diffAAID.getVersion());
		
		this.newAAD.setAAID(newAAID);
	}
	
	private void setAuthor(){
				
		// get data from Diff AAD
		AuthorType diffAuthor = this.diffAAD.getAuthor();

		// get data from Base AAD
		AuthorType baseAuthor = this.baseAAD.getAuthor();

		// which Author should be copied
		if(diffAuthor == null || diffAuthor.getName() == null || diffAuthor.getName().trim().equals("")){
			setAuthorDetail(baseAuthor);
		}else{
			setAuthorDetail(diffAuthor);
		}
	}

	private void setAuthorDetail(AuthorType _author){

		// create AuthorType
		AuthorType newAuthor = new AuthorType();

		// name
		newAuthor.setName(_author.getName());
	
		// description
		DescriptionType[] _list = _author.getDescription();
	
		if(_list != null){
		
			// create container
			DescriptionType[] descArray = new DescriptionType[_list.length];
			newAuthor.setDescription(descArray);
			
			for(int i = 0; i < _list.length; i++){
		
				// create DescriptionType
				DescriptionType desc = new DescriptionType();
		
				// get value
				String value = _list[i].get_value();
			
				// set to gt4
				desc.set_value(value);
				newAuthor.setDescription(i, desc);
			}
		}
	
		// location
		AuthorTypeLocation _loc = _author.getLocation();
	
		if(_loc != null){
			AuthorTypeLocation newLoc = new AuthorTypeLocation();

			if(_loc.getCountry() != null){
			
				// country
				newLoc.setCountry(_loc.getCountry());				
			}
			
			if(_loc.getAddress() != null){

				// address
				newLoc.setAddress(_loc.getAddress());				
			}
			newAuthor.setLocation(newLoc);
		}
		
		this.newAAD.setAuthor(newAuthor);
	}

	private void setDescriptions(){
		
		// get DiffDesctions
		DescriptionsType diffDescs = this.diffAAD.getDescriptions();
		
		// which Descriptions should be copied
		if(diffDescs != null){
			
			DescriptionType[] desc = diffDescs.getDescription();
			if(desc == null || desc.length < 1 || desc[0] == null 
					|| desc[0].get_value() == null || desc[0].get_value().trim().equals("")){
				setDescriptionsDetail(this.baseAAD.getDescriptions());
			
			}else{
				setDescriptionsDetail(this.diffAAD.getDescriptions());
			}
		}else{
			setDescriptionsDetail(this.baseAAD.getDescriptions());
		}
	}
	
	private void setDescriptionsDetail(DescriptionsType _descs){
		
		if(_descs != null){
			
			// create Descriptions
			DescriptionsType newDescs = new DescriptionsType();
			DescriptionType[] _desc = _descs.getDescription();
	
			if(_desc != null){
			
				// create container
				DescriptionType[] descArray = 
							new DescriptionType[_desc.length];
				newDescs.setDescription(descArray);

				for(int i = 0; i < _desc.length; i++){
				
					// create DescriptionType
					DescriptionType desc = new DescriptionType();
				
					// get value
					String value = _desc[i].get_value();
				
					// set to new
					desc.set_value(value);
					newDescs.setDescription(i, desc);
				}
				
				// set newDescs
				this.newAAD.setDescriptions(newDescs);
				
			}else{
				// set newDescs
				this.newAAD.setDescriptions(newDescs);
			}	
		}
	}

	private void setContents(){
		
		// contents
		ContentsType baseContents = this.baseAAD.getContents();
		DiffContentsType diffContents = this.diffAAD.getContents();
	
		// copy baseContents
		ContentsType newContents = copyContents(baseContents);
		
		// change to ArrayList
		ArrayList list = toArrayList(newContents.getContent());
		
		// analyze DiffContents
		analyzeDiffContents(list, diffContents.getContent());
		
		// toArray
		ContentType[] revised = (ContentType[])list.toArray(new ContentType[0]);
		
		// set
		newContents.setContent(revised);
		this.newAAD.setContents(newContents);
	}
	
	private void analyzeDiffContents(ArrayList list, DiffContentType[] diff){
		
		OperationType op = null;
		
		for(int i = 0; i < diff.length; i++){
			
			op = diff[i].getOperation();
			
			if(OperationType._add.equals(op.getValue())){
				list.add(copyContent(diff[i]));
				
			}else if(OperationType._delete.equals(op.getValue())){
				int index = findIndex(list, diff[i].getPathname());
				list.remove(index);
				
			}else if(OperationType._replace.equals(op.getValue())){
				
				// remove
				int index = findIndex(list, diff[i].getPathname());
				list.remove(index);

				// add
				list.add(copyContent(diff[i]));				
			}
		}
	}
	
	private int findIndex(ArrayList list, String pathName){
		
		int index = -1;
		for(int i = 0; i < list.size(); i++){
			if(((ContentType)list.get(i)).getPathname().equals(pathName)){
				
				index = i;
				break;
			}
		}
		return index;
	}
	
	private ArrayList toArrayList(ContentType[] array){
		
		ArrayList list = new ArrayList();
		
		for(int i = 0; i < array.length; i++){
			list.add(array[i]);	
		}
		return list;
	}
	
	private ContentsType copyContents(ContentsType fromContents){
		
		ContentsType newContents = new ContentsType();
		
		ContentType[] cons = fromContents.getContent();
		
		if(cons != null){
			
			// create container
			ContentType[] contentArray = 
					new ContentType[cons.length];
			newContents.setContent(contentArray);
			
			for(int i = 0; i < cons.length; i++){
			
				ContentType con = 
					copyContent(cons[i]);

				newContents.setContent(i, con);
			}			
		}
		
		return newContents;
	}

	private ContentType copyContent(ContentType _con){

		// create ContentType
		ContentType con = new ContentType();
			
		// get pathName
		String pathName = _con.getPathname();
			
		// get type
		QName type = _con.getType();
			
		// set
		if(type != null){
			con.setType(type);
		}
		con.setPathname(pathName);
		
		return con;
	}		
}