/*
 * Copyright 2004-2006 National Institute of Informatics in Japan, 
 * All rights reserved.
 *
 * This file or a portion of this file is licensed under the terms of
 * the NAREGI Public License, found at http://www.naregi.org/download/index.html.
 * If you redistribute this file, with or without modifications, you must 
 * include this notice in the file.
 *
 * [History]
 * Date         Name                  Contents
 * ------------------------------------------------------------------------
 * 2006/06/09   Fujitsu)Yositake      First Version 
 */

package org.naregi.pse.acs.service.impl.functionbean;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import org.naregi.pse.acs.aaf.stubs.AADType;
import org.naregi.pse.acs.aaf.stubs.OperationType;
import org.naregi.pse.acs.service.impl.constants.ACSConstants;
import org.naregi.pse.acs.service.impl.jaxb.DiffAADType;
import org.naregi.pse.acs.service.impl.jaxb.DiffAAIDType;

/**
 * The class of AcsFileManager.
 * 
 */
public class AcsFileManager extends ZipFileManager {

	private static Logger log = Logger.getLogger(AcsFileManager.class.getName());

	/**
	 * get AARPAAIDType, return AAID Location
	 * 
	 * @param aaID
	 * @return directory identified by AAID
	 */
	public File getAAStoredDirFromAAID(File repoBaseDir, org.naregi.pse.acs.service.impl.jaxb.AAIDType aaID) throws Exception {

		log.info("getRepoLocationFromAAID START");

		String name = aaID.getName().toString();
		String version = aaID.getVersion();
		File aaStoredDir = getAAStoredDirFromAAID(repoBaseDir, name, version);

		log.info("getRepoLocationFromAAID END AADIR = " + aaStoredDir.getPath());
		return aaStoredDir;
	}

	/**
	 * get AARPAAIDType, return AAID Location
	 * 
	 * @param aaID
	 * @return directory identified by AAID
	 */
	public File getAAStoredDirFromAAID(File repoBaseDir, org.naregi.pse.acs.aaf.stubs.AAIDType aaID) throws Exception {

		log.info("getRepoLocationFromAAID START");

		String name = aaID.getName().toString();
		String version = aaID.getVersion();
		File aaStoredDir = getAAStoredDirFromAAID(repoBaseDir, name, version);

		log.info("getRepoLocationFromAAID END AADIR = " + aaStoredDir.getPath());
		return aaStoredDir;
	}

	private File getAAStoredDirFromAAID(File repoBaseDir, String name, String version) throws Exception {

		String revisedName = changeNotAllowedChar(name);
		String revisedVersion = changeNotAllowedChar(version);
		
		String strAAStoredDir =
			(repoBaseDir.getPath() + "/" + revisedName + "/" + revisedVersion + "/");

		File aaStoredDir = new File(strAAStoredDir);

		return aaStoredDir;
	}


	public File getBaseAAStoredDirFromDiffAAID(File repoBaseDir, DiffAAIDType diffAAID) throws Exception {

		log.info("getBaseAAStoredDirFromDiffAAID START");

		String name = diffAAID.getName().toString();
		String revisedName = changeNotAllowedChar(name);
		
		String baseVersion = diffAAID.getBaseVersion();
		String revisedBaseVersion = changeNotAllowedChar(baseVersion);
		
		String strBaseAAStoredDir =
			(repoBaseDir.getPath() + "/" + revisedName + "/" + revisedBaseVersion + "/");

		File baseAAStoredDir = new File(strBaseAAStoredDir);

		log.info("getBaseAAStoredDirFromDiffAAID END AADIR = " + baseAAStoredDir.getPath());
		return baseAAStoredDir;
	}

	/**
	 * Allowed charactor is only "a-z", "A-Z", "1-9", "-", "+", "_".
	 * Change other charactor to _
	 * 
	 * @param str
	 * @return
	 */
	private String changeNotAllowedChar(String str){
		
		log.info("changeNotAllowedChar START");
		
		String revised = str.replaceAll("[^a-zA-Z0-9-+_]", "_");
		
		log.info("return = " + revised);
		log.info("changeNotAllowedChar END");
		return revised;
	}

	/**
	 * it move temp file to AAID location.
	 * @param tempFile
	 * @param aaDir
	 */
	public void moveTmpFileToAAStoredDir(
		File tmpFile,
		File aaStoredDir) throws Exception{

		log.info("moveTmpFileToAAStoredDir START");

		// file name of tmpFile
		String strTmpFileName = tmpFile.getName();

		// guaranteed aaStoredDir not exist because of AADCheck 
		if(aaStoredDir.mkdirs() == false){
			throw new Exception("Failed to make dir. " + aaStoredDir.getPath());
		}

		// move
		File dest = new File(aaStoredDir.getPath() + "/" + strTmpFileName);
		
		if(tmpFile.renameTo(dest) == false){
			throw new Exception("Failed to move from " + 
									tmpFile.getPath() + " to " + dest.getPath());
		}

		log.info("moveTmpFileToAAStoredDir END");
	}

	/**
	 * 
	 * @param baseStoredFile
	 * @param baseTmpDir
	 * @throws Exception
	 */
	public void copyBaseFileToTmpDir(File baseStoredFile, File baseTmpDir)
		throws Exception {

		log.info("moveBaseFileToTmpDir START");
		log.info("baseStoredFile = " + baseStoredFile.getPath());
		log.info("baseTmpDir = " + baseTmpDir.getPath());
		
		boolean check = false;

		// file name of tmpFile
		String baseFileName = baseStoredFile.getName();
			
		// guaranteed baseTmpDir not exist because parentDir created for only this update request.
		// In addition the name is base + timestamp
		// However, this existence check is for getArchive and getContents
		if(baseTmpDir.exists() == false){

			if(baseTmpDir.mkdirs() == false){
				throw new Exception("Failed to make dir. " + baseTmpDir.getPath());
			}		
		}

		// copy
		File destFile = new File(baseTmpDir.getPath() + "/" + baseFileName);

		if(destFile.createNewFile() == false){
			throw new Exception("Failed to create file. " + destFile.getPath());
		}

		copyFile(baseStoredFile, destFile);
		
		log.info("moveBaseFileToTmpDir END");
	}

	/**
	 * @param diffAAD - DifferentialAAD
	 * @param baseDir - dir base files are stored
	 * @param tempDir - dir differential files are stored
	 * @param relativeFileList - file list of differential files
	 * @throws Exception
	 */
	public ArrayList fileOperationForUpdate(
		AADType gt4BaseAAD,
		DiffAADType jaxbDiffAAD,
		File baseDir,
		File tempDir) throws Exception {

		log.info("fileOperationForUpdate START");
		
		// file list after update
		ArrayList fileList = new ArrayList();

		// get base file list
		getBaseFileList(fileList, gt4BaseAAD);

		// get list for Differential
		List list = jaxbDiffAAD.getContents().getContent();

		// loop for content.
		for(int i = 0; i < list.size(); i++){
			
			// get ContentType.
			org.naregi.pse.acs.service.impl.jaxb.DiffContentType con = 
				(org.naregi.pse.acs.service.impl.jaxb.DiffContentType) list.get(i);

			log.info("Content path = " + con.getPathname());
			log.info("Content operation = " + con.getOperation());

			if (OperationType._add.equalsIgnoreCase(con.getOperation())){

				// add
				add(baseDir, tempDir, con.getPathname());
				
				// update file list
				fileList.add(con.getPathname());

			}else if (OperationType._replace.equalsIgnoreCase(con.getOperation())){

				// delete 
				if(new File(baseDir.getPath() + "/" + con.getPathname()).exists() == false){
					throw new Exception("Request was Update/replace, but " + 
									con.getPathname() + " not found in Base.");
				}
				
				delete(baseDir, con.getPathname());

				// add
				add(baseDir, tempDir, con.getPathname());

			}else if (OperationType._delete.equalsIgnoreCase(con.getOperation())){

				// delete
				if(new File(baseDir.getPath() + "/" + con.getPathname()).exists() == false){
					throw new Exception("Request was Update/delete, but " + 
									con.getPathname() + " not found in Base.");
				}
				
				delete(baseDir, con.getPathname());
				
				// update file list
				int index = findIndex(fileList, con.getPathname());
				fileList.remove(index);
			}
		}
		
		log.info("fileOperationForUpdate END");
		
		return fileList;
	}

	private int findIndex(ArrayList fileList, String pathName){
		
		int index = -1;
		for(int i = 0; i < fileList.size(); i++){
			
			if(((String)fileList.get(i)).equals(pathName)){
				index = i;
				break;
			}
		}

		return index;
	}

	private void getBaseFileList(ArrayList fileList, AADType gt4BaseAAD){
		
		org.naregi.pse.acs.aaf.stubs.ContentsType contents = gt4BaseAAD.getContents();
		org.naregi.pse.acs.aaf.stubs.ContentType[] conArray = contents.getContent();
		
		for(int i = 0; i < conArray.length; i++){
			
			fileList.add(conArray[i].getPathname());
		}
	}

	/**
	 * If fileName was not found in baseDir, throws exception.
	 * 
	 * @param baseDir
	 * @param fileName
	 * @throws Exception
	 */
	private void delete(File baseDir, String fileName) throws Exception {
		
		String delete = baseDir.getPath() + "/" + fileName;
		File deleteFile = new File(delete);
		if(deleteFile.delete() == false){
			throw new Exception("Failed to delete." + baseDir.getPath() + "/" + fileName);
		}
	}
	
	/**
	 * This add is performed by "move" instead of "copy".
	 * If fileName was not found in tempDir, nothing will be performed.
	 * fileName Being threre is guaranteed. 
	 * 
	 * @param baseDir
	 * @param tempDir
	 * @param fileName
	 */
	private void add(File baseDir, File tmpDir, String fileName) throws Exception {
		
		File fromFile = new File(tmpDir.getPath() + "/" + fileName);
		File toFile = new File(baseDir.getPath() + "/" + fileName);
				
		// make dir if not exists
		File parentDir = toFile.getParentFile();
		if(parentDir.exists() == false){
			if(parentDir.mkdirs() == false){
				throw new Exception("Failed to mkdirs. " + parentDir.getPath());
			}
		}
		
		if(fromFile.renameTo(toFile) == false){
			throw new Exception("Failed to add. " + fileName);
		}
		
		log.info("added to base " + fileName);
	}
	
	public void deleteAAFileAndDir(File aaStoredDir) throws Exception{
		
		log.info("deleteAAFileAndDir START");
		
		// delete AAFile
		if(new File(aaStoredDir.getPath() + "/" + ACSConstants.BUNDLED_FILENAME_ZIP).exists()){
			this.delete(aaStoredDir, ACSConstants.BUNDLED_FILENAME_ZIP);
		}
		
		// delete version dir
		if(aaStoredDir.delete() == false){
			throw new Exception("Failed to delete " + aaStoredDir.getPath());
		}
		
		// delete name dir
		File nameDir = aaStoredDir.getParentFile();
		String[] list = nameDir.list();
		if(list == null || list.length < 1){
			if(nameDir.delete() == false){
				throw new Exception();
			}
		}else{
			for(int i = 0; i < list.length; i++){
				log.info(list[i]);
			}
		}
		
		log.info("deleteAAFileAndDir END");
	}
}