/*
 * Copyright 2004-2006 National Institute of Informatics in Japan, 
 * All rights reserved.
 *
 * This file or a portion of this file is licensed under the terms of
 * the NAREGI Public License, found at http://www.naregi.org/download/index.html.
 * If you redistribute this file, with or without modifications, you must 
 * include this notice in the file.
 *
 * [History]
 * Date         Name                  Contents
 * ------------------------------------------------------------------------
 * 2005/09/12   Fujitsu)Yositake      First Version 
 */

package org.naregi.pse.acs.service.impl.functionbean;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import org.apache.axis.utils.ByteArrayOutputStream;
import org.apache.log4j.Logger;

/**
 * 
 * 
 */
public class FileManager {

	private static Logger log = Logger.getLogger(FileManager.class.getName());
	
	/**
	 * read a file as byte array.
	 * @param file
	 * @return
	 * @throws IOException 
	 */
	public byte[] readFileAsbyte(String file) throws IOException {

		FileInputStream in = null;
		BufferedInputStream bin = null;
		ByteArrayOutputStream out = null;
		try {

			try {
				// read file as byte
				in = new FileInputStream(file);
			} catch (FileNotFoundException e) {
				throw e;
			}

			bin = new BufferedInputStream(in);
			out = new ByteArrayOutputStream();
			byte[] buffer = new byte[1024];
			int size;

			try {
				// write to outout stream by specified buffer size
				while ((size = in.read(buffer)) != -1) {
					out.write(buffer, 0, size);

				}
				out.flush();
			} catch (IOException e) {
				throw e;
			}

			byte[] result = out.toByteArray();
			return (result);

		} finally {
			if (in != null) {
				try {
					in.close();
					out.close();
				} catch (IOException e) {
					throw e;
				}
			}
		}
	}

	/**
	 * it copy file.
	 * @param in
	 * @param out
	 * @throws Exception
	 */
	public void copyFile(File in, File out) throws IOException {

		FileInputStream fis = null;
		FileOutputStream fos = null;
	
		try{
		
			fis = new FileInputStream(in);
			fos = new FileOutputStream(out);
	
			byte[] buf = new byte[1024];
	
			int i = 0;
			while ((i = fis.read(buf)) != -1) {
	
				fos.write(buf, 0, i);
			}
		}catch(IOException e) {
			throw e;
			
		} finally {
			
			if(fis != null){
				fis.close();
			}
			if(fos != null){
				fos.close();
			}
		}
	}

	/**
	 * byte[] data is written into file.
	 * file must exist.
	 * 
	 * @param file 
	 * @param data  
	 */
	public void writeFileFrombyte(File file, byte[] data) throws IOException {

		FileOutputStream out = null;
		try {

			// file must be exist
			out = new FileOutputStream(file);

			// write byte[] data into file
			out.write(data);
			out.flush();
		} catch (FileNotFoundException e) {
			throw e;
		} catch (IOException e) {
			throw e;
		} finally {
			if (out != null) {
				try {
					out.close();
				} catch (IOException e) {
					throw e;
				}
			}
		}
	}
}
