/*
 * Copyright 2004-2006 National Institute of Informatics in Japan, 
 * All rights reserved.
 *
 * This file or a portion of this file is licensed under the terms of
 * the NAREGI Public License, found at http://www.naregi.org/download/index.html.
 * If you redistribute this file, with or without modifications, you must 
 * include this notice in the file.
 *
 * [History]
 * Date         Name                  Contents
 * ------------------------------------------------------------------------
 * 2006/06/27   Fujitsu)Yositake      First Version
 */

package org.naregi.pse.ap.batch.acscleaner;

import java.io.File;
import java.util.Date;

import org.apache.log4j.Logger;
import org.naregi.pse.acs.service.impl.property.ACSSystemInfo;

/**
 * File Manager Class.
 */
public class ACSTempFileDeleteManager {
	
	private static Logger log = Logger.getLogger(ACSTempFileDeleteManager.class.getName());
	
	private static ACSTempFileDeleteManager mySelf = null;
	
	private String objectDirPath = null;
	
	private long period = -1;
	
	private long deletedDirCounter = 0;

	/**
	 *  Constructor
	 */
	private ACSTempFileDeleteManager() throws Exception {
		
		//set up property from property setting file
		objectDirPath = ACSSystemInfo.getInstance().getProperty("ACS_SERVICE_TEMP_BASEDIR");

		period = Long.parseLong(ACSSystemInfo.getInstance().getProperty("TIME_FOR_DELETE"));
		
		log.info("TIME_FOR_DELETE = " + period);
	}

	/**
	 * This is method to get Instance.
	 * 
	 * @return PSEFileManager
	 */
	public static ACSTempFileDeleteManager getInstance() throws Exception {
		if (mySelf == null) {
			mySelf = new ACSTempFileDeleteManager();
		}
		return mySelf;
	}
	
	/**
	 * This is method to delete directories under the pathname setting by property file.
	 * 
	 */
	public void deleteTmpDir(){
		deletedDirCounter = 0;
		long delTimePoint = new Date().getTime() - period;
		
		log.info("deleTimePoint   = " + delTimePoint);
		
		File parentDir = new File(objectDirPath);

		// if ACS_SERVICE_TEMP_BASEDIR exists, then delete.
		if(parentDir.exists()){
			
			File[] subDirs = parentDir.listFiles();
			for (int i = 0; i < subDirs.length; i++) {
				long tmpDirTimstamp =0;
			
				try{
					tmpDirTimstamp = Long.parseLong(subDirs[i].getName());
					
					log.info("tmpDirTimestamp = " + tmpDirTimstamp);
					
				}catch(NumberFormatException ne){
					tmpDirTimstamp = delTimePoint + 1;
				}
	
				if (!(tmpDirTimstamp > delTimePoint)) {
					delDir(subDirs[i]);
					deletedDirCounter++;
				}			
			}
		}
	}
	
	/**
	 * Delete the specified directory and the children if there are.
	 * @param delTimePoint
	 * @param file
	 */
	private void delDir(File file) {
		File[] subFiles = file.listFiles();
		
		if(subFiles == null || subFiles.length<1){
			file.delete();
			log.info(file.getName() + " is deleted");
		}else{
			for (int i = 0; i < subFiles.length; i++) {
				delDir(subFiles[i]);
			}
			file.delete();
			log.info(file.getName() + " is deleted");
		}
	}
}

